/* Generated by Cython 0.23.4 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [
            "/home/femanov/cx/4cx/src/include/cx.h", 
            "/home/femanov/cx/4cx/src/include/cx_common_types.h", 
            "/home/femanov/cx/4cx/src/include/cxscheduler.h", 
            "/home/femanov/cx/4cx/src/include/misc_types.h", 
            "/usr/lib64/python3.3/site-packages/numpy/core/include/numpy/arrayobject.h", 
            "/usr/lib64/python3.3/site-packages/numpy/core/include/numpy/ufuncobject.h", 
            "pycx4/cda.h"
        ], 
        "include_dirs": [
            "/usr/lib64/python3.3/site-packages/numpy/core/include", 
            "/home/femanov/cx/4cx/src/include"
        ], 
        "libraries": [
            "cda", 
            "cx_async", 
            "useful", 
            "misc", 
            "cxscheduler"
        ], 
        "library_dirs": [
            "/home/femanov/cx/4cx/src/lib/cda", 
            "/home/femanov/cx/4cx/src/lib/cxlib", 
            "/home/femanov/cx/4cx/src/lib/useful", 
            "/home/femanov/cx/4cx/src/lib/misc"
        ]
    }
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_23_4"
#include <stddef.h>
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if !defined(CYTHON_USE_PYLONG_INTERNALS) && CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x02070000
#define CYTHON_USE_PYLONG_INTERNALS 1
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
#define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if PY_VERSION_HEX >= 0x030500B1
#define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
#elif CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
typedef struct {
    unaryfunc am_await;
    unaryfunc am_aiter;
    unaryfunc am_anext;
} __Pyx_PyAsyncMethodsStruct;
#define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
#else
#define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pycx4__pycda
#define __PYX_HAVE_API__pycx4__pycda
#include "cxscheduler.h"
#include "misc_types.h"
#include "cx_common_types.h"
#include "cx.h"
#include "cda.h"
#include "string.h"
#include "stdlib.h"
#include "stdio.h"
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 1
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING "ascii"
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "pycx4/cda_signal.pxi",
  "pycx4/pycdabase.pxi",
  "pycx4/pycdauser.pxi",
  "__init__.pxd",
  "pycx4/pycda.pyx",
  "type.pxd",
};

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":725
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":726
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":727
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":728
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":732
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":733
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":734
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":735
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":739
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":740
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":749
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":750
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":751
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":753
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":754
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":755
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":757
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":758
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":760
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":761
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":762
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif


/*--- Type declarations ---*/
struct __pyx_obj_5pycx4_5pycda_cda_signal;
struct __pyx_obj_5pycx4_5pycda_cda_object;
struct __pyx_obj_5pycx4_5pycda_cda_context;
struct __pyx_obj_5pycx4_5pycda_cda_base_chan;
struct __pyx_obj_5pycx4_5pycda_sdchan;
struct __pyx_obj_5pycx4_5pycda_strchan;
struct __pyx_obj_5pycx4_5pycda_schan;
struct __pyx_obj_5pycx4_5pycda_vchan;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":764
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":765
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":766
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":768
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;
struct __pyx_t_5pycx4_5pycda_event;
typedef struct __pyx_t_5pycx4_5pycda_event __pyx_t_5pycx4_5pycda_event;

/* "pycx4/pycdabase.pxi":25
 * 
 * # struct to extend private pointer
 * ctypedef struct event:             # <<<<<<<<<<<<<<
 *     int evmask    # cda eventmask
 *     void *evproc  # pointer to proc function
 */
struct __pyx_t_5pycx4_5pycda_event {
  int evmask;
  void *evproc;
  void *objptr;
  void *userptr;
};

/* "pycx4/cda_signal.pxi":3
 * # simple implementation of Qt-like multiply callback handler
 * # for use with
 * cdef class cda_signal:             # <<<<<<<<<<<<<<
 *     cdef:
 *         void **callbacks
 */
struct __pyx_obj_5pycx4_5pycda_cda_signal {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pycx4_5pycda_cda_signal *__pyx_vtab;
  void **callbacks;
  int cnum;
};


/* "pycx4/pycdabase.pxi":77
 * # classes
 * 
 * cdef class cda_object:             # <<<<<<<<<<<<<<
 *     cdef:
 *         event *events
 */
struct __pyx_obj_5pycx4_5pycda_cda_object {
  PyObject_HEAD
  struct __pyx_vtabstruct_5pycx4_5pycda_cda_object *__pyx_vtab;
  __pyx_t_5pycx4_5pycda_event *events;
  int evnum;
};


/* "pycx4/pycdabase.pxi":139
 * 
 * 
 * cdef class cda_context(cda_object):             # <<<<<<<<<<<<<<
 *     cdef readonly:
 *         cda_context_t cid
 */
struct __pyx_obj_5pycx4_5pycda_cda_context {
  struct __pyx_obj_5pycx4_5pycda_cda_object __pyx_base;
  cda_context_t cid;
  PyObject *defpfx;
  void **chans;
  int channum;
  struct __pyx_obj_5pycx4_5pycda_cda_signal *serverCycle;
};


/* "pycx4/pycdabase.pxi":221
 * 
 * # wrapper-class for low-level functions and channel registration
 * cdef class cda_base_chan(cda_object):             # <<<<<<<<<<<<<<
 *     cdef readonly:
 *         cda_dataref_t ref
 */
struct __pyx_obj_5pycx4_5pycda_cda_base_chan {
  struct __pyx_obj_5pycx4_5pycda_cda_object __pyx_base;
  cda_dataref_t ref;
  PyObject *name;
  int max_nelems;
  cxdtype_t dtype;
  size_t itemsize;
  int64 time;
  int64 prev_time;
  int found;
  int first_cycle;
  void *context;
  int registered;
  struct __pyx_obj_5pycx4_5pycda_cda_signal *valueMeasured;
  struct __pyx_obj_5pycx4_5pycda_cda_signal *valueChanged;
};


/* "pycx4/pycdauser.pxi":6
 * 
 * # scalar double channel
 * cdef class sdchan(cda_base_chan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         # all general properties defined in base classes
 */
struct __pyx_obj_5pycx4_5pycda_sdchan {
  struct __pyx_obj_5pycx4_5pycda_cda_base_chan __pyx_base;
  double val;
  double prev_val;
  double tolerance;
};


/* "pycx4/pycdauser.pxi":30
 * 
 * 
 * cdef class strchan(cda_base_chan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly bytes val
 */
struct __pyx_obj_5pycx4_5pycda_strchan {
  struct __pyx_obj_5pycx4_5pycda_cda_base_chan __pyx_base;
  PyObject *val;
  char *cval;
  int allocated;
};


/* "pycx4/pycdauser.pxi":123
 * 
 * # general channel (any cx type)
 * cdef class schan(cda_base_chan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly object val, prev_val
 */
struct __pyx_obj_5pycx4_5pycda_schan {
  struct __pyx_obj_5pycx4_5pycda_cda_base_chan __pyx_base;
  PyObject *val;
  PyObject *prev_val;
};


/* "pycx4/pycdauser.pxi":147
 * 
 * # vector-data channel class
 * cdef class vchan(cda_base_chan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly np.ndarray val
 */
struct __pyx_obj_5pycx4_5pycda_vchan {
  struct __pyx_obj_5pycx4_5pycda_cda_base_chan __pyx_base;
  PyArrayObject *val;
  PyObject *npdtype;
};



/* "pycx4/cda_signal.pxi":3
 * # simple implementation of Qt-like multiply callback handler
 * # for use with
 * cdef class cda_signal:             # <<<<<<<<<<<<<<
 *     cdef:
 *         void **callbacks
 */

struct __pyx_vtabstruct_5pycx4_5pycda_cda_signal {
  PyObject *(*connect)(struct __pyx_obj_5pycx4_5pycda_cda_signal *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*disconnect)(struct __pyx_obj_5pycx4_5pycda_cda_signal *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*emit)(struct __pyx_obj_5pycx4_5pycda_cda_signal *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_cda_signal *__pyx_vtabptr_5pycx4_5pycda_cda_signal;


/* "pycx4/pycdabase.pxi":77
 * # classes
 * 
 * cdef class cda_object:             # <<<<<<<<<<<<<<
 *     cdef:
 *         event *events
 */

struct __pyx_vtabstruct_5pycx4_5pycda_cda_object {
  int (*add_event)(struct __pyx_obj_5pycx4_5pycda_cda_object *, int, void *, void *, void *);
  int (*del_event)(struct __pyx_obj_5pycx4_5pycda_cda_object *, int, void *, void *, void *);
  int (*search_event)(struct __pyx_obj_5pycx4_5pycda_cda_object *, __pyx_t_5pycx4_5pycda_event *);
  void (*register_event)(struct __pyx_obj_5pycx4_5pycda_cda_object *, __pyx_t_5pycx4_5pycda_event *);
  void (*unregister_event)(struct __pyx_obj_5pycx4_5pycda_cda_object *, __pyx_t_5pycx4_5pycda_event *);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_cda_object *__pyx_vtabptr_5pycx4_5pycda_cda_object;


/* "pycx4/pycdabase.pxi":139
 * 
 * 
 * cdef class cda_context(cda_object):             # <<<<<<<<<<<<<<
 *     cdef readonly:
 *         cda_context_t cid
 */

struct __pyx_vtabstruct_5pycx4_5pycda_cda_context {
  struct __pyx_vtabstruct_5pycx4_5pycda_cda_object __pyx_base;
  void (*save_chan)(struct __pyx_obj_5pycx4_5pycda_cda_context *, void *);
  void (*drop_chan)(struct __pyx_obj_5pycx4_5pycda_cda_context *, void *);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_cda_context *__pyx_vtabptr_5pycx4_5pycda_cda_context;


/* "pycx4/pycdabase.pxi":221
 * 
 * # wrapper-class for low-level functions and channel registration
 * cdef class cda_base_chan(cda_object):             # <<<<<<<<<<<<<<
 *     cdef readonly:
 *         cda_dataref_t ref
 */

struct __pyx_vtabstruct_5pycx4_5pycda_cda_base_chan {
  struct __pyx_vtabstruct_5pycx4_5pycda_cda_object __pyx_base;
  void (*cb)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *);
  void (*snd_data)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *, cxdtype_t, int, void *);
  void (*get_data)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *, size_t, size_t, void *);
  int (*current_nelems)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *);
  void (*get_src)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *, char const **);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_cda_base_chan *__pyx_vtabptr_5pycx4_5pycda_cda_base_chan;


/* "pycx4/pycdauser.pxi":6
 * 
 * # scalar double channel
 * cdef class sdchan(cda_base_chan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         # all general properties defined in base classes
 */

struct __pyx_vtabstruct_5pycx4_5pycda_sdchan {
  struct __pyx_vtabstruct_5pycx4_5pycda_cda_base_chan __pyx_base;
  PyObject *(*setValue)(struct __pyx_obj_5pycx4_5pycda_sdchan *, double, int __pyx_skip_dispatch);
  PyObject *(*setTolerance)(struct __pyx_obj_5pycx4_5pycda_sdchan *, double, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_sdchan *__pyx_vtabptr_5pycx4_5pycda_sdchan;


/* "pycx4/pycdauser.pxi":30
 * 
 * 
 * cdef class strchan(cda_base_chan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly bytes val
 */

struct __pyx_vtabstruct_5pycx4_5pycda_strchan {
  struct __pyx_vtabstruct_5pycx4_5pycda_cda_base_chan __pyx_base;
  PyObject *(*setValue)(struct __pyx_obj_5pycx4_5pycda_strchan *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_strchan *__pyx_vtabptr_5pycx4_5pycda_strchan;


/* "pycx4/pycdauser.pxi":123
 * 
 * # general channel (any cx type)
 * cdef class schan(cda_base_chan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly object val, prev_val
 */

struct __pyx_vtabstruct_5pycx4_5pycda_schan {
  struct __pyx_vtabstruct_5pycx4_5pycda_cda_base_chan __pyx_base;
  PyObject *(*setValue)(struct __pyx_obj_5pycx4_5pycda_schan *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_schan *__pyx_vtabptr_5pycx4_5pycda_schan;


/* "pycx4/pycdauser.pxi":147
 * 
 * # vector-data channel class
 * cdef class vchan(cda_base_chan):             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly np.ndarray val
 */

struct __pyx_vtabstruct_5pycx4_5pycda_vchan {
  struct __pyx_vtabstruct_5pycx4_5pycda_cda_base_chan __pyx_base;
  PyObject *(*setValue)(struct __pyx_obj_5pycx4_5pycda_vchan *, PyArrayObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5pycx4_5pycda_vchan *__pyx_vtabptr_5pycx4_5pycda_vchan;

/* --- Runtime support code (head) --- */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

#if CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
#define __Pyx_PyCallable_Check(obj)   ((obj)->ob_type->tp_call != NULL)
#else
#define __Pyx_PyCallable_Check(obj)   PyCallable_Check(obj)
#endif

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb);

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

static CYTHON_INLINE int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact);

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#else
    #define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#endif

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

static int __Pyx_SetVtable(PyObject *dict, void *vtable);

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

static CYTHON_INLINE uint8 __Pyx_PyInt_As_uint8(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_cda_context_t(cda_context_t value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_CxDataRef_t(CxDataRef_t value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8(uint8 value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int64(int64 value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int32(int32 value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int8(int8 value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int16(int16 value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16(uint16 value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint32(uint32 value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint64(uint64 value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_char8(char8 value);

static CYTHON_INLINE int32 __Pyx_PyInt_As_int32(PyObject *);

static CYTHON_INLINE int8 __Pyx_PyInt_As_int8(PyObject *);

static CYTHON_INLINE int16 __Pyx_PyInt_As_int16(PyObject *);

static CYTHON_INLINE int64 __Pyx_PyInt_As_int64(PyObject *);

static CYTHON_INLINE uint16 __Pyx_PyInt_As_uint16(PyObject *);

static CYTHON_INLINE uint32 __Pyx_PyInt_As_uint32(PyObject *);

static CYTHON_INLINE uint64 __Pyx_PyInt_As_uint64(PyObject *);

static CYTHON_INLINE char8 __Pyx_PyInt_As_char8(PyObject *);

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if (defined(_WIN32) || defined(__clang__)) && defined(__cplusplus) && CYTHON_CCOMPLEX
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eqf(a, b)   ((a)==(b))
    #define __Pyx_c_sumf(a, b)  ((a)+(b))
    #define __Pyx_c_difff(a, b) ((a)-(b))
    #define __Pyx_c_prodf(a, b) ((a)*(b))
    #define __Pyx_c_quotf(a, b) ((a)/(b))
    #define __Pyx_c_negf(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zerof(z) ((z)==(float)0)
    #define __Pyx_c_conjf(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_absf(z)     (::std::abs(z))
        #define __Pyx_c_powf(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zerof(z) ((z)==0)
    #define __Pyx_c_conjf(z)    (conjf(z))
    #if 1
        #define __Pyx_c_absf(z)     (cabsf(z))
        #define __Pyx_c_powf(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq(a, b)   ((a)==(b))
    #define __Pyx_c_sum(a, b)  ((a)+(b))
    #define __Pyx_c_diff(a, b) ((a)-(b))
    #define __Pyx_c_prod(a, b) ((a)*(b))
    #define __Pyx_c_quot(a, b) ((a)/(b))
    #define __Pyx_c_neg(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero(z) ((z)==(double)0)
    #define __Pyx_c_conj(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (::std::abs(z))
        #define __Pyx_c_pow(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero(z) ((z)==0)
    #define __Pyx_c_conj(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (cabs(z))
        #define __Pyx_c_pow(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

static int __Pyx_check_binary_version(void);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name);

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_5pycx4_5pycda_10cda_signal_connect(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self, PyObject *__pyx_v_callback, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_10cda_signal_disconnect(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self, PyObject *__pyx_v_callback, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_10cda_signal_emit(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self, PyObject *__pyx_v_arg, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_5pycx4_5pycda_10cda_object_add_event(struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self, int __pyx_v_evmask, void *__pyx_v_evproc, void *__pyx_v_objptr, void *__pyx_v_userptr); /* proto*/
static int __pyx_f_5pycx4_5pycda_10cda_object_del_event(struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self, int __pyx_v_evmask, void *__pyx_v_evproc, void *__pyx_v_objptr, void *__pyx_v_userptr); /* proto*/
static int __pyx_f_5pycx4_5pycda_10cda_object_search_event(struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev); /* proto*/
static void __pyx_f_5pycx4_5pycda_10cda_object_register_event(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self, CYTHON_UNUSED __pyx_t_5pycx4_5pycda_event *__pyx_v_ev); /* proto*/
static void __pyx_f_5pycx4_5pycda_10cda_object_unregister_event(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self, CYTHON_UNUSED __pyx_t_5pycx4_5pycda_event *__pyx_v_ev); /* proto*/
static void __pyx_f_5pycx4_5pycda_11cda_context_save_chan(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self, void *__pyx_v_chan); /* proto*/
static void __pyx_f_5pycx4_5pycda_11cda_context_drop_chan(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self, void *__pyx_v_chan); /* proto*/
static void __pyx_f_5pycx4_5pycda_11cda_context_register_event(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev); /* proto*/
static void __pyx_f_5pycx4_5pycda_11cda_context_unregister_event(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev); /* proto*/
static void __pyx_f_5pycx4_5pycda_13cda_base_chan_cb(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto*/
static void __pyx_f_5pycx4_5pycda_13cda_base_chan_snd_data(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self, cxdtype_t __pyx_v_dtype, int __pyx_v_nelems, void *__pyx_v_data_p); /* proto*/
static void __pyx_f_5pycx4_5pycda_13cda_base_chan_get_data(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self, size_t __pyx_v_ofs, size_t __pyx_v_size, void *__pyx_v_buf); /* proto*/
static int __pyx_f_5pycx4_5pycda_13cda_base_chan_current_nelems(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto*/
static void __pyx_f_5pycx4_5pycda_13cda_base_chan_get_src(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self, char const **__pyx_v_src_p); /* proto*/
static void __pyx_f_5pycx4_5pycda_13cda_base_chan_register_event(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev); /* proto*/
static void __pyx_f_5pycx4_5pycda_13cda_base_chan_unregister_event(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev); /* proto*/
static void __pyx_f_5pycx4_5pycda_6sdchan_cb(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_6sdchan_setValue(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self, double __pyx_v_value, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_6sdchan_setTolerance(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self, double __pyx_v_new_tolerance, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_7strchan_cb(struct __pyx_obj_5pycx4_5pycda_strchan *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_7strchan_setValue(struct __pyx_obj_5pycx4_5pycda_strchan *__pyx_v_self, PyObject *__pyx_v_value, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_5schan_cb(struct __pyx_obj_5pycx4_5pycda_schan *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_5schan_setValue(struct __pyx_obj_5pycx4_5pycda_schan *__pyx_v_self, PyObject *__pyx_v_value, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5pycx4_5pycda_5vchan_cb(struct __pyx_obj_5pycx4_5pycda_vchan *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5pycx4_5pycda_5vchan_setValue(struct __pyx_obj_5pycx4_5pycda_vchan *__pyx_v_self, PyArrayObject *__pyx_v_value, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'pycx4.cxscheduler' */

/* Module declarations from 'pycx4.misc_types' */

/* Module declarations from 'pycx4.cx_common_types' */

/* Module declarations from 'pycx4.cx' */

/* Module declarations from 'pycx4.cda' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *, char *, char *, int *); /*proto*/

/* Module declarations from 'pycx4.pycda' */
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_cda_signal = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_cda_object = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_cda_context = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_cda_base_chan = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_sdchan = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_strchan = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_schan = 0;
static PyTypeObject *__pyx_ptype_5pycx4_5pycda_vchan = 0;
static struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_5pycx4_5pycda_default_context = 0;
static CYTHON_INLINE int __pyx_f_5pycx4_5pycda_cda_check_exception(int); /*proto*/
static void __pyx_f_5pycx4_5pycda_evproc_cont_cycle(int, void *, cda_context_t, int, int, void *); /*proto*/
static void __pyx_f_5pycx4_5pycda_evproc_rslvstat(int, void *, cda_dataref_t, int, void *, void *); /*proto*/
static void __pyx_f_5pycx4_5pycda_evproc_update(int, void *, cda_dataref_t, int, void *, void *); /*proto*/
static CYTHON_INLINE int __pyx_f_5pycx4_5pycda_event_feasible(__pyx_t_5pycx4_5pycda_event *); /*proto*/
static CYTHON_INLINE int __pyx_f_5pycx4_5pycda_cmp_events(__pyx_t_5pycx4_5pycda_event *, __pyx_t_5pycx4_5pycda_event *); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_aval_value(CxAnyVal_t *, cxdtype_t); /*proto*/
static void __pyx_f_5pycx4_5pycda_aval_setvalue(PyObject *, CxAnyVal_t *, cxdtype_t); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_cxdtype2np(cxdtype_t); /*proto*/
static cxdtype_t __pyx_f_5pycx4_5pycda_np2cxdtype(PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "pycx4.pycda"
int __pyx_module_is_main_pycx4__pycda = 0;

/* Implementation of 'pycx4.pycda' */
static PyObject *__pyx_builtin_Exception;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_RuntimeError;
static char __pyx_k_B[] = "B";
static char __pyx_k_C[] = "C";
static char __pyx_k_H[] = "H";
static char __pyx_k_I[] = "I";
static char __pyx_k_L[] = "L";
static char __pyx_k_O[] = "O";
static char __pyx_k_Q[] = "Q";
static char __pyx_k_b[] = "b";
static char __pyx_k_d[] = "d";
static char __pyx_k_f[] = "f";
static char __pyx_k_g[] = "g";
static char __pyx_k_h[] = "h";
static char __pyx_k_i[] = "i";
static char __pyx_k_l[] = "l";
static char __pyx_k_q[] = "q";
static char __pyx_k_Zd[] = "Zd";
static char __pyx_k_Zf[] = "Zf";
static char __pyx_k_Zg[] = "Zg";
static char __pyx_k_cx[] = "cx::";
static char __pyx_k_np[] = "np";
static char __pyx_k_emit[] = "emit";
static char __pyx_k_init[] = "__init__";
static char __pyx_k_int8[] = "int8";
static char __pyx_k_main[] = "__main__";
static char __pyx_k_name[] = "name";
static char __pyx_k_size[] = "size";
static char __pyx_k_test[] = "__test__";
static char __pyx_k_ascii[] = "ascii";
static char __pyx_k_dtype[] = "dtype";
static char __pyx_k_int16[] = "int16";
static char __pyx_k_int32[] = "int32";
static char __pyx_k_int64[] = "int64";
static char __pyx_k_numpy[] = "numpy";
static char __pyx_k_order[] = "order";
static char __pyx_k_range[] = "range";
static char __pyx_k_uint8[] = "uint8";
static char __pyx_k_zeros[] = "zeros";
static char __pyx_k_defpfx[] = "defpfx";
static char __pyx_k_double[] = "double";
static char __pyx_k_encode[] = "encode";
static char __pyx_k_import[] = "__import__";
static char __pyx_k_single[] = "single";
static char __pyx_k_uint16[] = "uint16";
static char __pyx_k_uint32[] = "uint32";
static char __pyx_k_uint64[] = "uint64";
static char __pyx_k_connect[] = "connect";
static char __pyx_k_context[] = "context";
static char __pyx_k_setValue[] = "setValue";
static char __pyx_k_Exception[] = "Exception";
static char __pyx_k_ValueError[] = "ValueError";
static char __pyx_k_disconnect[] = "disconnect";
static char __pyx_k_max_nelems[] = "max_nelems";
static char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static char __pyx_k_MemoryError[] = "MemoryError";
static char __pyx_k_py_sl_break[] = "py_sl_break";
static char __pyx_k_pycx4_pycda[] = "pycx4.pycda";
static char __pyx_k_RuntimeError[] = "RuntimeError";
static char __pyx_k_setTolerance[] = "setTolerance";
static char __pyx_k_PY_CXDTYPE_INT8[] = "PY_CXDTYPE_INT8";
static char __pyx_k_PY_CXDTYPE_TEXT[] = "PY_CXDTYPE_TEXT";
static char __pyx_k_py_sl_main_loop[] = "py_sl_main_loop";
static char __pyx_k_PY_CXDTYPE_INT16[] = "PY_CXDTYPE_INT16";
static char __pyx_k_PY_CXDTYPE_INT32[] = "PY_CXDTYPE_INT32";
static char __pyx_k_PY_CXDTYPE_INT64[] = "PY_CXDTYPE_INT64";
static char __pyx_k_PY_CXDTYPE_UINT8[] = "PY_CXDTYPE_UINT8";
static char __pyx_k_PY_CXDTYPE_DOUBLE[] = "PY_CXDTYPE_DOUBLE";
static char __pyx_k_PY_CXDTYPE_SINGLE[] = "PY_CXDTYPE_SINGLE";
static char __pyx_k_PY_CXDTYPE_UCTEXT[] = "PY_CXDTYPE_UCTEXT";
static char __pyx_k_PY_CXDTYPE_UINT16[] = "PY_CXDTYPE_UINT16";
static char __pyx_k_PY_CXDTYPE_UINT32[] = "PY_CXDTYPE_UINT32";
static char __pyx_k_PY_CXDTYPE_UINT64[] = "PY_CXDTYPE_UINT64";
static char __pyx_k_PY_CXDTYPE_UNKNOWN[] = "PY_CXDTYPE_UNKNOWN";
static char __pyx_k_cda_error_s_errcode_s[] = "cda error: %s, errcode: %s";
static char __pyx_k_A_function_was_expected[] = "A function was expected";
static char __pyx_k_cda_channel_ref_d_name_s[] = "<cda_channel: ref=%d, name=%s>";
static char __pyx_k_ndarray_is_not_C_contiguous[] = "ndarray is not C contiguous";
static char __pyx_k_cda_context_cid_d_defpfx_s_chan[] = "<cda_context: cid=%d, defpfx=%s, channum=%d>";
static char __pyx_k_home_femanov_python_pycx4_pycx4[] = "/home/femanov/python/pycx4/pycx4/pycda.pyx";
static char __pyx_k_unknown_dtype_code_in_numpy_pxd[] = "unknown dtype code in numpy.pxd (%d)";
static char __pyx_k_value_size_greater_than_channel[] = "value size greater than channel.max_nelems";
static char __pyx_k_Format_string_allocated_too_shor[] = "Format string allocated too short, see comment in numpy.pxd";
static char __pyx_k_Non_native_byte_order_not_suppor[] = "Non-native byte order not supported";
static char __pyx_k_ndarray_is_not_Fortran_contiguou[] = "ndarray is not Fortran contiguous";
static char __pyx_k_Format_string_allocated_too_shor_2[] = "Format string allocated too short.";
static PyObject *__pyx_kp_s_A_function_was_expected;
static PyObject *__pyx_n_s_C;
static PyObject *__pyx_n_s_Exception;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor_2;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_kp_u_Non_native_byte_order_not_suppor;
static PyObject *__pyx_n_s_PY_CXDTYPE_DOUBLE;
static PyObject *__pyx_n_s_PY_CXDTYPE_INT16;
static PyObject *__pyx_n_s_PY_CXDTYPE_INT32;
static PyObject *__pyx_n_s_PY_CXDTYPE_INT64;
static PyObject *__pyx_n_s_PY_CXDTYPE_INT8;
static PyObject *__pyx_n_s_PY_CXDTYPE_SINGLE;
static PyObject *__pyx_n_s_PY_CXDTYPE_TEXT;
static PyObject *__pyx_n_s_PY_CXDTYPE_UCTEXT;
static PyObject *__pyx_n_s_PY_CXDTYPE_UINT16;
static PyObject *__pyx_n_s_PY_CXDTYPE_UINT32;
static PyObject *__pyx_n_s_PY_CXDTYPE_UINT64;
static PyObject *__pyx_n_s_PY_CXDTYPE_UINT8;
static PyObject *__pyx_n_s_PY_CXDTYPE_UNKNOWN;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_ascii;
static PyObject *__pyx_kp_s_cda_channel_ref_d_name_s;
static PyObject *__pyx_kp_s_cda_context_cid_d_defpfx_s_chan;
static PyObject *__pyx_kp_s_cda_error_s_errcode_s;
static PyObject *__pyx_n_s_connect;
static PyObject *__pyx_n_s_context;
static PyObject *__pyx_kp_s_cx;
static PyObject *__pyx_n_s_defpfx;
static PyObject *__pyx_n_s_disconnect;
static PyObject *__pyx_n_s_double;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_s_emit;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_kp_s_home_femanov_python_pycx4_pycx4;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_int16;
static PyObject *__pyx_n_s_int32;
static PyObject *__pyx_n_s_int64;
static PyObject *__pyx_n_s_int8;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_max_nelems;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_kp_u_ndarray_is_not_C_contiguous;
static PyObject *__pyx_kp_u_ndarray_is_not_Fortran_contiguou;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_n_s_order;
static PyObject *__pyx_n_s_py_sl_break;
static PyObject *__pyx_n_s_py_sl_main_loop;
static PyObject *__pyx_n_s_pycx4_pycda;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_setTolerance;
static PyObject *__pyx_n_s_setValue;
static PyObject *__pyx_n_s_single;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_uint16;
static PyObject *__pyx_n_s_uint32;
static PyObject *__pyx_n_s_uint64;
static PyObject *__pyx_n_s_uint8;
static PyObject *__pyx_kp_u_unknown_dtype_code_in_numpy_pxd;
static PyObject *__pyx_kp_s_value_size_greater_than_channel;
static PyObject *__pyx_n_s_zeros;
static PyObject *__pyx_pf_5pycx4_5pycda_py_sl_main_loop(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_2py_sl_break(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static void __pyx_pf_5pycx4_5pycda_10cda_signal___dealloc__(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_10cda_signal_2connect(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self, PyObject *__pyx_v_callback); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_10cda_signal_4disconnect(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self, PyObject *__pyx_v_callback); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_10cda_signal_6emit(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self, PyObject *__pyx_v_arg); /* proto */
static int __pyx_pf_5pycx4_5pycda_10cda_object___init__(struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self); /* proto */
static void __pyx_pf_5pycx4_5pycda_10cda_object_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self); /* proto */
static int __pyx_pf_5pycx4_5pycda_11cda_context___init__(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self, PyObject *__pyx_v_defpfx); /* proto */
static void __pyx_pf_5pycx4_5pycda_11cda_context_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_11cda_context_4__str__(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_11cda_context_6enable_serverCycle(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_11cda_context_3cid___get__(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_11cda_context_6defpfx___get__(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_11cda_context_11serverCycle___get__(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self); /* proto */
static int __pyx_pf_5pycx4_5pycda_13cda_base_chan___init__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_context, cxdtype_t __pyx_v_dtype, int __pyx_v_max_nelems); /* proto */
static void __pyx_pf_5pycx4_5pycda_13cda_base_chan_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_4__str__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_3ref___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_4name___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_10max_nelems___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_5dtype___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_8itemsize___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_4time___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_9prev_time___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_5found___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_13valueMeasured___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_12valueChanged___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self); /* proto */
static int __pyx_pf_5pycx4_5pycda_6sdchan___init__(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_context); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6sdchan_2setValue(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self, double __pyx_v_value); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6sdchan_4setTolerance(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self, double __pyx_v_new_tolerance); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6sdchan_3val___get__(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6sdchan_8prev_val___get__(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_6sdchan_9tolerance___get__(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self); /* proto */
static int __pyx_pf_5pycx4_5pycda_7strchan___init__(struct __pyx_obj_5pycx4_5pycda_strchan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_context, int __pyx_v_max_nelems); /* proto */
static void __pyx_pf_5pycx4_5pycda_7strchan_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_strchan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7strchan_4setValue(struct __pyx_obj_5pycx4_5pycda_strchan *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_7strchan_3val___get__(struct __pyx_obj_5pycx4_5pycda_strchan *__pyx_v_self); /* proto */
static int __pyx_pf_5pycx4_5pycda_5schan___init__(struct __pyx_obj_5pycx4_5pycda_schan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_context, cxdtype_t __pyx_v_dtype); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5schan_2setValue(struct __pyx_obj_5pycx4_5pycda_schan *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5schan_3val___get__(struct __pyx_obj_5pycx4_5pycda_schan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5schan_8prev_val___get__(struct __pyx_obj_5pycx4_5pycda_schan *__pyx_v_self); /* proto */
static int __pyx_pf_5pycx4_5pycda_5vchan___init__(struct __pyx_obj_5pycx4_5pycda_vchan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_context, cxdtype_t __pyx_v_dtype, int __pyx_v_max_nelems); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5vchan_2setValue(struct __pyx_obj_5pycx4_5pycda_vchan *__pyx_v_self, PyArrayObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5vchan_3val___get__(struct __pyx_obj_5pycx4_5pycda_vchan *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5pycx4_5pycda_5vchan_7npdtype___get__(struct __pyx_obj_5pycx4_5pycda_vchan *__pyx_v_self); /* proto */
static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static PyObject *__pyx_tp_new_5pycx4_5pycda_cda_signal(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_cda_object(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_cda_context(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_cda_base_chan(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_sdchan(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_strchan(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_schan(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5pycx4_5pycda_vchan(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static cxdtype_t __pyx_k__3;
static cxdtype_t __pyx_k__5;
static cxdtype_t __pyx_k__6;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_codeobj__14;
static PyObject *__pyx_codeobj__15;

/* "pycx4/pycda.pyx":3
 * from cxscheduler cimport *
 * 
 * def py_sl_main_loop():             # <<<<<<<<<<<<<<
 *     sl_main_loop()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_1py_sl_main_loop(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_5pycx4_5pycda_1py_sl_main_loop = {"py_sl_main_loop", (PyCFunction)__pyx_pw_5pycx4_5pycda_1py_sl_main_loop, METH_NOARGS, 0};
static PyObject *__pyx_pw_5pycx4_5pycda_1py_sl_main_loop(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("py_sl_main_loop (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_py_sl_main_loop(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_py_sl_main_loop(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("py_sl_main_loop", 0);

  /* "pycx4/pycda.pyx":4
 * 
 * def py_sl_main_loop():
 *     sl_main_loop()             # <<<<<<<<<<<<<<
 * 
 * def py_sl_break():
 */
  sl_main_loop();

  /* "pycx4/pycda.pyx":3
 * from cxscheduler cimport *
 * 
 * def py_sl_main_loop():             # <<<<<<<<<<<<<<
 *     sl_main_loop()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycda.pyx":6
 *     sl_main_loop()
 * 
 * def py_sl_break():             # <<<<<<<<<<<<<<
 *     sl_break()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_3py_sl_break(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_5pycx4_5pycda_3py_sl_break = {"py_sl_break", (PyCFunction)__pyx_pw_5pycx4_5pycda_3py_sl_break, METH_NOARGS, 0};
static PyObject *__pyx_pw_5pycx4_5pycda_3py_sl_break(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("py_sl_break (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_2py_sl_break(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_2py_sl_break(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("py_sl_break", 0);

  /* "pycx4/pycda.pyx":7
 * 
 * def py_sl_break():
 *     sl_break()             # <<<<<<<<<<<<<<
 * 
 * # conpile-time define for contitional compilation
 */
  sl_break();

  /* "pycx4/pycda.pyx":6
 *     sl_main_loop()
 * 
 * def py_sl_break():             # <<<<<<<<<<<<<<
 *     sl_break()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/cda_signal.pxi":13
 * #        self.cnum = 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         free(self.callbacks)
 * 
 */

/* Python wrapper */
static void __pyx_pw_5pycx4_5pycda_10cda_signal_1__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pycx4_5pycda_10cda_signal_1__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pycx4_5pycda_10cda_signal___dealloc__(((struct __pyx_obj_5pycx4_5pycda_cda_signal *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pycx4_5pycda_10cda_signal___dealloc__(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pycx4/cda_signal.pxi":14
 * 
 *     def __dealloc__(self):
 *         free(self.callbacks)             # <<<<<<<<<<<<<<
 * 
 *     cpdef connect(self, callback):
 */
  free(__pyx_v_self->callbacks);

  /* "pycx4/cda_signal.pxi":13
 * #        self.cnum = 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         free(self.callbacks)
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/cda_signal.pxi":16
 *         free(self.callbacks)
 * 
 *     cpdef connect(self, callback):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp
 */

static PyObject *__pyx_pw_5pycx4_5pycda_10cda_signal_3connect(PyObject *__pyx_v_self, PyObject *__pyx_v_callback); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_10cda_signal_connect(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self, PyObject *__pyx_v_callback, int __pyx_skip_dispatch) {
  void *__pyx_v_tmp;
  int __pyx_v_ind;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("connect", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_connect); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 16; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5pycx4_5pycda_10cda_signal_3connect)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_callback); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 16; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 16; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_callback);
        __Pyx_GIVEREF(__pyx_v_callback);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_callback);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 16; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pycx4/cda_signal.pxi":20
 *             void *tmp
 *             int ind
 *         if not callable(callback): raise Exception('A function was expected')             # <<<<<<<<<<<<<<
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:
 */
  __pyx_t_6 = __Pyx_PyCallable_Check(__pyx_v_callback); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_7 = ((!(__pyx_t_6 != 0)) != 0);
  if (__pyx_t_7) {
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_Exception, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "pycx4/cda_signal.pxi":21
 *             int ind
 *         if not callable(callback): raise Exception('A function was expected')
 *         for ind in range(self.cnum):             # <<<<<<<<<<<<<<
 *             if callback == <object>self.callbacks[ind]:
 *                 return
 */
  __pyx_t_8 = __pyx_v_self->cnum;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_ind = __pyx_t_9;

    /* "pycx4/cda_signal.pxi":22
 *         if not callable(callback): raise Exception('A function was expected')
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:             # <<<<<<<<<<<<<<
 *                 return
 *         tmp = realloc(<void*>self.callbacks, sizeof(void*) * (self.cnum+1))
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_callback, ((PyObject *)(__pyx_v_self->callbacks[__pyx_v_ind])), Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 22; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 22; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_7) {

      /* "pycx4/cda_signal.pxi":23
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:
 *                 return             # <<<<<<<<<<<<<<
 *         tmp = realloc(<void*>self.callbacks, sizeof(void*) * (self.cnum+1))
 *         if not tmp: raise MemoryError()
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L0;

      /* "pycx4/cda_signal.pxi":22
 *         if not callable(callback): raise Exception('A function was expected')
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:             # <<<<<<<<<<<<<<
 *                 return
 *         tmp = realloc(<void*>self.callbacks, sizeof(void*) * (self.cnum+1))
 */
    }
  }

  /* "pycx4/cda_signal.pxi":24
 *             if callback == <object>self.callbacks[ind]:
 *                 return
 *         tmp = realloc(<void*>self.callbacks, sizeof(void*) * (self.cnum+1))             # <<<<<<<<<<<<<<
 *         if not tmp: raise MemoryError()
 *         self.callbacks = <void**>tmp
 */
  __pyx_v_tmp = realloc(((void *)__pyx_v_self->callbacks), ((sizeof(void *)) * (__pyx_v_self->cnum + 1)));

  /* "pycx4/cda_signal.pxi":25
 *                 return
 *         tmp = realloc(<void*>self.callbacks, sizeof(void*) * (self.cnum+1))
 *         if not tmp: raise MemoryError()             # <<<<<<<<<<<<<<
 *         self.callbacks = <void**>tmp
 *         self.callbacks[self.cnum] = <void*>callback
 */
  __pyx_t_7 = ((!(__pyx_v_tmp != 0)) != 0);
  if (__pyx_t_7) {
    PyErr_NoMemory(); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "pycx4/cda_signal.pxi":26
 *         tmp = realloc(<void*>self.callbacks, sizeof(void*) * (self.cnum+1))
 *         if not tmp: raise MemoryError()
 *         self.callbacks = <void**>tmp             # <<<<<<<<<<<<<<
 *         self.callbacks[self.cnum] = <void*>callback
 *         self.cnum += 1
 */
  __pyx_v_self->callbacks = ((void **)__pyx_v_tmp);

  /* "pycx4/cda_signal.pxi":27
 *         if not tmp: raise MemoryError()
 *         self.callbacks = <void**>tmp
 *         self.callbacks[self.cnum] = <void*>callback             # <<<<<<<<<<<<<<
 *         self.cnum += 1
 * 
 */
  (__pyx_v_self->callbacks[__pyx_v_self->cnum]) = ((void *)__pyx_v_callback);

  /* "pycx4/cda_signal.pxi":28
 *         self.callbacks = <void**>tmp
 *         self.callbacks[self.cnum] = <void*>callback
 *         self.cnum += 1             # <<<<<<<<<<<<<<
 * 
 *     cpdef disconnect(self, callback):
 */
  __pyx_v_self->cnum = (__pyx_v_self->cnum + 1);

  /* "pycx4/cda_signal.pxi":16
 *         free(self.callbacks)
 * 
 *     cpdef connect(self, callback):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.cda_signal.connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_10cda_signal_3connect(PyObject *__pyx_v_self, PyObject *__pyx_v_callback); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_10cda_signal_3connect(PyObject *__pyx_v_self, PyObject *__pyx_v_callback) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("connect (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_10cda_signal_2connect(((struct __pyx_obj_5pycx4_5pycda_cda_signal *)__pyx_v_self), ((PyObject *)__pyx_v_callback));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_10cda_signal_2connect(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self, PyObject *__pyx_v_callback) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("connect", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_10cda_signal_connect(__pyx_v_self, __pyx_v_callback, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 16; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.cda_signal.connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/cda_signal.pxi":30
 *         self.cnum += 1
 * 
 *     cpdef disconnect(self, callback):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp
 */

static PyObject *__pyx_pw_5pycx4_5pycda_10cda_signal_5disconnect(PyObject *__pyx_v_self, PyObject *__pyx_v_callback); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_10cda_signal_disconnect(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self, PyObject *__pyx_v_callback, int __pyx_skip_dispatch) {
  void *__pyx_v_tmp;
  int __pyx_v_ind;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("disconnect", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_disconnect); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5pycx4_5pycda_10cda_signal_5disconnect)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_callback); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_callback);
        __Pyx_GIVEREF(__pyx_v_callback);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_callback);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pycx4/cda_signal.pxi":34
 *             void *tmp
 *             int ind
 *         for ind in range(self.cnum):             # <<<<<<<<<<<<<<
 *             if callback == <object>self.callbacks[ind]:
 *                 if self.cnum == 1:
 */
  __pyx_t_6 = __pyx_v_self->cnum;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_ind = __pyx_t_7;

    /* "pycx4/cda_signal.pxi":35
 *             int ind
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:             # <<<<<<<<<<<<<<
 *                 if self.cnum == 1:
 *                     free(self.callbacks)
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_callback, ((PyObject *)(__pyx_v_self->callbacks[__pyx_v_ind])), Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 35; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 35; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_8) {

      /* "pycx4/cda_signal.pxi":36
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:
 *                 if self.cnum == 1:             # <<<<<<<<<<<<<<
 *                     free(self.callbacks)
 *                 else:
 */
      __pyx_t_8 = ((__pyx_v_self->cnum == 1) != 0);
      if (__pyx_t_8) {

        /* "pycx4/cda_signal.pxi":37
 *             if callback == <object>self.callbacks[ind]:
 *                 if self.cnum == 1:
 *                     free(self.callbacks)             # <<<<<<<<<<<<<<
 *                 else:
 *                     memmove(&(self.callbacks[ind]), &(self.callbacks[self.cnum-1]), sizeof(void*))
 */
        free(__pyx_v_self->callbacks);

        /* "pycx4/cda_signal.pxi":36
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:
 *                 if self.cnum == 1:             # <<<<<<<<<<<<<<
 *                     free(self.callbacks)
 *                 else:
 */
        goto __pyx_L6;
      }

      /* "pycx4/cda_signal.pxi":39
 *                     free(self.callbacks)
 *                 else:
 *                     memmove(&(self.callbacks[ind]), &(self.callbacks[self.cnum-1]), sizeof(void*))             # <<<<<<<<<<<<<<
 *                     tmp = realloc(self.callbacks, sizeof(void*) * (self.cnum-1))
 *                     if not tmp: raise MemoryError()
 */
      /*else*/ {
        memmove((&(__pyx_v_self->callbacks[__pyx_v_ind])), (&(__pyx_v_self->callbacks[(__pyx_v_self->cnum - 1)])), (sizeof(void *)));

        /* "pycx4/cda_signal.pxi":40
 *                 else:
 *                     memmove(&(self.callbacks[ind]), &(self.callbacks[self.cnum-1]), sizeof(void*))
 *                     tmp = realloc(self.callbacks, sizeof(void*) * (self.cnum-1))             # <<<<<<<<<<<<<<
 *                     if not tmp: raise MemoryError()
 *                     self.callbacks = <void**>tmp
 */
        __pyx_v_tmp = realloc(__pyx_v_self->callbacks, ((sizeof(void *)) * (__pyx_v_self->cnum - 1)));

        /* "pycx4/cda_signal.pxi":41
 *                     memmove(&(self.callbacks[ind]), &(self.callbacks[self.cnum-1]), sizeof(void*))
 *                     tmp = realloc(self.callbacks, sizeof(void*) * (self.cnum-1))
 *                     if not tmp: raise MemoryError()             # <<<<<<<<<<<<<<
 *                     self.callbacks = <void**>tmp
 *                 self.cnum -= 1
 */
        __pyx_t_8 = ((!(__pyx_v_tmp != 0)) != 0);
        if (__pyx_t_8) {
          PyErr_NoMemory(); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 41; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }

        /* "pycx4/cda_signal.pxi":42
 *                     tmp = realloc(self.callbacks, sizeof(void*) * (self.cnum-1))
 *                     if not tmp: raise MemoryError()
 *                     self.callbacks = <void**>tmp             # <<<<<<<<<<<<<<
 *                 self.cnum -= 1
 * 
 */
        __pyx_v_self->callbacks = ((void **)__pyx_v_tmp);
      }
      __pyx_L6:;

      /* "pycx4/cda_signal.pxi":43
 *                     if not tmp: raise MemoryError()
 *                     self.callbacks = <void**>tmp
 *                 self.cnum -= 1             # <<<<<<<<<<<<<<
 * 
 *     cpdef emit(self, object arg):
 */
      __pyx_v_self->cnum = (__pyx_v_self->cnum - 1);

      /* "pycx4/cda_signal.pxi":35
 *             int ind
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:             # <<<<<<<<<<<<<<
 *                 if self.cnum == 1:
 *                     free(self.callbacks)
 */
    }
  }

  /* "pycx4/cda_signal.pxi":30
 *         self.cnum += 1
 * 
 *     cpdef disconnect(self, callback):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.cda_signal.disconnect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_10cda_signal_5disconnect(PyObject *__pyx_v_self, PyObject *__pyx_v_callback); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_10cda_signal_5disconnect(PyObject *__pyx_v_self, PyObject *__pyx_v_callback) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("disconnect (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_10cda_signal_4disconnect(((struct __pyx_obj_5pycx4_5pycda_cda_signal *)__pyx_v_self), ((PyObject *)__pyx_v_callback));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_10cda_signal_4disconnect(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self, PyObject *__pyx_v_callback) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("disconnect", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_10cda_signal_disconnect(__pyx_v_self, __pyx_v_callback, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.cda_signal.disconnect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/cda_signal.pxi":45
 *                 self.cnum -= 1
 * 
 *     cpdef emit(self, object arg):             # <<<<<<<<<<<<<<
 *         cdef int ind
 *         for ind in range(self.cnum):
 */

static PyObject *__pyx_pw_5pycx4_5pycda_10cda_signal_7emit(PyObject *__pyx_v_self, PyObject *__pyx_v_arg); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_10cda_signal_emit(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self, PyObject *__pyx_v_arg, int __pyx_skip_dispatch) {
  int __pyx_v_ind;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("emit", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_emit); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5pycx4_5pycda_10cda_signal_7emit)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_arg); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_arg);
        __Pyx_GIVEREF(__pyx_v_arg);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_arg);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pycx4/cda_signal.pxi":47
 *     cpdef emit(self, object arg):
 *         cdef int ind
 *         for ind in range(self.cnum):             # <<<<<<<<<<<<<<
 *             (<object>(self.callbacks[ind]))(arg)
 */
  __pyx_t_6 = __pyx_v_self->cnum;
  for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
    __pyx_v_ind = __pyx_t_7;

    /* "pycx4/cda_signal.pxi":48
 *         cdef int ind
 *         for ind in range(self.cnum):
 *             (<object>(self.callbacks[ind]))(arg)             # <<<<<<<<<<<<<<
 */
    __Pyx_INCREF(((PyObject *)(__pyx_v_self->callbacks[__pyx_v_ind])));
    __pyx_t_2 = ((PyObject *)(__pyx_v_self->callbacks[__pyx_v_ind])); __pyx_t_3 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_arg); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_arg);
      __Pyx_GIVEREF(__pyx_v_arg);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_arg);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 48; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pycx4/cda_signal.pxi":45
 *                 self.cnum -= 1
 * 
 *     cpdef emit(self, object arg):             # <<<<<<<<<<<<<<
 *         cdef int ind
 *         for ind in range(self.cnum):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.cda_signal.emit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_10cda_signal_7emit(PyObject *__pyx_v_self, PyObject *__pyx_v_arg); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_10cda_signal_7emit(PyObject *__pyx_v_self, PyObject *__pyx_v_arg) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("emit (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_10cda_signal_6emit(((struct __pyx_obj_5pycx4_5pycda_cda_signal *)__pyx_v_self), ((PyObject *)__pyx_v_arg));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_10cda_signal_6emit(struct __pyx_obj_5pycx4_5pycda_cda_signal *__pyx_v_self, PyObject *__pyx_v_arg) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("emit", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_10cda_signal_emit(__pyx_v_self, __pyx_v_arg, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.cda_signal.emit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":32
 * 
 * # check for cda exception
 * cdef inline int cda_check_exception(int code) except -1:             # <<<<<<<<<<<<<<
 *     if code < 0:
 *         raise Exception("cda error: %s, errcode: %s" % (cda_last_err(), code))
 */

static CYTHON_INLINE int __pyx_f_5pycx4_5pycda_cda_check_exception(int __pyx_v_code) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cda_check_exception", 0);

  /* "pycx4/pycdabase.pxi":33
 * # check for cda exception
 * cdef inline int cda_check_exception(int code) except -1:
 *     if code < 0:             # <<<<<<<<<<<<<<
 *         raise Exception("cda error: %s, errcode: %s" % (cda_last_err(), code))
 *     return 0
 */
  __pyx_t_1 = ((__pyx_v_code < 0) != 0);
  if (__pyx_t_1) {

    /* "pycx4/pycdabase.pxi":34
 * cdef inline int cda_check_exception(int code) except -1:
 *     if code < 0:
 *         raise Exception("cda error: %s, errcode: %s" % (cda_last_err(), code))             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
    __pyx_t_2 = __Pyx_PyBytes_FromString(cda_last_err()); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_code); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_cda_error_s_errcode_s, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_Exception, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "pycx4/pycdabase.pxi":33
 * # check for cda exception
 * cdef inline int cda_check_exception(int code) except -1:
 *     if code < 0:             # <<<<<<<<<<<<<<
 *         raise Exception("cda error: %s, errcode: %s" % (cda_last_err(), code))
 *     return 0
 */
  }

  /* "pycx4/pycdabase.pxi":35
 *     if code < 0:
 *         raise Exception("cda error: %s, errcode: %s" % (cda_last_err(), code))
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * # C callback for context
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pycx4/pycdabase.pxi":32
 * 
 * # check for cda exception
 * cdef inline int cda_check_exception(int code) except -1:             # <<<<<<<<<<<<<<
 *     if code < 0:
 *         raise Exception("cda error: %s, errcode: %s" % (cda_last_err(), code))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.cda_check_exception", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":38
 * 
 * # C callback for context
 * cdef void evproc_cont_cycle(int uniq, void *privptr1, cda_context_t cid, int reason,             # <<<<<<<<<<<<<<
 *                       int info_int, void *privptr2) with gil:
 *     cdef cda_context cont=<cda_context>(<event*>privptr2).objptr
 */

static void __pyx_f_5pycx4_5pycda_evproc_cont_cycle(CYTHON_UNUSED int __pyx_v_uniq, CYTHON_UNUSED void *__pyx_v_privptr1, CYTHON_UNUSED cda_context_t __pyx_v_cid, CYTHON_UNUSED int __pyx_v_reason, CYTHON_UNUSED int __pyx_v_info_int, void *__pyx_v_privptr2) {
  struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_cont = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("evproc_cont_cycle", 0);

  /* "pycx4/pycdabase.pxi":40
 * cdef void evproc_cont_cycle(int uniq, void *privptr1, cda_context_t cid, int reason,
 *                       int info_int, void *privptr2) with gil:
 *     cdef cda_context cont=<cda_context>(<event*>privptr2).objptr             # <<<<<<<<<<<<<<
 *     cont.serverCycle.emit(cont)
 * 
 */
  __pyx_t_1 = ((PyObject *)((__pyx_t_5pycx4_5pycda_event *)__pyx_v_privptr2)->objptr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_cont = ((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":41
 *                       int info_int, void *privptr2) with gil:
 *     cdef cda_context cont=<cda_context>(<event*>privptr2).objptr
 *     cont.serverCycle.emit(cont)             # <<<<<<<<<<<<<<
 * 
 * # C callback function for ref's (channels)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_signal *)__pyx_v_cont->serverCycle->__pyx_vtab)->emit(__pyx_v_cont->serverCycle, ((PyObject *)__pyx_v_cont), 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 41; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":38
 * 
 * # C callback for context
 * cdef void evproc_cont_cycle(int uniq, void *privptr1, cda_context_t cid, int reason,             # <<<<<<<<<<<<<<
 *                       int info_int, void *privptr2) with gil:
 *     cdef cda_context cont=<cda_context>(<event*>privptr2).objptr
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("pycx4.pycda.evproc_cont_cycle", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_cont);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "pycx4/pycdabase.pxi":44
 * 
 * # C callback function for ref's (channels)
 * cdef void evproc_rslvstat(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                           void *info_ptr, void *privptr2) with gil:
 *     cdef cda_base_chan chan = <cda_base_chan>(<event*>privptr2).objptr
 */

static void __pyx_f_5pycx4_5pycda_evproc_rslvstat(CYTHON_UNUSED int __pyx_v_uniq, CYTHON_UNUSED void *__pyx_v_privptr1, CYTHON_UNUSED cda_dataref_t __pyx_v_ref, CYTHON_UNUSED int __pyx_v_reason, void *__pyx_v_info_ptr, void *__pyx_v_privptr2) {
  struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_chan = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("evproc_rslvstat", 0);

  /* "pycx4/pycdabase.pxi":46
 * cdef void evproc_rslvstat(int uniq, void *privptr1, cda_dataref_t ref, int reason,
 *                           void *info_ptr, void *privptr2) with gil:
 *     cdef cda_base_chan chan = <cda_base_chan>(<event*>privptr2).objptr             # <<<<<<<<<<<<<<
 *     if <long>info_ptr == 0: # this is channel not found
 *         chan.found=-1
 */
  __pyx_t_1 = ((PyObject *)((__pyx_t_5pycx4_5pycda_event *)__pyx_v_privptr2)->objptr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_chan = ((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":47
 *                           void *info_ptr, void *privptr2) with gil:
 *     cdef cda_base_chan chan = <cda_base_chan>(<event*>privptr2).objptr
 *     if <long>info_ptr == 0: # this is channel not found             # <<<<<<<<<<<<<<
 *         chan.found=-1
 *         pass
 */
  __pyx_t_2 = ((((long)__pyx_v_info_ptr) == 0) != 0);
  if (__pyx_t_2) {

    /* "pycx4/pycdabase.pxi":48
 *     cdef cda_base_chan chan = <cda_base_chan>(<event*>privptr2).objptr
 *     if <long>info_ptr == 0: # this is channel not found
 *         chan.found=-1             # <<<<<<<<<<<<<<
 *         pass
 * 
 */
    __pyx_v_chan->found = -1;

    /* "pycx4/pycdabase.pxi":47
 *                           void *info_ptr, void *privptr2) with gil:
 *     cdef cda_base_chan chan = <cda_base_chan>(<event*>privptr2).objptr
 *     if <long>info_ptr == 0: # this is channel not found             # <<<<<<<<<<<<<<
 *         chan.found=-1
 *         pass
 */
  }

  /* "pycx4/pycdabase.pxi":44
 * 
 * # C callback function for ref's (channels)
 * cdef void evproc_rslvstat(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                           void *info_ptr, void *privptr2) with gil:
 *     cdef cda_base_chan chan = <cda_base_chan>(<event*>privptr2).objptr
 */

  /* function exit code */
  __Pyx_XDECREF((PyObject *)__pyx_v_chan);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "pycx4/pycdabase.pxi":51
 *         pass
 * 
 * cdef void evproc_update(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 */

static void __pyx_f_5pycx4_5pycda_evproc_update(CYTHON_UNUSED int __pyx_v_uniq, CYTHON_UNUSED void *__pyx_v_privptr1, CYTHON_UNUSED cda_dataref_t __pyx_v_ref, CYTHON_UNUSED int __pyx_v_reason, CYTHON_UNUSED void *__pyx_v_info_ptr, void *__pyx_v_privptr2) {
  cx_time_t __pyx_v_timestr;
  rflags_t __pyx_v_rflags;
  struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_chan = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int64 __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("evproc_update", 0);

  /* "pycx4/pycdabase.pxi":56
 *         cx_time_t timestr
 *         rflags_t rflags
 *         cda_base_chan chan = <cda_base_chan>(<event*>privptr2).objptr             # <<<<<<<<<<<<<<
 *     cda_check_exception( cda_get_ref_stat(chan.ref, &rflags, &timestr) )
 *     chan.prev_time = chan.time
 */
  __pyx_t_1 = ((PyObject *)((__pyx_t_5pycx4_5pycda_event *)__pyx_v_privptr2)->objptr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_chan = ((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":57
 *         rflags_t rflags
 *         cda_base_chan chan = <cda_base_chan>(<event*>privptr2).objptr
 *     cda_check_exception( cda_get_ref_stat(chan.ref, &rflags, &timestr) )             # <<<<<<<<<<<<<<
 *     chan.prev_time = chan.time
 *     chan.time = <int64>timestr.sec * 1000000 + timestr.nsec / 1000
 */
  __pyx_t_2 = __pyx_f_5pycx4_5pycda_cda_check_exception(cda_get_ref_stat(__pyx_v_chan->ref, (&__pyx_v_rflags), (&__pyx_v_timestr))); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 57; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "pycx4/pycdabase.pxi":58
 *         cda_base_chan chan = <cda_base_chan>(<event*>privptr2).objptr
 *     cda_check_exception( cda_get_ref_stat(chan.ref, &rflags, &timestr) )
 *     chan.prev_time = chan.time             # <<<<<<<<<<<<<<
 *     chan.time = <int64>timestr.sec * 1000000 + timestr.nsec / 1000
 *     chan.cb()
 */
  __pyx_t_3 = __pyx_v_chan->time;
  __pyx_v_chan->prev_time = __pyx_t_3;

  /* "pycx4/pycdabase.pxi":59
 *     cda_check_exception( cda_get_ref_stat(chan.ref, &rflags, &timestr) )
 *     chan.prev_time = chan.time
 *     chan.time = <int64>timestr.sec * 1000000 + timestr.nsec / 1000             # <<<<<<<<<<<<<<
 *     chan.cb()
 * 
 */
  __pyx_v_chan->time = ((((int64)__pyx_v_timestr.sec) * 0xF4240) + (__pyx_v_timestr.nsec / 0x3E8));

  /* "pycx4/pycdabase.pxi":60
 *     chan.prev_time = chan.time
 *     chan.time = <int64>timestr.sec * 1000000 + timestr.nsec / 1000
 *     chan.cb()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_base_chan *)__pyx_v_chan->__pyx_base.__pyx_vtab)->cb(__pyx_v_chan);

  /* "pycx4/pycdabase.pxi":51
 *         pass
 * 
 * cdef void evproc_update(int uniq, void *privptr1, cda_dataref_t ref, int reason,             # <<<<<<<<<<<<<<
 *                         void *info_ptr, void *privptr2) with gil:
 *     cdef:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("pycx4.pycda.evproc_update", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_chan);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "pycx4/pycdabase.pxi":64
 * 
 * # event handling functions
 * cdef inline int event_feasible(event *ev):             # <<<<<<<<<<<<<<
 *     if ev.evmask == 0 or ev.evproc == NULL or ev.objptr == NULL:
 *         return 0
 */

static CYTHON_INLINE int __pyx_f_5pycx4_5pycda_event_feasible(__pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("event_feasible", 0);

  /* "pycx4/pycdabase.pxi":65
 * # event handling functions
 * cdef inline int event_feasible(event *ev):
 *     if ev.evmask == 0 or ev.evproc == NULL or ev.objptr == NULL:             # <<<<<<<<<<<<<<
 *         return 0
 *     return 1
 */
  __pyx_t_2 = ((__pyx_v_ev->evmask == 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ev->evproc == NULL) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ev->objptr == NULL) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "pycx4/pycdabase.pxi":66
 * cdef inline int event_feasible(event *ev):
 *     if ev.evmask == 0 or ev.evproc == NULL or ev.objptr == NULL:
 *         return 0             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pycx4/pycdabase.pxi":65
 * # event handling functions
 * cdef inline int event_feasible(event *ev):
 *     if ev.evmask == 0 or ev.evproc == NULL or ev.objptr == NULL:             # <<<<<<<<<<<<<<
 *         return 0
 *     return 1
 */
  }

  /* "pycx4/pycdabase.pxi":67
 *     if ev.evmask == 0 or ev.evproc == NULL or ev.objptr == NULL:
 *         return 0
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * cdef inline int cmp_events(event *ev1, event *ev2):
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "pycx4/pycdabase.pxi":64
 * 
 * # event handling functions
 * cdef inline int event_feasible(event *ev):             # <<<<<<<<<<<<<<
 *     if ev.evmask == 0 or ev.evproc == NULL or ev.objptr == NULL:
 *         return 0
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":69
 *     return 1
 * 
 * cdef inline int cmp_events(event *ev1, event *ev2):             # <<<<<<<<<<<<<<
 *     if ev1.evmask == ev2.evmask and ev1.evproc == ev2.evproc and \
 *         ev1.objptr == ev2.objptr and ev1.userptr == ev2.userptr:
 */

static CYTHON_INLINE int __pyx_f_5pycx4_5pycda_cmp_events(__pyx_t_5pycx4_5pycda_event *__pyx_v_ev1, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev2) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("cmp_events", 0);

  /* "pycx4/pycdabase.pxi":70
 * 
 * cdef inline int cmp_events(event *ev1, event *ev2):
 *     if ev1.evmask == ev2.evmask and ev1.evproc == ev2.evproc and \             # <<<<<<<<<<<<<<
 *         ev1.objptr == ev2.objptr and ev1.userptr == ev2.userptr:
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ev1->evmask == __pyx_v_ev2->evmask) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ev1->evproc == __pyx_v_ev2->evproc) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }

  /* "pycx4/pycdabase.pxi":71
 * cdef inline int cmp_events(event *ev1, event *ev2):
 *     if ev1.evmask == ev2.evmask and ev1.evproc == ev2.evproc and \
 *         ev1.objptr == ev2.objptr and ev1.userptr == ev2.userptr:             # <<<<<<<<<<<<<<
 *         return 0
 *     return 1
 */
  __pyx_t_2 = ((__pyx_v_ev1->objptr == __pyx_v_ev2->objptr) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ev1->userptr == __pyx_v_ev2->userptr) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;

  /* "pycx4/pycdabase.pxi":70
 * 
 * cdef inline int cmp_events(event *ev1, event *ev2):
 *     if ev1.evmask == ev2.evmask and ev1.evproc == ev2.evproc and \             # <<<<<<<<<<<<<<
 *         ev1.objptr == ev2.objptr and ev1.userptr == ev2.userptr:
 *         return 0
 */
  if (__pyx_t_1) {

    /* "pycx4/pycdabase.pxi":72
 *     if ev1.evmask == ev2.evmask and ev1.evproc == ev2.evproc and \
 *         ev1.objptr == ev2.objptr and ev1.userptr == ev2.userptr:
 *         return 0             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pycx4/pycdabase.pxi":70
 * 
 * cdef inline int cmp_events(event *ev1, event *ev2):
 *     if ev1.evmask == ev2.evmask and ev1.evproc == ev2.evproc and \             # <<<<<<<<<<<<<<
 *         ev1.objptr == ev2.objptr and ev1.userptr == ev2.userptr:
 *         return 0
 */
  }

  /* "pycx4/pycdabase.pxi":73
 *         ev1.objptr == ev2.objptr and ev1.userptr == ev2.userptr:
 *         return 0
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * # classes
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "pycx4/pycdabase.pxi":69
 *     return 1
 * 
 * cdef inline int cmp_events(event *ev1, event *ev2):             # <<<<<<<<<<<<<<
 *     if ev1.evmask == ev2.evmask and ev1.evproc == ev2.evproc and \
 *         ev1.objptr == ev2.objptr and ev1.userptr == ev2.userptr:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":82
 *         int evnum
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.events = NULL
 *         self.evnum = 0
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_10cda_object_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_10cda_object_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __pyx_r = __pyx_pf_5pycx4_5pycda_10cda_object___init__(((struct __pyx_obj_5pycx4_5pycda_cda_object *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_10cda_object___init__(struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/pycdabase.pxi":83
 * 
 *     def __init__(self):
 *         self.events = NULL             # <<<<<<<<<<<<<<
 *         self.evnum = 0
 * 
 */
  __pyx_v_self->events = NULL;

  /* "pycx4/pycdabase.pxi":84
 *     def __init__(self):
 *         self.events = NULL
 *         self.evnum = 0             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->evnum = 0;

  /* "pycx4/pycdabase.pxi":82
 *         int evnum
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self.events = NULL
 *         self.evnum = 0
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":86
 *         self.evnum = 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         free(self.events)
 * 
 */

/* Python wrapper */
static void __pyx_pw_5pycx4_5pycda_10cda_object_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pycx4_5pycda_10cda_object_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pycx4_5pycda_10cda_object_2__dealloc__(((struct __pyx_obj_5pycx4_5pycda_cda_object *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pycx4_5pycda_10cda_object_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pycx4/pycdabase.pxi":87
 * 
 *     def __dealloc__(self):
 *         free(self.events)             # <<<<<<<<<<<<<<
 * 
 *     cdef int add_event(self,int evmask, void *evproc, void *objptr, void *userptr):
 */
  free(__pyx_v_self->events);

  /* "pycx4/pycdabase.pxi":86
 *         self.evnum = 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         free(self.events)
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":89
 *         free(self.events)
 * 
 *     cdef int add_event(self,int evmask, void *evproc, void *objptr, void *userptr):             # <<<<<<<<<<<<<<
 *         cdef:
 *             event ev
 */

static int __pyx_f_5pycx4_5pycda_10cda_object_add_event(struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self, int __pyx_v_evmask, void *__pyx_v_evproc, void *__pyx_v_objptr, void *__pyx_v_userptr) {
  __pyx_t_5pycx4_5pycda_event __pyx_v_ev;
  void *__pyx_v_tmp;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  void *__pyx_t_2;
  void *__pyx_t_3;
  void *__pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_event", 0);

  /* "pycx4/pycdabase.pxi":93
 *             event ev
 *             void *tmp
 *         ev.evmask, ev.evproc, ev.objptr, ev.userptr = evmask, evproc, objptr, userptr             # <<<<<<<<<<<<<<
 * 
 *         if event_feasible(&ev) == 0 or self.search_event(&ev) > -1:
 */
  __pyx_t_1 = __pyx_v_evmask;
  __pyx_t_2 = __pyx_v_evproc;
  __pyx_t_3 = __pyx_v_objptr;
  __pyx_t_4 = __pyx_v_userptr;
  __pyx_v_ev.evmask = __pyx_t_1;
  __pyx_v_ev.evproc = __pyx_t_2;
  __pyx_v_ev.objptr = __pyx_t_3;
  __pyx_v_ev.userptr = __pyx_t_4;

  /* "pycx4/pycdabase.pxi":95
 *         ev.evmask, ev.evproc, ev.objptr, ev.userptr = evmask, evproc, objptr, userptr
 * 
 *         if event_feasible(&ev) == 0 or self.search_event(&ev) > -1:             # <<<<<<<<<<<<<<
 *             # event will not run or already exist
 *             return 0
 */
  __pyx_t_6 = ((__pyx_f_5pycx4_5pycda_event_feasible((&__pyx_v_ev)) == 0) != 0);
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = ((((struct __pyx_vtabstruct_5pycx4_5pycda_cda_object *)__pyx_v_self->__pyx_vtab)->search_event(__pyx_v_self, (&__pyx_v_ev)) > -1L) != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_5) {

    /* "pycx4/pycdabase.pxi":97
 *         if event_feasible(&ev) == 0 or self.search_event(&ev) > -1:
 *             # event will not run or already exist
 *             return 0             # <<<<<<<<<<<<<<
 *         tmp = realloc(self.events, sizeof(event) * (self.evnum + 1) )
 *         if not tmp: raise MemoryError()
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pycx4/pycdabase.pxi":95
 *         ev.evmask, ev.evproc, ev.objptr, ev.userptr = evmask, evproc, objptr, userptr
 * 
 *         if event_feasible(&ev) == 0 or self.search_event(&ev) > -1:             # <<<<<<<<<<<<<<
 *             # event will not run or already exist
 *             return 0
 */
  }

  /* "pycx4/pycdabase.pxi":98
 *             # event will not run or already exist
 *             return 0
 *         tmp = realloc(self.events, sizeof(event) * (self.evnum + 1) )             # <<<<<<<<<<<<<<
 *         if not tmp: raise MemoryError()
 *         self.events = <event*>tmp
 */
  __pyx_v_tmp = realloc(__pyx_v_self->events, ((sizeof(__pyx_t_5pycx4_5pycda_event)) * (__pyx_v_self->evnum + 1)));

  /* "pycx4/pycdabase.pxi":99
 *             return 0
 *         tmp = realloc(self.events, sizeof(event) * (self.evnum + 1) )
 *         if not tmp: raise MemoryError()             # <<<<<<<<<<<<<<
 *         self.events = <event*>tmp
 *         self.events[self.evnum] = ev
 */
  __pyx_t_5 = ((!(__pyx_v_tmp != 0)) != 0);
  if (__pyx_t_5) {
    PyErr_NoMemory(); {__pyx_filename = __pyx_f[1]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "pycx4/pycdabase.pxi":100
 *         tmp = realloc(self.events, sizeof(event) * (self.evnum + 1) )
 *         if not tmp: raise MemoryError()
 *         self.events = <event*>tmp             # <<<<<<<<<<<<<<
 *         self.events[self.evnum] = ev
 *         self.register_event(&self.events[self.evnum])
 */
  __pyx_v_self->events = ((__pyx_t_5pycx4_5pycda_event *)__pyx_v_tmp);

  /* "pycx4/pycdabase.pxi":101
 *         if not tmp: raise MemoryError()
 *         self.events = <event*>tmp
 *         self.events[self.evnum] = ev             # <<<<<<<<<<<<<<
 *         self.register_event(&self.events[self.evnum])
 *         self.evnum += 1
 */
  (__pyx_v_self->events[__pyx_v_self->evnum]) = __pyx_v_ev;

  /* "pycx4/pycdabase.pxi":102
 *         self.events = <event*>tmp
 *         self.events[self.evnum] = ev
 *         self.register_event(&self.events[self.evnum])             # <<<<<<<<<<<<<<
 *         self.evnum += 1
 *         return 1
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_object *)__pyx_v_self->__pyx_vtab)->register_event(__pyx_v_self, (&(__pyx_v_self->events[__pyx_v_self->evnum])));

  /* "pycx4/pycdabase.pxi":103
 *         self.events[self.evnum] = ev
 *         self.register_event(&self.events[self.evnum])
 *         self.evnum += 1             # <<<<<<<<<<<<<<
 *         return 1
 * 
 */
  __pyx_v_self->evnum = (__pyx_v_self->evnum + 1);

  /* "pycx4/pycdabase.pxi":104
 *         self.register_event(&self.events[self.evnum])
 *         self.evnum += 1
 *         return 1             # <<<<<<<<<<<<<<
 * 
 *     cdef int del_event(self, int evmask, void *evproc, void *objptr, void *userptr):
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "pycx4/pycdabase.pxi":89
 *         free(self.events)
 * 
 *     cdef int add_event(self,int evmask, void *evproc, void *objptr, void *userptr):             # <<<<<<<<<<<<<<
 *         cdef:
 *             event ev
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.cda_object.add_event", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":106
 *         return 1
 * 
 *     cdef int del_event(self, int evmask, void *evproc, void *objptr, void *userptr):             # <<<<<<<<<<<<<<
 *         cdef:
 *             event ev
 */

static int __pyx_f_5pycx4_5pycda_10cda_object_del_event(struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self, int __pyx_v_evmask, void *__pyx_v_evproc, void *__pyx_v_objptr, void *__pyx_v_userptr) {
  __pyx_t_5pycx4_5pycda_event __pyx_v_ev;
  int __pyx_v_ev_ind;
  void *__pyx_v_tmp;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  void *__pyx_t_2;
  void *__pyx_t_3;
  void *__pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("del_event", 0);

  /* "pycx4/pycdabase.pxi":111
 *             int ev_ind
 * 
 *         ev.evmask, ev.evproc, ev.objptr, ev.userptr = evmask, evproc, objptr, userptr             # <<<<<<<<<<<<<<
 *         ev_ind = self.search_event(&ev)
 *         if ev_ind < 0: # event does not exist
 */
  __pyx_t_1 = __pyx_v_evmask;
  __pyx_t_2 = __pyx_v_evproc;
  __pyx_t_3 = __pyx_v_objptr;
  __pyx_t_4 = __pyx_v_userptr;
  __pyx_v_ev.evmask = __pyx_t_1;
  __pyx_v_ev.evproc = __pyx_t_2;
  __pyx_v_ev.objptr = __pyx_t_3;
  __pyx_v_ev.userptr = __pyx_t_4;

  /* "pycx4/pycdabase.pxi":112
 * 
 *         ev.evmask, ev.evproc, ev.objptr, ev.userptr = evmask, evproc, objptr, userptr
 *         ev_ind = self.search_event(&ev)             # <<<<<<<<<<<<<<
 *         if ev_ind < 0: # event does not exist
 *             return 0
 */
  __pyx_v_ev_ind = ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_object *)__pyx_v_self->__pyx_vtab)->search_event(__pyx_v_self, (&__pyx_v_ev));

  /* "pycx4/pycdabase.pxi":113
 *         ev.evmask, ev.evproc, ev.objptr, ev.userptr = evmask, evproc, objptr, userptr
 *         ev_ind = self.search_event(&ev)
 *         if ev_ind < 0: # event does not exist             # <<<<<<<<<<<<<<
 *             return 0
 *         self.unregister_event(&(self.events[ev_ind]))
 */
  __pyx_t_5 = ((__pyx_v_ev_ind < 0) != 0);
  if (__pyx_t_5) {

    /* "pycx4/pycdabase.pxi":114
 *         ev_ind = self.search_event(&ev)
 *         if ev_ind < 0: # event does not exist
 *             return 0             # <<<<<<<<<<<<<<
 *         self.unregister_event(&(self.events[ev_ind]))
 *         if self.evnum == 1:
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "pycx4/pycdabase.pxi":113
 *         ev.evmask, ev.evproc, ev.objptr, ev.userptr = evmask, evproc, objptr, userptr
 *         ev_ind = self.search_event(&ev)
 *         if ev_ind < 0: # event does not exist             # <<<<<<<<<<<<<<
 *             return 0
 *         self.unregister_event(&(self.events[ev_ind]))
 */
  }

  /* "pycx4/pycdabase.pxi":115
 *         if ev_ind < 0: # event does not exist
 *             return 0
 *         self.unregister_event(&(self.events[ev_ind]))             # <<<<<<<<<<<<<<
 *         if self.evnum == 1:
 *             free(self.events)
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_object *)__pyx_v_self->__pyx_vtab)->unregister_event(__pyx_v_self, (&(__pyx_v_self->events[__pyx_v_ev_ind])));

  /* "pycx4/pycdabase.pxi":116
 *             return 0
 *         self.unregister_event(&(self.events[ev_ind]))
 *         if self.evnum == 1:             # <<<<<<<<<<<<<<
 *             free(self.events)
 *         else:
 */
  __pyx_t_5 = ((__pyx_v_self->evnum == 1) != 0);
  if (__pyx_t_5) {

    /* "pycx4/pycdabase.pxi":117
 *         self.unregister_event(&(self.events[ev_ind]))
 *         if self.evnum == 1:
 *             free(self.events)             # <<<<<<<<<<<<<<
 *         else:
 *             memmove(&(self.events[ev_ind]), &(self.events[self.evnum-1]), sizeof(event))
 */
    free(__pyx_v_self->events);

    /* "pycx4/pycdabase.pxi":116
 *             return 0
 *         self.unregister_event(&(self.events[ev_ind]))
 *         if self.evnum == 1:             # <<<<<<<<<<<<<<
 *             free(self.events)
 *         else:
 */
    goto __pyx_L4;
  }

  /* "pycx4/pycdabase.pxi":119
 *             free(self.events)
 *         else:
 *             memmove(&(self.events[ev_ind]), &(self.events[self.evnum-1]), sizeof(event))             # <<<<<<<<<<<<<<
 *             tmp = realloc(self.events, sizeof(event) * (self.evnum - 1))
 *             if not tmp: raise MemoryError()
 */
  /*else*/ {
    memmove((&(__pyx_v_self->events[__pyx_v_ev_ind])), (&(__pyx_v_self->events[(__pyx_v_self->evnum - 1)])), (sizeof(__pyx_t_5pycx4_5pycda_event)));

    /* "pycx4/pycdabase.pxi":120
 *         else:
 *             memmove(&(self.events[ev_ind]), &(self.events[self.evnum-1]), sizeof(event))
 *             tmp = realloc(self.events, sizeof(event) * (self.evnum - 1))             # <<<<<<<<<<<<<<
 *             if not tmp: raise MemoryError()
 *             self.events = <event*>tmp
 */
    __pyx_v_tmp = realloc(__pyx_v_self->events, ((sizeof(__pyx_t_5pycx4_5pycda_event)) * (__pyx_v_self->evnum - 1)));

    /* "pycx4/pycdabase.pxi":121
 *             memmove(&(self.events[ev_ind]), &(self.events[self.evnum-1]), sizeof(event))
 *             tmp = realloc(self.events, sizeof(event) * (self.evnum - 1))
 *             if not tmp: raise MemoryError()             # <<<<<<<<<<<<<<
 *             self.events = <event*>tmp
 *         self.evnum -= 1
 */
    __pyx_t_5 = ((!(__pyx_v_tmp != 0)) != 0);
    if (__pyx_t_5) {
      PyErr_NoMemory(); {__pyx_filename = __pyx_f[1]; __pyx_lineno = 121; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }

    /* "pycx4/pycdabase.pxi":122
 *             tmp = realloc(self.events, sizeof(event) * (self.evnum - 1))
 *             if not tmp: raise MemoryError()
 *             self.events = <event*>tmp             # <<<<<<<<<<<<<<
 *         self.evnum -= 1
 *         return 1
 */
    __pyx_v_self->events = ((__pyx_t_5pycx4_5pycda_event *)__pyx_v_tmp);
  }
  __pyx_L4:;

  /* "pycx4/pycdabase.pxi":123
 *             if not tmp: raise MemoryError()
 *             self.events = <event*>tmp
 *         self.evnum -= 1             # <<<<<<<<<<<<<<
 *         return 1
 * 
 */
  __pyx_v_self->evnum = (__pyx_v_self->evnum - 1);

  /* "pycx4/pycdabase.pxi":124
 *             self.events = <event*>tmp
 *         self.evnum -= 1
 *         return 1             # <<<<<<<<<<<<<<
 * 
 *     cdef int search_event(self, event *ev):
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "pycx4/pycdabase.pxi":106
 *         return 1
 * 
 *     cdef int del_event(self, int evmask, void *evproc, void *objptr, void *userptr):             # <<<<<<<<<<<<<<
 *         cdef:
 *             event ev
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.cda_object.del_event", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":126
 *         return 1
 * 
 *     cdef int search_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         for ind in range(self.evnum):
 *             if cmp_events(&self.events[ind], ev) == 0:
 */

static int __pyx_f_5pycx4_5pycda_10cda_object_search_event(struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  int __pyx_v_ind;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("search_event", 0);

  /* "pycx4/pycdabase.pxi":127
 * 
 *     cdef int search_event(self, event *ev):
 *         for ind in range(self.evnum):             # <<<<<<<<<<<<<<
 *             if cmp_events(&self.events[ind], ev) == 0:
 *                 return ind
 */
  __pyx_t_1 = __pyx_v_self->evnum;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_ind = __pyx_t_2;

    /* "pycx4/pycdabase.pxi":128
 *     cdef int search_event(self, event *ev):
 *         for ind in range(self.evnum):
 *             if cmp_events(&self.events[ind], ev) == 0:             # <<<<<<<<<<<<<<
 *                 return ind
 *         return -1
 */
    __pyx_t_3 = ((__pyx_f_5pycx4_5pycda_cmp_events((&(__pyx_v_self->events[__pyx_v_ind])), __pyx_v_ev) == 0) != 0);
    if (__pyx_t_3) {

      /* "pycx4/pycdabase.pxi":129
 *         for ind in range(self.evnum):
 *             if cmp_events(&self.events[ind], ev) == 0:
 *                 return ind             # <<<<<<<<<<<<<<
 *         return -1
 * 
 */
      __pyx_r = __pyx_v_ind;
      goto __pyx_L0;

      /* "pycx4/pycdabase.pxi":128
 *     cdef int search_event(self, event *ev):
 *         for ind in range(self.evnum):
 *             if cmp_events(&self.events[ind], ev) == 0:             # <<<<<<<<<<<<<<
 *                 return ind
 *         return -1
 */
    }
  }

  /* "pycx4/pycdabase.pxi":130
 *             if cmp_events(&self.events[ind], ev) == 0:
 *                 return ind
 *         return -1             # <<<<<<<<<<<<<<
 * 
 *     cdef void register_event(self, event *ev):
 */
  __pyx_r = -1;
  goto __pyx_L0;

  /* "pycx4/pycdabase.pxi":126
 *         return 1
 * 
 *     cdef int search_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         for ind in range(self.evnum):
 *             if cmp_events(&self.events[ind], ev) == 0:
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":132
 *         return -1
 * 
 *     cdef void register_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         return
 * 
 */

static void __pyx_f_5pycx4_5pycda_10cda_object_register_event(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self, CYTHON_UNUSED __pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("register_event", 0);

  /* "pycx4/pycdabase.pxi":133
 * 
 *     cdef void register_event(self, event *ev):
 *         return             # <<<<<<<<<<<<<<
 * 
 *     cdef void unregister_event(self, event *ev):
 */
  goto __pyx_L0;

  /* "pycx4/pycdabase.pxi":132
 *         return -1
 * 
 *     cdef void register_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         return
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":135
 *         return
 * 
 *     cdef void unregister_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         return
 * 
 */

static void __pyx_f_5pycx4_5pycda_10cda_object_unregister_event(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_cda_object *__pyx_v_self, CYTHON_UNUSED __pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("unregister_event", 0);

  /* "pycx4/pycdabase.pxi":136
 * 
 *     cdef void unregister_event(self, event *ev):
 *         return             # <<<<<<<<<<<<<<
 * 
 * 
 */
  goto __pyx_L0;

  /* "pycx4/pycdabase.pxi":135
 *         return
 * 
 *     cdef void unregister_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         return
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":155
 *             public object serverCycle
 * 
 *     def __init__(self, defpfx="cx::"):             # <<<<<<<<<<<<<<
 *         cda_object.__init__(self)
 *         cdef:
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_11cda_context_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_11cda_context_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_defpfx = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_defpfx,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_kp_s_cx);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defpfx);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_defpfx = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[1]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.cda_context.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_11cda_context___init__(((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_v_self), __pyx_v_defpfx);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_11cda_context___init__(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self, PyObject *__pyx_v_defpfx) {
  int __pyx_v_ret;
  char *__pyx_v_c_defpfx;
  PyObject *__pyx_v_ascii_pfx = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  char *__pyx_t_5;
  int __pyx_t_6;
  void *__pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/pycdabase.pxi":156
 * 
 *     def __init__(self, defpfx="cx::"):
 *         cda_object.__init__(self)             # <<<<<<<<<<<<<<
 *         cdef:
 *             int ret
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_cda_object), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 156; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 156; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 156; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_self));
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 156; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":160
 *             int ret
 *             char *c_defpfx
 *         ascii_pfx = defpfx.encode("ascii") # encode to ascii             # <<<<<<<<<<<<<<
 *         c_defpfx = ascii_pfx  # convert to char*
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_defpfx, __pyx_n_s_encode); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ascii_pfx = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pycx4/pycdabase.pxi":161
 *             char *c_defpfx
 *         ascii_pfx = defpfx.encode("ascii") # encode to ascii
 *         c_defpfx = ascii_pfx  # convert to char*             # <<<<<<<<<<<<<<
 * 
 *         ret = cda_new_context(0, NULL, c_defpfx, 0, NULL, 0, <cda_context_evproc_t>NULL, NULL)
 */
  __pyx_t_5 = __Pyx_PyObject_AsString(__pyx_v_ascii_pfx); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 161; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_c_defpfx = __pyx_t_5;

  /* "pycx4/pycdabase.pxi":163
 *         c_defpfx = ascii_pfx  # convert to char*
 * 
 *         ret = cda_new_context(0, NULL, c_defpfx, 0, NULL, 0, <cda_context_evproc_t>NULL, NULL)             # <<<<<<<<<<<<<<
 *         cda_check_exception(ret)
 *         self.cid, self.defpfx, self.chans, self.channum = ret, defpfx, NULL, 0
 */
  __pyx_v_ret = cda_new_context(0, NULL, __pyx_v_c_defpfx, 0, NULL, 0, ((cda_context_evproc_t)NULL), NULL);

  /* "pycx4/pycdabase.pxi":164
 * 
 *         ret = cda_new_context(0, NULL, c_defpfx, 0, NULL, 0, <cda_context_evproc_t>NULL, NULL)
 *         cda_check_exception(ret)             # <<<<<<<<<<<<<<
 *         self.cid, self.defpfx, self.chans, self.channum = ret, defpfx, NULL, 0
 * 
 */
  __pyx_t_6 = __pyx_f_5pycx4_5pycda_cda_check_exception(__pyx_v_ret); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "pycx4/pycdabase.pxi":165
 *         ret = cda_new_context(0, NULL, c_defpfx, 0, NULL, 0, <cda_context_evproc_t>NULL, NULL)
 *         cda_check_exception(ret)
 *         self.cid, self.defpfx, self.chans, self.channum = ret, defpfx, NULL, 0             # <<<<<<<<<<<<<<
 * 
 *         IF SIGNAL_IMPL=='cda_signal':
 */
  __pyx_t_6 = __pyx_v_ret;
  if (!(likely(PyString_CheckExact(__pyx_v_defpfx))||((__pyx_v_defpfx) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_defpfx)->tp_name), 0))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 165; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __pyx_v_defpfx;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_7 = NULL;
  __pyx_t_8 = 0;
  __pyx_v_self->cid = __pyx_t_6;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->defpfx);
  __Pyx_DECREF(__pyx_v_self->defpfx);
  __pyx_v_self->defpfx = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_v_self->chans = __pyx_t_7;
  __pyx_v_self->channum = __pyx_t_8;

  /* "pycx4/pycdabase.pxi":168
 * 
 *         IF SIGNAL_IMPL=='cda_signal':
 *             self.serverCycle = cda_signal()             # <<<<<<<<<<<<<<
 *         ELIF SIGNAL_IMPL=='pyqtSignal':
 *             self.signaler = ContSignaler()
 */
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_cda_signal), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->serverCycle);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->serverCycle));
  __pyx_v_self->serverCycle = ((struct __pyx_obj_5pycx4_5pycda_cda_signal *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pycx4/pycdabase.pxi":155
 *             public object serverCycle
 * 
 *     def __init__(self, defpfx="cx::"):             # <<<<<<<<<<<<<<
 *         cda_object.__init__(self)
 *         cdef:
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.cda_context.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ascii_pfx);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":173
 *             self.serverCycle = self.signaler.serverCycle
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.cid > 0:
 *             cda_check_exception( cda_del_context(self.cid) )
 */

/* Python wrapper */
static void __pyx_pw_5pycx4_5pycda_11cda_context_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pycx4_5pycda_11cda_context_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pycx4_5pycda_11cda_context_2__dealloc__(((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pycx4_5pycda_11cda_context_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pycx4/pycdabase.pxi":174
 * 
 *     def __dealloc__(self):
 *         if self.cid > 0:             # <<<<<<<<<<<<<<
 *             cda_check_exception( cda_del_context(self.cid) )
 *             self.cid = 0
 */
  __pyx_t_1 = ((__pyx_v_self->cid > 0) != 0);
  if (__pyx_t_1) {

    /* "pycx4/pycdabase.pxi":175
 *     def __dealloc__(self):
 *         if self.cid > 0:
 *             cda_check_exception( cda_del_context(self.cid) )             # <<<<<<<<<<<<<<
 *             self.cid = 0
 *         free(self.chans)
 */
    __pyx_t_2 = __pyx_f_5pycx4_5pycda_cda_check_exception(cda_del_context(__pyx_v_self->cid)); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "pycx4/pycdabase.pxi":176
 *         if self.cid > 0:
 *             cda_check_exception( cda_del_context(self.cid) )
 *             self.cid = 0             # <<<<<<<<<<<<<<
 *         free(self.chans)
 *         self.channum = 0
 */
    __pyx_v_self->cid = 0;

    /* "pycx4/pycdabase.pxi":174
 * 
 *     def __dealloc__(self):
 *         if self.cid > 0:             # <<<<<<<<<<<<<<
 *             cda_check_exception( cda_del_context(self.cid) )
 *             self.cid = 0
 */
  }

  /* "pycx4/pycdabase.pxi":177
 *             cda_check_exception( cda_del_context(self.cid) )
 *             self.cid = 0
 *         free(self.chans)             # <<<<<<<<<<<<<<
 *         self.channum = 0
 * 
 */
  free(__pyx_v_self->chans);

  /* "pycx4/pycdabase.pxi":178
 *             self.cid = 0
 *         free(self.chans)
 *         self.channum = 0             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_v_self->channum = 0;

  /* "pycx4/pycdabase.pxi":173
 *             self.serverCycle = self.signaler.serverCycle
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.cid > 0:
 *             cda_check_exception( cda_del_context(self.cid) )
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.cda_context.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":180
 *         self.channum = 0
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return '<cda_context: cid=%d, defpfx=%s, channum=%d>' % (self.cid, self.defpfx, self.channum)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_11cda_context_5__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_11cda_context_5__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_11cda_context_4__str__(((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_11cda_context_4__str__(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "pycx4/pycdabase.pxi":181
 * 
 *     def __str__(self):
 *         return '<cda_context: cid=%d, defpfx=%s, channum=%d>' % (self.cid, self.defpfx, self.channum)             # <<<<<<<<<<<<<<
 * 
 *     def enable_serverCycle(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_cda_context_t(__pyx_v_self->cid); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->channum); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->defpfx);
  __Pyx_GIVEREF(__pyx_v_self->defpfx);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_self->defpfx);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_cda_context_cid_d_defpfx_s_chan, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pycx4/pycdabase.pxi":180
 *         self.channum = 0
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return '<cda_context: cid=%d, defpfx=%s, channum=%d>' % (self.cid, self.defpfx, self.channum)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pycx4.pycda.cda_context.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":183
 *         return '<cda_context: cid=%d, defpfx=%s, channum=%d>' % (self.cid, self.defpfx, self.channum)
 * 
 *     def enable_serverCycle(self):             # <<<<<<<<<<<<<<
 *         self.add_event(CDA_CTX_EVMASK_CYCLE, <void*>evproc_cont_cycle, <void*>self, NULL)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_11cda_context_7enable_serverCycle(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_11cda_context_7enable_serverCycle(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("enable_serverCycle (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_11cda_context_6enable_serverCycle(((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_11cda_context_6enable_serverCycle(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("enable_serverCycle", 0);

  /* "pycx4/pycdabase.pxi":184
 * 
 *     def enable_serverCycle(self):
 *         self.add_event(CDA_CTX_EVMASK_CYCLE, <void*>evproc_cont_cycle, <void*>self, NULL)             # <<<<<<<<<<<<<<
 * 
 *     cdef void save_chan(self, void *chan):
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_context *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.add_event(((struct __pyx_obj_5pycx4_5pycda_cda_object *)__pyx_v_self), CDA_CTX_EVMASK_CYCLE, ((void *)__pyx_f_5pycx4_5pycda_evproc_cont_cycle), ((void *)__pyx_v_self), NULL);

  /* "pycx4/pycdabase.pxi":183
 *         return '<cda_context: cid=%d, defpfx=%s, channum=%d>' % (self.cid, self.defpfx, self.channum)
 * 
 *     def enable_serverCycle(self):             # <<<<<<<<<<<<<<
 *         self.add_event(CDA_CTX_EVMASK_CYCLE, <void*>evproc_cont_cycle, <void*>self, NULL)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":186
 *         self.add_event(CDA_CTX_EVMASK_CYCLE, <void*>evproc_cont_cycle, <void*>self, NULL)
 * 
 *     cdef void save_chan(self, void *chan):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum+1))
 */

static void __pyx_f_5pycx4_5pycda_11cda_context_save_chan(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self, void *__pyx_v_chan) {
  void *__pyx_v_tmp;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("save_chan", 0);

  /* "pycx4/pycdabase.pxi":188
 *     cdef void save_chan(self, void *chan):
 *         cdef:
 *             void *tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum+1))             # <<<<<<<<<<<<<<
 *         if not tmp: raise MemoryError()
 *         self.chans = <void**>tmp
 */
  __pyx_v_tmp = realloc(((void *)__pyx_v_self->chans), ((sizeof(void *)) * (__pyx_v_self->channum + 1)));

  /* "pycx4/pycdabase.pxi":189
 *         cdef:
 *             void *tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum+1))
 *         if not tmp: raise MemoryError()             # <<<<<<<<<<<<<<
 *         self.chans = <void**>tmp
 *         self.chans[self.channum] = chan
 */
  __pyx_t_1 = ((!(__pyx_v_tmp != 0)) != 0);
  if (__pyx_t_1) {
    PyErr_NoMemory(); {__pyx_filename = __pyx_f[1]; __pyx_lineno = 189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "pycx4/pycdabase.pxi":190
 *             void *tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum+1))
 *         if not tmp: raise MemoryError()
 *         self.chans = <void**>tmp             # <<<<<<<<<<<<<<
 *         self.chans[self.channum] = chan
 *         self.channum += 1
 */
  __pyx_v_self->chans = ((void **)__pyx_v_tmp);

  /* "pycx4/pycdabase.pxi":191
 *         if not tmp: raise MemoryError()
 *         self.chans = <void**>tmp
 *         self.chans[self.channum] = chan             # <<<<<<<<<<<<<<
 *         self.channum += 1
 * 
 */
  (__pyx_v_self->chans[__pyx_v_self->channum]) = __pyx_v_chan;

  /* "pycx4/pycdabase.pxi":192
 *         self.chans = <void**>tmp
 *         self.chans[self.channum] = chan
 *         self.channum += 1             # <<<<<<<<<<<<<<
 * 
 *     cdef void drop_chan(self, void *chan):
 */
  __pyx_v_self->channum = (__pyx_v_self->channum + 1);

  /* "pycx4/pycdabase.pxi":186
 *         self.add_event(CDA_CTX_EVMASK_CYCLE, <void*>evproc_cont_cycle, <void*>self, NULL)
 * 
 *     cdef void save_chan(self, void *chan):             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum+1))
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.cda_context.save_chan", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":194
 *         self.channum += 1
 * 
 *     cdef void drop_chan(self, void *chan):             # <<<<<<<<<<<<<<
 *         if not self.cid: return
 *         cdef:
 */

static void __pyx_f_5pycx4_5pycda_11cda_context_drop_chan(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self, void *__pyx_v_chan) {
  void *__pyx_v_tmp;
  int __pyx_v_ind;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("drop_chan", 0);

  /* "pycx4/pycdabase.pxi":195
 * 
 *     cdef void drop_chan(self, void *chan):
 *         if not self.cid: return             # <<<<<<<<<<<<<<
 *         cdef:
 *             void *tmp
 */
  __pyx_t_1 = ((!(__pyx_v_self->cid != 0)) != 0);
  if (__pyx_t_1) {
    goto __pyx_L0;
  }

  /* "pycx4/pycdabase.pxi":199
 *             void *tmp
 *             int ind
 *         for ind in range(self.channum):             # <<<<<<<<<<<<<<
 *             if chan == self.chans[ind]:
 *                 if self.channum == 1:
 */
  __pyx_t_2 = __pyx_v_self->channum;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_ind = __pyx_t_3;

    /* "pycx4/pycdabase.pxi":200
 *             int ind
 *         for ind in range(self.channum):
 *             if chan == self.chans[ind]:             # <<<<<<<<<<<<<<
 *                 if self.channum == 1:
 *                     free(self.chans)
 */
    __pyx_t_1 = ((__pyx_v_chan == (__pyx_v_self->chans[__pyx_v_ind])) != 0);
    if (__pyx_t_1) {

      /* "pycx4/pycdabase.pxi":201
 *         for ind in range(self.channum):
 *             if chan == self.chans[ind]:
 *                 if self.channum == 1:             # <<<<<<<<<<<<<<
 *                     free(self.chans)
 *                 else:
 */
      __pyx_t_1 = ((__pyx_v_self->channum == 1) != 0);
      if (__pyx_t_1) {

        /* "pycx4/pycdabase.pxi":202
 *             if chan == self.chans[ind]:
 *                 if self.channum == 1:
 *                     free(self.chans)             # <<<<<<<<<<<<<<
 *                 else:
 *                     memmove(&(self.chans[ind]), &(self.chans[self.channum-1]), sizeof(void*) )
 */
        free(__pyx_v_self->chans);

        /* "pycx4/pycdabase.pxi":201
 *         for ind in range(self.channum):
 *             if chan == self.chans[ind]:
 *                 if self.channum == 1:             # <<<<<<<<<<<<<<
 *                     free(self.chans)
 *                 else:
 */
        goto __pyx_L7;
      }

      /* "pycx4/pycdabase.pxi":204
 *                     free(self.chans)
 *                 else:
 *                     memmove(&(self.chans[ind]), &(self.chans[self.channum-1]), sizeof(void*) )             # <<<<<<<<<<<<<<
 *                     tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum-1))
 *                     if not tmp and self.channum>1: raise MemoryError()
 */
      /*else*/ {
        memmove((&(__pyx_v_self->chans[__pyx_v_ind])), (&(__pyx_v_self->chans[(__pyx_v_self->channum - 1)])), (sizeof(void *)));

        /* "pycx4/pycdabase.pxi":205
 *                 else:
 *                     memmove(&(self.chans[ind]), &(self.chans[self.channum-1]), sizeof(void*) )
 *                     tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum-1))             # <<<<<<<<<<<<<<
 *                     if not tmp and self.channum>1: raise MemoryError()
 *                     self.chans = <void**>tmp
 */
        __pyx_v_tmp = realloc(((void *)__pyx_v_self->chans), ((sizeof(void *)) * (__pyx_v_self->channum - 1)));

        /* "pycx4/pycdabase.pxi":206
 *                     memmove(&(self.chans[ind]), &(self.chans[self.channum-1]), sizeof(void*) )
 *                     tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum-1))
 *                     if not tmp and self.channum>1: raise MemoryError()             # <<<<<<<<<<<<<<
 *                     self.chans = <void**>tmp
 *                 self.channum -= 1
 */
        __pyx_t_4 = ((!(__pyx_v_tmp != 0)) != 0);
        if (__pyx_t_4) {
        } else {
          __pyx_t_1 = __pyx_t_4;
          goto __pyx_L9_bool_binop_done;
        }
        __pyx_t_4 = ((__pyx_v_self->channum > 1) != 0);
        __pyx_t_1 = __pyx_t_4;
        __pyx_L9_bool_binop_done:;
        if (__pyx_t_1) {
          PyErr_NoMemory(); {__pyx_filename = __pyx_f[1]; __pyx_lineno = 206; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }

        /* "pycx4/pycdabase.pxi":207
 *                     tmp = realloc(<void*>self.chans, sizeof(void*) * (self.channum-1))
 *                     if not tmp and self.channum>1: raise MemoryError()
 *                     self.chans = <void**>tmp             # <<<<<<<<<<<<<<
 *                 self.channum -= 1
 *                 return
 */
        __pyx_v_self->chans = ((void **)__pyx_v_tmp);
      }
      __pyx_L7:;

      /* "pycx4/pycdabase.pxi":208
 *                     if not tmp and self.channum>1: raise MemoryError()
 *                     self.chans = <void**>tmp
 *                 self.channum -= 1             # <<<<<<<<<<<<<<
 *                 return
 * 
 */
      __pyx_v_self->channum = (__pyx_v_self->channum - 1);

      /* "pycx4/pycdabase.pxi":209
 *                     self.chans = <void**>tmp
 *                 self.channum -= 1
 *                 return             # <<<<<<<<<<<<<<
 * 
 *     cdef void register_event(self, event *ev):
 */
      goto __pyx_L0;

      /* "pycx4/pycdabase.pxi":200
 *             int ind
 *         for ind in range(self.channum):
 *             if chan == self.chans[ind]:             # <<<<<<<<<<<<<<
 *                 if self.channum == 1:
 *                     free(self.chans)
 */
    }
  }

  /* "pycx4/pycdabase.pxi":194
 *         self.channum += 1
 * 
 *     cdef void drop_chan(self, void *chan):             # <<<<<<<<<<<<<<
 *         if not self.cid: return
 *         cdef:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.cda_context.drop_chan", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":211
 *                 return
 * 
 *     cdef void register_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_add_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 */

static void __pyx_f_5pycx4_5pycda_11cda_context_register_event(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("register_event", 0);

  /* "pycx4/pycdabase.pxi":212
 * 
 *     cdef void register_event(self, event *ev):
 *         cda_check_exception( cda_add_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )             # <<<<<<<<<<<<<<
 * 
 *     cdef void unregister_event(self, event *ev):
 */
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_cda_check_exception(cda_add_context_evproc(__pyx_v_self->cid, __pyx_v_ev->evmask, ((cda_context_evproc_t)__pyx_v_ev->evproc), __pyx_v_ev)); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 212; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "pycx4/pycdabase.pxi":211
 *                 return
 * 
 *     cdef void register_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_add_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.cda_context.register_event", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":214
 *         cda_check_exception( cda_add_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 *     cdef void unregister_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_del_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 */

static void __pyx_f_5pycx4_5pycda_11cda_context_unregister_event(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("unregister_event", 0);

  /* "pycx4/pycdabase.pxi":215
 * 
 *     cdef void unregister_event(self, event *ev):
 *         cda_check_exception( cda_del_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_cda_check_exception(cda_del_context_evproc(__pyx_v_self->cid, __pyx_v_ev->evmask, ((cda_context_evproc_t)__pyx_v_ev->evproc), __pyx_v_ev)); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "pycx4/pycdabase.pxi":214
 *         cda_check_exception( cda_add_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 *     cdef void unregister_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_del_context_evproc(self.cid, ev.evmask, <cda_context_evproc_t>ev.evproc, ev) )
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.cda_context.unregister_event", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":141
 * cdef class cda_context(cda_object):
 *     cdef readonly:
 *         cda_context_t cid             # <<<<<<<<<<<<<<
 *         str defpfx
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_11cda_context_3cid_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_11cda_context_3cid_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_11cda_context_3cid___get__(((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_11cda_context_3cid___get__(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_cda_context_t(__pyx_v_self->cid); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 141; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.cda_context.cid.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":142
 *     cdef readonly:
 *         cda_context_t cid
 *         str defpfx             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_11cda_context_6defpfx_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_11cda_context_6defpfx_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_11cda_context_6defpfx___get__(((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_11cda_context_6defpfx___get__(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->defpfx);
  __pyx_r = __pyx_v_self->defpfx;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":149
 *     IF SIGNAL_IMPL=='cda_signal':
 *         cdef readonly:
 *             cda_signal serverCycle             # <<<<<<<<<<<<<<
 *     ELIF SIGNAL_IMPL=='pyqtSignal':
 *         cdef:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_11cda_context_11serverCycle_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_11cda_context_11serverCycle_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_11cda_context_11serverCycle___get__(((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_11cda_context_11serverCycle___get__(struct __pyx_obj_5pycx4_5pycda_cda_context *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->serverCycle));
  __pyx_r = ((PyObject *)__pyx_v_self->serverCycle);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":246
 *             public object valueMeasured
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE, int max_nelems=1):             # <<<<<<<<<<<<<<
 *         cda_object.__init__(self)
 *         if isinstance(context, cda_context): self.context = <void*>context
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_13cda_base_chan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_13cda_base_chan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_context = 0;
  cxdtype_t __pyx_v_dtype;
  int __pyx_v_max_nelems;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_context,&__pyx_n_s_dtype,&__pyx_n_s_max_nelems,0};
    PyObject* values[4] = {0,0,0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_context);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dtype);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_max_nelems);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 246; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_name = ((PyObject*)values[0]);
    __pyx_v_context = values[1];
    if (values[2]) {
      __pyx_v_dtype = __Pyx_PyInt_As_uint8(values[2]); if (unlikely((__pyx_v_dtype == (cxdtype_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 246; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_dtype = __pyx_k__3;
    }
    if (values[3]) {
      __pyx_v_max_nelems = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_max_nelems == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 246; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_max_nelems = ((int)1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[1]; __pyx_lineno = 246; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.cda_base_chan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyString_Type), 1, "name", 1))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 246; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5pycx4_5pycda_13cda_base_chan___init__(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self), __pyx_v_name, __pyx_v_context, __pyx_v_dtype, __pyx_v_max_nelems);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_13cda_base_chan___init__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_context, cxdtype_t __pyx_v_dtype, int __pyx_v_max_nelems) {
  PyObject *__pyx_v_b_name = NULL;
  char *__pyx_v_c_name;
  int __pyx_v_ret;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  char *__pyx_t_7;
  int __pyx_t_8;
  cxdtype_t __pyx_t_9;
  int __pyx_t_10;
  size_t __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/pycdabase.pxi":247
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE, int max_nelems=1):
 *         cda_object.__init__(self)             # <<<<<<<<<<<<<<
 *         if isinstance(context, cda_context): self.context = <void*>context
 *         else: self.context = <void*>default_context
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_cda_object), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_self));
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":248
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE, int max_nelems=1):
 *         cda_object.__init__(self)
 *         if isinstance(context, cda_context): self.context = <void*>context             # <<<<<<<<<<<<<<
 *         else: self.context = <void*>default_context
 * 
 */
  __pyx_t_5 = __Pyx_TypeCheck(__pyx_v_context, __pyx_ptype_5pycx4_5pycda_cda_context); 
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {
    __pyx_v_self->context = ((void *)__pyx_v_context);
    goto __pyx_L3;
  }

  /* "pycx4/pycdabase.pxi":249
 *         cda_object.__init__(self)
 *         if isinstance(context, cda_context): self.context = <void*>context
 *         else: self.context = <void*>default_context             # <<<<<<<<<<<<<<
 * 
 *         IF SIGNAL_IMPL=='cda_signal':
 */
  /*else*/ {
    __pyx_v_self->context = ((void *)__pyx_v_5pycx4_5pycda_default_context);
  }
  __pyx_L3:;

  /* "pycx4/pycdabase.pxi":252
 * 
 *         IF SIGNAL_IMPL=='cda_signal':
 *             self.valueMeasured = cda_signal()             # <<<<<<<<<<<<<<
 *             self.valueChanged = cda_signal()
 *         ELIF SIGNAL_IMPL=='pyqtSignal':
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_cda_signal), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 252; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->valueMeasured);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->valueMeasured));
  __pyx_v_self->valueMeasured = ((struct __pyx_obj_5pycx4_5pycda_cda_signal *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":253
 *         IF SIGNAL_IMPL=='cda_signal':
 *             self.valueMeasured = cda_signal()
 *             self.valueChanged = cda_signal()             # <<<<<<<<<<<<<<
 *         ELIF SIGNAL_IMPL=='pyqtSignal':
 *             self.signaler = ChanSignaler()
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_cda_signal), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 253; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->valueChanged);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->valueChanged));
  __pyx_v_self->valueChanged = ((struct __pyx_obj_5pycx4_5pycda_cda_signal *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":259
 *             self.valueMeasured = self.signaler.valueMeasured
 * 
 *         b_name = name.encode("ascii")             # <<<<<<<<<<<<<<
 *         cdef:
 *             char *c_name = b_name
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_name, __pyx_n_s_encode); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_b_name = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pycx4/pycdabase.pxi":261
 *         b_name = name.encode("ascii")
 *         cdef:
 *             char *c_name = b_name             # <<<<<<<<<<<<<<
 *             int ret
 * 
 */
  __pyx_t_7 = __Pyx_PyObject_AsString(__pyx_v_b_name); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 261; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_c_name = __pyx_t_7;

  /* "pycx4/pycdabase.pxi":264
 *             int ret
 * 
 *         ret = cda_add_chan((<cda_context>self.context).cid, NULL, c_name, 0, dtype, max_nelems,             # <<<<<<<<<<<<<<
 *                                 0, <cda_dataref_evproc_t>NULL, NULL)
 *         cda_check_exception(ret)
 */
  __pyx_v_ret = cda_add_chan(((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_v_self->context)->cid, NULL, __pyx_v_c_name, 0, __pyx_v_dtype, __pyx_v_max_nelems, 0, ((cda_dataref_evproc_t)NULL), NULL);

  /* "pycx4/pycdabase.pxi":266
 *         ret = cda_add_chan((<cda_context>self.context).cid, NULL, c_name, 0, dtype, max_nelems,
 *                                 0, <cda_dataref_evproc_t>NULL, NULL)
 *         cda_check_exception(ret)             # <<<<<<<<<<<<<<
 *         self.ref, self.name, self.dtype, self.max_nelems, self.first_cycle, self.itemsize =\
 *             ret, name, dtype, max_nelems, True, sizeof_cxdtype(dtype)
 */
  __pyx_t_8 = __pyx_f_5pycx4_5pycda_cda_check_exception(__pyx_v_ret); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 266; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "pycx4/pycdabase.pxi":268
 *         cda_check_exception(ret)
 *         self.ref, self.name, self.dtype, self.max_nelems, self.first_cycle, self.itemsize =\
 *             ret, name, dtype, max_nelems, True, sizeof_cxdtype(dtype)             # <<<<<<<<<<<<<<
 * 
 *         (<cda_context>self.context).save_chan(<void*>self)
 */
  __pyx_t_8 = __pyx_v_ret;
  __pyx_t_2 = __pyx_v_name;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_9 = __pyx_v_dtype;
  __pyx_t_10 = __pyx_v_max_nelems;
  __pyx_t_6 = 1;
  __pyx_t_11 = sizeof_cxdtype(__pyx_v_dtype);

  /* "pycx4/pycdabase.pxi":267
 *                                 0, <cda_dataref_evproc_t>NULL, NULL)
 *         cda_check_exception(ret)
 *         self.ref, self.name, self.dtype, self.max_nelems, self.first_cycle, self.itemsize =\             # <<<<<<<<<<<<<<
 *             ret, name, dtype, max_nelems, True, sizeof_cxdtype(dtype)
 * 
 */
  __pyx_v_self->ref = __pyx_t_8;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_v_self->dtype = __pyx_t_9;
  __pyx_v_self->max_nelems = __pyx_t_10;
  __pyx_v_self->first_cycle = __pyx_t_6;
  __pyx_v_self->itemsize = __pyx_t_11;

  /* "pycx4/pycdabase.pxi":270
 *             ret, name, dtype, max_nelems, True, sizeof_cxdtype(dtype)
 * 
 *         (<cda_context>self.context).save_chan(<void*>self)             # <<<<<<<<<<<<<<
 * 
 *         self.add_event(CDA_REF_EVMASK_RSLVSTAT, <void*>evproc_rslvstat, <void*>self, NULL)
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_context *)((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_v_self->context)->__pyx_base.__pyx_vtab)->save_chan(((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_v_self->context), ((void *)__pyx_v_self));

  /* "pycx4/pycdabase.pxi":272
 *         (<cda_context>self.context).save_chan(<void*>self)
 * 
 *         self.add_event(CDA_REF_EVMASK_RSLVSTAT, <void*>evproc_rslvstat, <void*>self, NULL)             # <<<<<<<<<<<<<<
 *         self.add_event(CDA_REF_EVMASK_UPDATE,   <void*>evproc_update,   <void*>self, NULL)
 *         self.registered = 1
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_base_chan *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.add_event(((struct __pyx_obj_5pycx4_5pycda_cda_object *)__pyx_v_self), CDA_REF_EVMASK_RSLVSTAT, ((void *)__pyx_f_5pycx4_5pycda_evproc_rslvstat), ((void *)__pyx_v_self), NULL);

  /* "pycx4/pycdabase.pxi":273
 * 
 *         self.add_event(CDA_REF_EVMASK_RSLVSTAT, <void*>evproc_rslvstat, <void*>self, NULL)
 *         self.add_event(CDA_REF_EVMASK_UPDATE,   <void*>evproc_update,   <void*>self, NULL)             # <<<<<<<<<<<<<<
 *         self.registered = 1
 * 
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_base_chan *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.add_event(((struct __pyx_obj_5pycx4_5pycda_cda_object *)__pyx_v_self), CDA_REF_EVMASK_UPDATE, ((void *)__pyx_f_5pycx4_5pycda_evproc_update), ((void *)__pyx_v_self), NULL);

  /* "pycx4/pycdabase.pxi":274
 *         self.add_event(CDA_REF_EVMASK_RSLVSTAT, <void*>evproc_rslvstat, <void*>self, NULL)
 *         self.add_event(CDA_REF_EVMASK_UPDATE,   <void*>evproc_update,   <void*>self, NULL)
 *         self.registered = 1             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->registered = 1;

  /* "pycx4/pycdabase.pxi":246
 *             public object valueMeasured
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE, int max_nelems=1):             # <<<<<<<<<<<<<<
 *         cda_object.__init__(self)
 *         if isinstance(context, cda_context): self.context = <void*>context
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pycx4.pycda.cda_base_chan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_b_name);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":276
 *         self.registered = 1
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         cda_del_chan(self.ref)
 *         if self.registered:
 */

/* Python wrapper */
static void __pyx_pw_5pycx4_5pycda_13cda_base_chan_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pycx4_5pycda_13cda_base_chan_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pycx4_5pycda_13cda_base_chan_2__dealloc__(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pycx4_5pycda_13cda_base_chan_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pycx4/pycdabase.pxi":277
 * 
 *     def __dealloc__(self):
 *         cda_del_chan(self.ref)             # <<<<<<<<<<<<<<
 *         if self.registered:
 *             (<cda_context>self.context).drop_chan(<void*>self)
 */
  cda_del_chan(__pyx_v_self->ref);

  /* "pycx4/pycdabase.pxi":278
 *     def __dealloc__(self):
 *         cda_del_chan(self.ref)
 *         if self.registered:             # <<<<<<<<<<<<<<
 *             (<cda_context>self.context).drop_chan(<void*>self)
 * 
 */
  __pyx_t_1 = (__pyx_v_self->registered != 0);
  if (__pyx_t_1) {

    /* "pycx4/pycdabase.pxi":279
 *         cda_del_chan(self.ref)
 *         if self.registered:
 *             (<cda_context>self.context).drop_chan(<void*>self)             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
    ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_context *)((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_v_self->context)->__pyx_base.__pyx_vtab)->drop_chan(((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_v_self->context), ((void *)__pyx_v_self));

    /* "pycx4/pycdabase.pxi":278
 *     def __dealloc__(self):
 *         cda_del_chan(self.ref)
 *         if self.registered:             # <<<<<<<<<<<<<<
 *             (<cda_context>self.context).drop_chan(<void*>self)
 * 
 */
  }

  /* "pycx4/pycdabase.pxi":276
 *         self.registered = 1
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         cda_del_chan(self.ref)
 *         if self.registered:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":281
 *             (<cda_context>self.context).drop_chan(<void*>self)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return '<cda_channel: ref=%d, name=%s>' % (self.ref, self.name)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_5__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_5__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_13cda_base_chan_4__str__(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_4__str__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "pycx4/pycdabase.pxi":282
 * 
 *     def __str__(self):
 *         return '<cda_channel: ref=%d, name=%s>' % (self.ref, self.name)             # <<<<<<<<<<<<<<
 * 
 *     cdef void cb(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_CxDataRef_t(__pyx_v_self->ref); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 282; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 282; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->name);
  __Pyx_GIVEREF(__pyx_v_self->name);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self->name);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_cda_channel_ref_d_name_s, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 282; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pycx4/pycdabase.pxi":281
 *             (<cda_context>self.context).drop_chan(<void*>self)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return '<cda_channel: ref=%d, name=%s>' % (self.ref, self.name)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pycx4.pycda.cda_base_chan.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":284
 *         return '<cda_channel: ref=%d, name=%s>' % (self.ref, self.name)
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         #empty callback for overrideing
 *         pass
 */

static void __pyx_f_5pycx4_5pycda_13cda_base_chan_cb(CYTHON_UNUSED struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cb", 0);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":288
 *         pass
 * 
 *     cdef void snd_data(self, cxdtype_t dtype, int nelems, void* data_p):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_snd_ref_data(self.ref, dtype, nelems, data_p) )
 * 
 */

static void __pyx_f_5pycx4_5pycda_13cda_base_chan_snd_data(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self, cxdtype_t __pyx_v_dtype, int __pyx_v_nelems, void *__pyx_v_data_p) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("snd_data", 0);

  /* "pycx4/pycdabase.pxi":289
 * 
 *     cdef void snd_data(self, cxdtype_t dtype, int nelems, void* data_p):
 *         cda_check_exception( cda_snd_ref_data(self.ref, dtype, nelems, data_p) )             # <<<<<<<<<<<<<<
 * 
 *     cdef void get_data(self, size_t ofs, size_t size, void* buf):
 */
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_cda_check_exception(cda_snd_ref_data(__pyx_v_self->ref, __pyx_v_dtype, __pyx_v_nelems, __pyx_v_data_p)); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 289; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "pycx4/pycdabase.pxi":288
 *         pass
 * 
 *     cdef void snd_data(self, cxdtype_t dtype, int nelems, void* data_p):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_snd_ref_data(self.ref, dtype, nelems, data_p) )
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.cda_base_chan.snd_data", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":291
 *         cda_check_exception( cda_snd_ref_data(self.ref, dtype, nelems, data_p) )
 * 
 *     cdef void get_data(self, size_t ofs, size_t size, void* buf):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_get_ref_data(self.ref, ofs, size, buf) )
 * 
 */

static void __pyx_f_5pycx4_5pycda_13cda_base_chan_get_data(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self, size_t __pyx_v_ofs, size_t __pyx_v_size, void *__pyx_v_buf) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_data", 0);

  /* "pycx4/pycdabase.pxi":292
 * 
 *     cdef void get_data(self, size_t ofs, size_t size, void* buf):
 *         cda_check_exception( cda_get_ref_data(self.ref, ofs, size, buf) )             # <<<<<<<<<<<<<<
 * 
 *     cdef int current_nelems(self):
 */
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_cda_check_exception(cda_get_ref_data(__pyx_v_self->ref, __pyx_v_ofs, __pyx_v_size, __pyx_v_buf)); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "pycx4/pycdabase.pxi":291
 *         cda_check_exception( cda_snd_ref_data(self.ref, dtype, nelems, data_p) )
 * 
 *     cdef void get_data(self, size_t ofs, size_t size, void* buf):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_get_ref_data(self.ref, ofs, size, buf) )
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.cda_base_chan.get_data", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":294
 *         cda_check_exception( cda_get_ref_data(self.ref, ofs, size, buf) )
 * 
 *     cdef int current_nelems(self):             # <<<<<<<<<<<<<<
 *         return cda_current_nelems_of_ref(self.ref)
 * 
 */

static int __pyx_f_5pycx4_5pycda_13cda_base_chan_current_nelems(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("current_nelems", 0);

  /* "pycx4/pycdabase.pxi":295
 * 
 *     cdef int current_nelems(self):
 *         return cda_current_nelems_of_ref(self.ref)             # <<<<<<<<<<<<<<
 * 
 *     cdef void get_src(self, const char **src_p):
 */
  __pyx_r = cda_current_nelems_of_ref(__pyx_v_self->ref);
  goto __pyx_L0;

  /* "pycx4/pycdabase.pxi":294
 *         cda_check_exception( cda_get_ref_data(self.ref, ofs, size, buf) )
 * 
 *     cdef int current_nelems(self):             # <<<<<<<<<<<<<<
 *         return cda_current_nelems_of_ref(self.ref)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":297
 *         return cda_current_nelems_of_ref(self.ref)
 * 
 *     cdef void get_src(self, const char **src_p):             # <<<<<<<<<<<<<<
 *         cda_check_exception(cda_src_of_ref(self.ref, src_p))
 * 
 */

static void __pyx_f_5pycx4_5pycda_13cda_base_chan_get_src(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self, char const **__pyx_v_src_p) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_src", 0);

  /* "pycx4/pycdabase.pxi":298
 * 
 *     cdef void get_src(self, const char **src_p):
 *         cda_check_exception(cda_src_of_ref(self.ref, src_p))             # <<<<<<<<<<<<<<
 * 
 *     # overriding cda_object methods
 */
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_cda_check_exception(cda_src_of_ref(__pyx_v_self->ref, __pyx_v_src_p)); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "pycx4/pycdabase.pxi":297
 *         return cda_current_nelems_of_ref(self.ref)
 * 
 *     cdef void get_src(self, const char **src_p):             # <<<<<<<<<<<<<<
 *         cda_check_exception(cda_src_of_ref(self.ref, src_p))
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.cda_base_chan.get_src", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":301
 * 
 *     # overriding cda_object methods
 *     cdef void register_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_add_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 */

static void __pyx_f_5pycx4_5pycda_13cda_base_chan_register_event(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("register_event", 0);

  /* "pycx4/pycdabase.pxi":302
 *     # overriding cda_object methods
 *     cdef void register_event(self, event *ev):
 *         cda_check_exception( cda_add_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )             # <<<<<<<<<<<<<<
 * 
 *     cdef void unregister_event(self, event *ev):
 */
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_cda_check_exception(cda_add_dataref_evproc(__pyx_v_self->ref, __pyx_v_ev->evmask, ((cda_dataref_evproc_t)__pyx_v_ev->evproc), __pyx_v_ev)); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "pycx4/pycdabase.pxi":301
 * 
 *     # overriding cda_object methods
 *     cdef void register_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_add_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.cda_base_chan.register_event", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":304
 *         cda_check_exception( cda_add_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 *     cdef void unregister_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_del_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 */

static void __pyx_f_5pycx4_5pycda_13cda_base_chan_unregister_event(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self, __pyx_t_5pycx4_5pycda_event *__pyx_v_ev) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("unregister_event", 0);

  /* "pycx4/pycdabase.pxi":305
 * 
 *     cdef void unregister_event(self, event *ev):
 *         cda_check_exception( cda_del_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_cda_check_exception(cda_del_dataref_evproc(__pyx_v_self->ref, __pyx_v_ev->evmask, ((cda_dataref_evproc_t)__pyx_v_ev->evproc), __pyx_v_ev)); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 305; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "pycx4/pycdabase.pxi":304
 *         cda_check_exception( cda_add_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 *     cdef void unregister_event(self, event *ev):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_del_dataref_evproc(self.ref, ev.evmask, <cda_dataref_evproc_t>ev.evproc, ev) )
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.cda_base_chan.unregister_event", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdabase.pxi":223
 * cdef class cda_base_chan(cda_object):
 *     cdef readonly:
 *         cda_dataref_t ref             # <<<<<<<<<<<<<<
 *         str name
 *         int max_nelems
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_3ref_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_3ref_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_13cda_base_chan_3ref___get__(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_3ref___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_CxDataRef_t(__pyx_v_self->ref); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.cda_base_chan.ref.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":224
 *     cdef readonly:
 *         cda_dataref_t ref
 *         str name             # <<<<<<<<<<<<<<
 *         int max_nelems
 *         cxdtype_t dtype
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_4name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_4name_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_13cda_base_chan_4name___get__(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_4name___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":225
 *         cda_dataref_t ref
 *         str name
 *         int max_nelems             # <<<<<<<<<<<<<<
 *         cxdtype_t dtype
 *         size_t itemsize
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_10max_nelems_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_10max_nelems_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_13cda_base_chan_10max_nelems___get__(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_10max_nelems___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->max_nelems); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 225; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.cda_base_chan.max_nelems.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":226
 *         str name
 *         int max_nelems
 *         cxdtype_t dtype             # <<<<<<<<<<<<<<
 *         size_t itemsize
 *         int64 time, prev_time
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_5dtype_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_5dtype_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_13cda_base_chan_5dtype___get__(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_5dtype___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_uint8(__pyx_v_self->dtype); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 226; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.cda_base_chan.dtype.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":227
 *         int max_nelems
 *         cxdtype_t dtype
 *         size_t itemsize             # <<<<<<<<<<<<<<
 *         int64 time, prev_time
 *         int found # -1 - not found, 0 - unknown, 1 found
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_8itemsize_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_8itemsize_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_13cda_base_chan_8itemsize___get__(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_8itemsize___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->itemsize); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.cda_base_chan.itemsize.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":228
 *         cxdtype_t dtype
 *         size_t itemsize
 *         int64 time, prev_time             # <<<<<<<<<<<<<<
 *         int found # -1 - not found, 0 - unknown, 1 found
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_4time_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_4time_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_13cda_base_chan_4time___get__(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_4time___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int64(__pyx_v_self->time); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 228; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.cda_base_chan.time.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_9prev_time_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_9prev_time_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_13cda_base_chan_9prev_time___get__(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_9prev_time___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int64(__pyx_v_self->prev_time); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 228; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.cda_base_chan.prev_time.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":229
 *         size_t itemsize
 *         int64 time, prev_time
 *         int found # -1 - not found, 0 - unknown, 1 found             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_5found_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_5found_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_13cda_base_chan_5found___get__(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_5found___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->found); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 229; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.cda_base_chan.found.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":238
 *     IF SIGNAL_IMPL=='cda_signal':
 *         cdef readonly:
 *             cda_signal valueMeasured             # <<<<<<<<<<<<<<
 *             cda_signal valueChanged
 *     ELIF SIGNAL_IMPL=='pyqtSignal':
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_13valueMeasured_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_13valueMeasured_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_13cda_base_chan_13valueMeasured___get__(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_13valueMeasured___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->valueMeasured));
  __pyx_r = ((PyObject *)__pyx_v_self->valueMeasured);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdabase.pxi":239
 *         cdef readonly:
 *             cda_signal valueMeasured
 *             cda_signal valueChanged             # <<<<<<<<<<<<<<
 *     ELIF SIGNAL_IMPL=='pyqtSignal':
 *         cdef:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_12valueChanged_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_13cda_base_chan_12valueChanged_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_13cda_base_chan_12valueChanged___get__(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_13cda_base_chan_12valueChanged___get__(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->valueChanged));
  __pyx_r = ((PyObject *)__pyx_v_self->valueChanged);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":11
 *         readonly double val, prev_val, tolerance
 * 
 *     def __init__(self, str name, object context=None):             # <<<<<<<<<<<<<<
 *         cda_base_chan.__init__(self, name, context)
 *         self.tolerance = 0.0
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_6sdchan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_6sdchan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_context,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_context);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 11; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_name = ((PyObject*)values[0]);
    __pyx_v_context = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 11; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.sdchan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyString_Type), 1, "name", 1))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 11; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5pycx4_5pycda_6sdchan___init__(((struct __pyx_obj_5pycx4_5pycda_sdchan *)__pyx_v_self), __pyx_v_name, __pyx_v_context);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_6sdchan___init__(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_context) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/pycdauser.pxi":12
 * 
 *     def __init__(self, str name, object context=None):
 *         cda_base_chan.__init__(self, name, context)             # <<<<<<<<<<<<<<
 *         self.tolerance = 0.0
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_cda_base_chan), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 12; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  __pyx_t_5 = PyTuple_New(3+__pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 12; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_name);
  __Pyx_INCREF(__pyx_v_context);
  __Pyx_GIVEREF(__pyx_v_context);
  PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, __pyx_v_context);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 12; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdauser.pxi":13
 *     def __init__(self, str name, object context=None):
 *         cda_base_chan.__init__(self, name, context)
 *         self.tolerance = 0.0             # <<<<<<<<<<<<<<
 * 
 *     cdef void cb(self):
 */
  __pyx_v_self->tolerance = 0.0;

  /* "pycx4/pycdauser.pxi":11
 *         readonly double val, prev_val, tolerance
 * 
 *     def __init__(self, str name, object context=None):             # <<<<<<<<<<<<<<
 *         cda_base_chan.__init__(self, name, context)
 *         self.tolerance = 0.0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.sdchan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":15
 *         self.tolerance = 0.0
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         self.prev_val = self.val
 *         cda_check_exception(cda_get_dcval(self.ref, &self.val))
 */

static void __pyx_f_5pycx4_5pycda_6sdchan_cb(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cb", 0);

  /* "pycx4/pycdauser.pxi":16
 * 
 *     cdef void cb(self):
 *         self.prev_val = self.val             # <<<<<<<<<<<<<<
 *         cda_check_exception(cda_get_dcval(self.ref, &self.val))
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:
 */
  __pyx_t_1 = __pyx_v_self->val;
  __pyx_v_self->prev_val = __pyx_t_1;

  /* "pycx4/pycdauser.pxi":17
 *     cdef void cb(self):
 *         self.prev_val = self.val
 *         cda_check_exception(cda_get_dcval(self.ref, &self.val))             # <<<<<<<<<<<<<<
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:
 *             self.valueChanged.emit(self)
 */
  __pyx_t_2 = __pyx_f_5pycx4_5pycda_cda_check_exception(cda_get_dcval(__pyx_v_self->__pyx_base.ref, (&__pyx_v_self->val))); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 17; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "pycx4/pycdauser.pxi":18
 *         self.prev_val = self.val
 *         cda_check_exception(cda_get_dcval(self.ref, &self.val))
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:             # <<<<<<<<<<<<<<
 *             self.valueChanged.emit(self)
 *             self.first_cycle = False
 */
  __pyx_t_4 = ((fabs((__pyx_v_self->val - __pyx_v_self->prev_val)) > __pyx_v_self->tolerance) != 0);
  if (!__pyx_t_4) {
  } else {
    __pyx_t_3 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = (__pyx_v_self->__pyx_base.first_cycle != 0);
  __pyx_t_3 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "pycx4/pycdauser.pxi":19
 *         cda_check_exception(cda_get_dcval(self.ref, &self.val))
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:
 *             self.valueChanged.emit(self)             # <<<<<<<<<<<<<<
 *             self.first_cycle = False
 *         self.valueMeasured.emit(self)
 */
    __pyx_t_5 = ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_signal *)__pyx_v_self->__pyx_base.valueChanged->__pyx_vtab)->emit(__pyx_v_self->__pyx_base.valueChanged, ((PyObject *)__pyx_v_self), 0); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 19; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "pycx4/pycdauser.pxi":20
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:
 *             self.valueChanged.emit(self)
 *             self.first_cycle = False             # <<<<<<<<<<<<<<
 *         self.valueMeasured.emit(self)
 * 
 */
    __pyx_v_self->__pyx_base.first_cycle = 0;

    /* "pycx4/pycdauser.pxi":18
 *         self.prev_val = self.val
 *         cda_check_exception(cda_get_dcval(self.ref, &self.val))
 *         if abs(self.val - self.prev_val) > self.tolerance or self.first_cycle:             # <<<<<<<<<<<<<<
 *             self.valueChanged.emit(self)
 *             self.first_cycle = False
 */
  }

  /* "pycx4/pycdauser.pxi":21
 *             self.valueChanged.emit(self)
 *             self.first_cycle = False
 *         self.valueMeasured.emit(self)             # <<<<<<<<<<<<<<
 * 
 *     cpdef setValue(self, double value):
 */
  __pyx_t_5 = ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_signal *)__pyx_v_self->__pyx_base.valueMeasured->__pyx_vtab)->emit(__pyx_v_self->__pyx_base.valueMeasured, ((PyObject *)__pyx_v_self), 0); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "pycx4/pycdauser.pxi":15
 *         self.tolerance = 0.0
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         self.prev_val = self.val
 *         cda_check_exception(cda_get_dcval(self.ref, &self.val))
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("pycx4.pycda.sdchan.cb", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdauser.pxi":23
 *         self.valueMeasured.emit(self)
 * 
 *     cpdef setValue(self, double value):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_set_dcval(self.ref, value) )
 * 
 */

static PyObject *__pyx_pw_5pycx4_5pycda_6sdchan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_6sdchan_setValue(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self, double __pyx_v_value, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setValue", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setValue); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5pycx4_5pycda_6sdchan_3setValue)) {
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      if (!__pyx_t_5) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pycx4/pycdauser.pxi":24
 * 
 *     cpdef setValue(self, double value):
 *         cda_check_exception( cda_set_dcval(self.ref, value) )             # <<<<<<<<<<<<<<
 * 
 *     cpdef setTolerance(self, double new_tolerance):
 */
  __pyx_t_7 = __pyx_f_5pycx4_5pycda_cda_check_exception(cda_set_dcval(__pyx_v_self->__pyx_base.ref, __pyx_v_value)); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 24; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "pycx4/pycdauser.pxi":23
 *         self.valueMeasured.emit(self)
 * 
 *     cpdef setValue(self, double value):             # <<<<<<<<<<<<<<
 *         cda_check_exception( cda_set_dcval(self.ref, value) )
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pycx4.pycda.sdchan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6sdchan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_arg_value); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6sdchan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_arg_value) {
  double __pyx_v_value;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setValue (wrapper)", 0);
  assert(__pyx_arg_value); {
    __pyx_v_value = __pyx_PyFloat_AsDouble(__pyx_arg_value); if (unlikely((__pyx_v_value == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.sdchan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_6sdchan_2setValue(((struct __pyx_obj_5pycx4_5pycda_sdchan *)__pyx_v_self), ((double)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6sdchan_2setValue(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self, double __pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setValue", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_6sdchan_setValue(__pyx_v_self, __pyx_v_value, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 23; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.sdchan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":26
 *         cda_check_exception( cda_set_dcval(self.ref, value) )
 * 
 *     cpdef setTolerance(self, double new_tolerance):             # <<<<<<<<<<<<<<
 *         self.tolerance = new_tolerance
 * 
 */

static PyObject *__pyx_pw_5pycx4_5pycda_6sdchan_5setTolerance(PyObject *__pyx_v_self, PyObject *__pyx_arg_new_tolerance); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_6sdchan_setTolerance(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self, double __pyx_v_new_tolerance, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setTolerance", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setTolerance); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5pycx4_5pycda_6sdchan_5setTolerance)) {
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_new_tolerance); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      if (!__pyx_t_5) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pycx4/pycdauser.pxi":27
 * 
 *     cpdef setTolerance(self, double new_tolerance):
 *         self.tolerance = new_tolerance             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_self->tolerance = __pyx_v_new_tolerance;

  /* "pycx4/pycdauser.pxi":26
 *         cda_check_exception( cda_set_dcval(self.ref, value) )
 * 
 *     cpdef setTolerance(self, double new_tolerance):             # <<<<<<<<<<<<<<
 *         self.tolerance = new_tolerance
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pycx4.pycda.sdchan.setTolerance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6sdchan_5setTolerance(PyObject *__pyx_v_self, PyObject *__pyx_arg_new_tolerance); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6sdchan_5setTolerance(PyObject *__pyx_v_self, PyObject *__pyx_arg_new_tolerance) {
  double __pyx_v_new_tolerance;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setTolerance (wrapper)", 0);
  assert(__pyx_arg_new_tolerance); {
    __pyx_v_new_tolerance = __pyx_PyFloat_AsDouble(__pyx_arg_new_tolerance); if (unlikely((__pyx_v_new_tolerance == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.sdchan.setTolerance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5pycx4_5pycda_6sdchan_4setTolerance(((struct __pyx_obj_5pycx4_5pycda_sdchan *)__pyx_v_self), ((double)__pyx_v_new_tolerance));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6sdchan_4setTolerance(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self, double __pyx_v_new_tolerance) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setTolerance", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_6sdchan_setTolerance(__pyx_v_self, __pyx_v_new_tolerance, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 26; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.sdchan.setTolerance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":9
 *     cdef:
 *         # all general properties defined in base classes
 *         readonly double val, prev_val, tolerance             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, str name, object context=None):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6sdchan_3val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6sdchan_3val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6sdchan_3val___get__(((struct __pyx_obj_5pycx4_5pycda_sdchan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6sdchan_3val___get__(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->val); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.sdchan.val.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6sdchan_8prev_val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6sdchan_8prev_val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6sdchan_8prev_val___get__(((struct __pyx_obj_5pycx4_5pycda_sdchan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6sdchan_8prev_val___get__(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->prev_val); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.sdchan.prev_val.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_6sdchan_9tolerance_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_6sdchan_9tolerance_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_6sdchan_9tolerance___get__(((struct __pyx_obj_5pycx4_5pycda_sdchan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_6sdchan_9tolerance___get__(struct __pyx_obj_5pycx4_5pycda_sdchan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->tolerance); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.sdchan.tolerance.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":36
 *         int allocated
 * 
 *     def __init__(self, str name, object context=None, int max_nelems=1024):             # <<<<<<<<<<<<<<
 *         cda_base_chan.__init__(self, name, context, CXDTYPE_TEXT, max_nelems)
 *         self.cval = <char*>malloc(max_nelems)
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_7strchan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_7strchan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_context = 0;
  int __pyx_v_max_nelems;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_context,&__pyx_n_s_max_nelems,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_context);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_max_nelems);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 36; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_name = ((PyObject*)values[0]);
    __pyx_v_context = values[1];
    if (values[2]) {
      __pyx_v_max_nelems = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_max_nelems == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 36; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_max_nelems = ((int)0x400);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 36; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.strchan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyString_Type), 1, "name", 1))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 36; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5pycx4_5pycda_7strchan___init__(((struct __pyx_obj_5pycx4_5pycda_strchan *)__pyx_v_self), __pyx_v_name, __pyx_v_context, __pyx_v_max_nelems);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_7strchan___init__(struct __pyx_obj_5pycx4_5pycda_strchan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_context, int __pyx_v_max_nelems) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/pycdauser.pxi":37
 * 
 *     def __init__(self, str name, object context=None, int max_nelems=1024):
 *         cda_base_chan.__init__(self, name, context, CXDTYPE_TEXT, max_nelems)             # <<<<<<<<<<<<<<
 *         self.cval = <char*>malloc(max_nelems)
 *         if not self.cval: raise MemoryError()
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_cda_base_chan), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(CXDTYPE_TEXT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_max_nelems); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  __pyx_t_7 = PyTuple_New(5+__pyx_t_6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  if (__pyx_t_5) {
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
  }
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_v_name);
  __Pyx_INCREF(__pyx_v_context);
  __Pyx_GIVEREF(__pyx_v_context);
  PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_v_context);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_7, 3+__pyx_t_6, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_7, 4+__pyx_t_6, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 37; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdauser.pxi":38
 *     def __init__(self, str name, object context=None, int max_nelems=1024):
 *         cda_base_chan.__init__(self, name, context, CXDTYPE_TEXT, max_nelems)
 *         self.cval = <char*>malloc(max_nelems)             # <<<<<<<<<<<<<<
 *         if not self.cval: raise MemoryError()
 *         self.allocated = 1
 */
  __pyx_v_self->cval = ((char *)malloc(__pyx_v_max_nelems));

  /* "pycx4/pycdauser.pxi":39
 *         cda_base_chan.__init__(self, name, context, CXDTYPE_TEXT, max_nelems)
 *         self.cval = <char*>malloc(max_nelems)
 *         if not self.cval: raise MemoryError()             # <<<<<<<<<<<<<<
 *         self.allocated = 1
 * 
 */
  __pyx_t_8 = ((!(__pyx_v_self->cval != 0)) != 0);
  if (__pyx_t_8) {
    PyErr_NoMemory(); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 39; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "pycx4/pycdauser.pxi":40
 *         self.cval = <char*>malloc(max_nelems)
 *         if not self.cval: raise MemoryError()
 *         self.allocated = 1             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->allocated = 1;

  /* "pycx4/pycdauser.pxi":36
 *         int allocated
 * 
 *     def __init__(self, str name, object context=None, int max_nelems=1024):             # <<<<<<<<<<<<<<
 *         cda_base_chan.__init__(self, name, context, CXDTYPE_TEXT, max_nelems)
 *         self.cval = <char*>malloc(max_nelems)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pycx4.pycda.strchan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":42
 *         self.allocated = 1
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.allocated:
 *             free(self.cval)
 */

/* Python wrapper */
static void __pyx_pw_5pycx4_5pycda_7strchan_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5pycx4_5pycda_7strchan_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5pycx4_5pycda_7strchan_2__dealloc__(((struct __pyx_obj_5pycx4_5pycda_strchan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5pycx4_5pycda_7strchan_2__dealloc__(struct __pyx_obj_5pycx4_5pycda_strchan *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "pycx4/pycdauser.pxi":43
 * 
 *     def __dealloc__(self):
 *         if self.allocated:             # <<<<<<<<<<<<<<
 *             free(self.cval)
 * 
 */
  __pyx_t_1 = (__pyx_v_self->allocated != 0);
  if (__pyx_t_1) {

    /* "pycx4/pycdauser.pxi":44
 *     def __dealloc__(self):
 *         if self.allocated:
 *             free(self.cval)             # <<<<<<<<<<<<<<
 * 
 *     cdef void cb(self):
 */
    free(__pyx_v_self->cval);

    /* "pycx4/pycdauser.pxi":43
 * 
 *     def __dealloc__(self):
 *         if self.allocated:             # <<<<<<<<<<<<<<
 *             free(self.cval)
 * 
 */
  }

  /* "pycx4/pycdauser.pxi":42
 *         self.allocated = 1
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.allocated:
 *             free(self.cval)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdauser.pxi":46
 *             free(self.cval)
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         length = cda_current_nelems_of_ref(self.ref)
 *         self.get_data(0, self.max_nelems, <void*>self.cval)
 */

static void __pyx_f_5pycx4_5pycda_7strchan_cb(struct __pyx_obj_5pycx4_5pycda_strchan *__pyx_v_self) {
  CYTHON_UNUSED int __pyx_v_length;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cb", 0);

  /* "pycx4/pycdauser.pxi":47
 * 
 *     cdef void cb(self):
 *         length = cda_current_nelems_of_ref(self.ref)             # <<<<<<<<<<<<<<
 *         self.get_data(0, self.max_nelems, <void*>self.cval)
 *         self.val = <bytes>self.cval
 */
  __pyx_v_length = cda_current_nelems_of_ref(__pyx_v_self->__pyx_base.ref);

  /* "pycx4/pycdauser.pxi":48
 *     cdef void cb(self):
 *         length = cda_current_nelems_of_ref(self.ref)
 *         self.get_data(0, self.max_nelems, <void*>self.cval)             # <<<<<<<<<<<<<<
 *         self.val = <bytes>self.cval
 *         self.valueMeasured.emit(self)
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_strchan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.get_data(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self), 0, __pyx_v_self->__pyx_base.max_nelems, ((void *)__pyx_v_self->cval));

  /* "pycx4/pycdauser.pxi":49
 *         length = cda_current_nelems_of_ref(self.ref)
 *         self.get_data(0, self.max_nelems, <void*>self.cval)
 *         self.val = <bytes>self.cval             # <<<<<<<<<<<<<<
 *         self.valueMeasured.emit(self)
 *         self.valueChanged.emit(self)
 */
  __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->cval); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 49; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->val);
  __Pyx_DECREF(__pyx_v_self->val);
  __pyx_v_self->val = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pycx4/pycdauser.pxi":50
 *         self.get_data(0, self.max_nelems, <void*>self.cval)
 *         self.val = <bytes>self.cval
 *         self.valueMeasured.emit(self)             # <<<<<<<<<<<<<<
 *         self.valueChanged.emit(self)
 * 
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_signal *)__pyx_v_self->__pyx_base.valueMeasured->__pyx_vtab)->emit(__pyx_v_self->__pyx_base.valueMeasured, ((PyObject *)__pyx_v_self), 0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/pycdauser.pxi":51
 *         self.val = <bytes>self.cval
 *         self.valueMeasured.emit(self)
 *         self.valueChanged.emit(self)             # <<<<<<<<<<<<<<
 * 
 *     cpdef setValue(self, bytes value):
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_signal *)__pyx_v_self->__pyx_base.valueChanged->__pyx_vtab)->emit(__pyx_v_self->__pyx_base.valueChanged, ((PyObject *)__pyx_v_self), 0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 51; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pycx4/pycdauser.pxi":46
 *             free(self.cval)
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         length = cda_current_nelems_of_ref(self.ref)
 *         self.get_data(0, self.max_nelems, <void*>self.cval)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("pycx4.pycda.strchan.cb", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdauser.pxi":53
 *         self.valueChanged.emit(self)
 * 
 *     cpdef setValue(self, bytes value):             # <<<<<<<<<<<<<<
 *         cdef char *v = value
 *         self.snd_data(CXDTYPE_TEXT, len(value), <void*>v)
 */

static PyObject *__pyx_pw_5pycx4_5pycda_7strchan_5setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_7strchan_setValue(struct __pyx_obj_5pycx4_5pycda_strchan *__pyx_v_self, PyObject *__pyx_v_value, int __pyx_skip_dispatch) {
  char *__pyx_v_v;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  char *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setValue", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setValue); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5pycx4_5pycda_7strchan_5setValue)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_value); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_value);
        __Pyx_GIVEREF(__pyx_v_value);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_value);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pycx4/pycdauser.pxi":54
 * 
 *     cpdef setValue(self, bytes value):
 *         cdef char *v = value             # <<<<<<<<<<<<<<
 *         self.snd_data(CXDTYPE_TEXT, len(value), <void*>v)
 * 
 */
  __pyx_t_6 = __Pyx_PyObject_AsString(__pyx_v_value); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_v = __pyx_t_6;

  /* "pycx4/pycdauser.pxi":55
 *     cpdef setValue(self, bytes value):
 *         cdef char *v = value
 *         self.snd_data(CXDTYPE_TEXT, len(value), <void*>v)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (unlikely(__pyx_v_value == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 55; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_7 = PyBytes_GET_SIZE(__pyx_v_value); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 55; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  ((struct __pyx_vtabstruct_5pycx4_5pycda_strchan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.snd_data(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self), CXDTYPE_TEXT, __pyx_t_7, ((void *)__pyx_v_v));

  /* "pycx4/pycdauser.pxi":53
 *         self.valueChanged.emit(self)
 * 
 *     cpdef setValue(self, bytes value):             # <<<<<<<<<<<<<<
 *         cdef char *v = value
 *         self.snd_data(CXDTYPE_TEXT, len(value), <void*>v)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.strchan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7strchan_5setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7strchan_5setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED int __pyx_lineno = 0;
  CYTHON_UNUSED const char *__pyx_filename = NULL;
  CYTHON_UNUSED int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setValue (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), (&PyBytes_Type), 1, "value", 1))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5pycx4_5pycda_7strchan_4setValue(((struct __pyx_obj_5pycx4_5pycda_strchan *)__pyx_v_self), ((PyObject*)__pyx_v_value));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7strchan_4setValue(struct __pyx_obj_5pycx4_5pycda_strchan *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setValue", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_7strchan_setValue(__pyx_v_self, __pyx_v_value, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.strchan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":32
 * cdef class strchan(cda_base_chan):
 *     cdef:
 *         readonly bytes val             # <<<<<<<<<<<<<<
 *         char *cval
 *         int allocated
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_7strchan_3val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_7strchan_3val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_7strchan_3val___get__(((struct __pyx_obj_5pycx4_5pycda_strchan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_7strchan_3val___get__(struct __pyx_obj_5pycx4_5pycda_strchan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->val);
  __pyx_r = __pyx_v_self->val;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":63
 * # not to effective, but not a bottleneck
 * cdef:
 *     aval_value(CxAnyVal_t *aval, cxdtype_t dtype):             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_DOUBLE: return aval.f64
 *         if dtype == CXDTYPE_INT32:  return aval.i32
 */

static PyObject *__pyx_f_5pycx4_5pycda_aval_value(CxAnyVal_t *__pyx_v_aval, cxdtype_t __pyx_v_dtype) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("aval_value", 0);

  /* "pycx4/pycdauser.pxi":64
 * cdef:
 *     aval_value(CxAnyVal_t *aval, cxdtype_t dtype):
 *         if dtype == CXDTYPE_DOUBLE: return aval.f64             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_INT32:  return aval.i32
 *         if dtype == CXDTYPE_INT8:   return aval.i8
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_DOUBLE) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_aval->f64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 64; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":65
 *     aval_value(CxAnyVal_t *aval, cxdtype_t dtype):
 *         if dtype == CXDTYPE_DOUBLE: return aval.f64
 *         if dtype == CXDTYPE_INT32:  return aval.i32             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_INT8:   return aval.i8
 *         if dtype == CXDTYPE_INT16:  return aval.i16
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_INT32) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_int32(__pyx_v_aval->i32); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 65; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":66
 *         if dtype == CXDTYPE_DOUBLE: return aval.f64
 *         if dtype == CXDTYPE_INT32:  return aval.i32
 *         if dtype == CXDTYPE_INT8:   return aval.i8             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_INT16:  return aval.i16
 *         if dtype == CXDTYPE_INT64:  return aval.i64
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_INT8) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_int8(__pyx_v_aval->i8); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 66; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":67
 *         if dtype == CXDTYPE_INT32:  return aval.i32
 *         if dtype == CXDTYPE_INT8:   return aval.i8
 *         if dtype == CXDTYPE_INT16:  return aval.i16             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_INT64:  return aval.i64
 *         if dtype == CXDTYPE_UINT8:  return aval.u8
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_INT16) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_int16(__pyx_v_aval->i16); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 67; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":68
 *         if dtype == CXDTYPE_INT8:   return aval.i8
 *         if dtype == CXDTYPE_INT16:  return aval.i16
 *         if dtype == CXDTYPE_INT64:  return aval.i64             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UINT8:  return aval.u8
 *         if dtype == CXDTYPE_UINT16: return aval.u16
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_INT64) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_int64(__pyx_v_aval->i64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 68; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":69
 *         if dtype == CXDTYPE_INT16:  return aval.i16
 *         if dtype == CXDTYPE_INT64:  return aval.i64
 *         if dtype == CXDTYPE_UINT8:  return aval.u8             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UINT16: return aval.u16
 *         if dtype == CXDTYPE_UINT32: return aval.u32
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UINT8) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_uint8(__pyx_v_aval->u8); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 69; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":70
 *         if dtype == CXDTYPE_INT64:  return aval.i64
 *         if dtype == CXDTYPE_UINT8:  return aval.u8
 *         if dtype == CXDTYPE_UINT16: return aval.u16             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UINT32: return aval.u32
 *         if dtype == CXDTYPE_UINT64: return aval.u64
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UINT16) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_uint16(__pyx_v_aval->u16); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 70; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":71
 *         if dtype == CXDTYPE_UINT8:  return aval.u8
 *         if dtype == CXDTYPE_UINT16: return aval.u16
 *         if dtype == CXDTYPE_UINT32: return aval.u32             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UINT64: return aval.u64
 *         if dtype == CXDTYPE_SINGLE: return aval.f32
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UINT32) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_uint32(__pyx_v_aval->u32); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 71; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":72
 *         if dtype == CXDTYPE_UINT16: return aval.u16
 *         if dtype == CXDTYPE_UINT32: return aval.u32
 *         if dtype == CXDTYPE_UINT64: return aval.u64             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_SINGLE: return aval.f32
 *         if dtype == CXDTYPE_TEXT:   return aval.c8
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UINT64) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_uint64(__pyx_v_aval->u64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 72; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":73
 *         if dtype == CXDTYPE_UINT32: return aval.u32
 *         if dtype == CXDTYPE_UINT64: return aval.u64
 *         if dtype == CXDTYPE_SINGLE: return aval.f32             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_TEXT:   return aval.c8
 *         if dtype == CXDTYPE_UCTEXT: return aval.c32
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_SINGLE) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_aval->f32); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":74
 *         if dtype == CXDTYPE_UINT64: return aval.u64
 *         if dtype == CXDTYPE_SINGLE: return aval.f32
 *         if dtype == CXDTYPE_TEXT:   return aval.c8             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UCTEXT: return aval.c32
 *         return None
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_TEXT) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_char8(__pyx_v_aval->c8); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 74; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":75
 *         if dtype == CXDTYPE_SINGLE: return aval.f32
 *         if dtype == CXDTYPE_TEXT:   return aval.c8
 *         if dtype == CXDTYPE_UCTEXT: return aval.c32             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UCTEXT) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_uint32(__pyx_v_aval->c32); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 75; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":76
 *         if dtype == CXDTYPE_TEXT:   return aval.c8
 *         if dtype == CXDTYPE_UCTEXT: return aval.c32
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     void aval_setvalue(value, CxAnyVal_t *aval, cxdtype_t dtype):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_None);
  __pyx_r = Py_None;
  goto __pyx_L0;

  /* "pycx4/pycdauser.pxi":63
 * # not to effective, but not a bottleneck
 * cdef:
 *     aval_value(CxAnyVal_t *aval, cxdtype_t dtype):             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_DOUBLE: return aval.f64
 *         if dtype == CXDTYPE_INT32:  return aval.i32
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pycx4.pycda.aval_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":78
 *         return None
 * 
 *     void aval_setvalue(value, CxAnyVal_t *aval, cxdtype_t dtype):             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_DOUBLE: aval.f64 = value
 *         if dtype == CXDTYPE_INT32:  aval.i32 = value
 */

static void __pyx_f_5pycx4_5pycda_aval_setvalue(PyObject *__pyx_v_value, CxAnyVal_t *__pyx_v_aval, cxdtype_t __pyx_v_dtype) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  float64 __pyx_t_2;
  int32 __pyx_t_3;
  int8 __pyx_t_4;
  int16 __pyx_t_5;
  int64 __pyx_t_6;
  uint8 __pyx_t_7;
  uint16 __pyx_t_8;
  uint32 __pyx_t_9;
  uint64 __pyx_t_10;
  float32 __pyx_t_11;
  char8 __pyx_t_12;
  char32 __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("aval_setvalue", 0);

  /* "pycx4/pycdauser.pxi":79
 * 
 *     void aval_setvalue(value, CxAnyVal_t *aval, cxdtype_t dtype):
 *         if dtype == CXDTYPE_DOUBLE: aval.f64 = value             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_INT32:  aval.i32 = value
 *         if dtype == CXDTYPE_INT8:   aval.i8 = value
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_DOUBLE) != 0);
  if (__pyx_t_1) {
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_2 == (float64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 79; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_aval->f64 = __pyx_t_2;
  }

  /* "pycx4/pycdauser.pxi":80
 *     void aval_setvalue(value, CxAnyVal_t *aval, cxdtype_t dtype):
 *         if dtype == CXDTYPE_DOUBLE: aval.f64 = value
 *         if dtype == CXDTYPE_INT32:  aval.i32 = value             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_INT8:   aval.i8 = value
 *         if dtype == CXDTYPE_INT16:  aval.i16 = value
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_INT32) != 0);
  if (__pyx_t_1) {
    __pyx_t_3 = __Pyx_PyInt_As_int32(__pyx_v_value); if (unlikely((__pyx_t_3 == (int32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 80; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_aval->i32 = __pyx_t_3;
  }

  /* "pycx4/pycdauser.pxi":81
 *         if dtype == CXDTYPE_DOUBLE: aval.f64 = value
 *         if dtype == CXDTYPE_INT32:  aval.i32 = value
 *         if dtype == CXDTYPE_INT8:   aval.i8 = value             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_INT16:  aval.i16 = value
 *         if dtype == CXDTYPE_INT64:  aval.i64 = value
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_INT8) != 0);
  if (__pyx_t_1) {
    __pyx_t_4 = __Pyx_PyInt_As_int8(__pyx_v_value); if (unlikely((__pyx_t_4 == (int8)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 81; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_aval->i8 = __pyx_t_4;
  }

  /* "pycx4/pycdauser.pxi":82
 *         if dtype == CXDTYPE_INT32:  aval.i32 = value
 *         if dtype == CXDTYPE_INT8:   aval.i8 = value
 *         if dtype == CXDTYPE_INT16:  aval.i16 = value             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_INT64:  aval.i64 = value
 *         if dtype == CXDTYPE_UINT8:  aval.u8 = value
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_INT16) != 0);
  if (__pyx_t_1) {
    __pyx_t_5 = __Pyx_PyInt_As_int16(__pyx_v_value); if (unlikely((__pyx_t_5 == (int16)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 82; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_aval->i16 = __pyx_t_5;
  }

  /* "pycx4/pycdauser.pxi":83
 *         if dtype == CXDTYPE_INT8:   aval.i8 = value
 *         if dtype == CXDTYPE_INT16:  aval.i16 = value
 *         if dtype == CXDTYPE_INT64:  aval.i64 = value             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UINT8:  aval.u8 = value
 *         if dtype == CXDTYPE_UINT16: aval.u16 = value
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_INT64) != 0);
  if (__pyx_t_1) {
    __pyx_t_6 = __Pyx_PyInt_As_int64(__pyx_v_value); if (unlikely((__pyx_t_6 == (int64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 83; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_aval->i64 = __pyx_t_6;
  }

  /* "pycx4/pycdauser.pxi":84
 *         if dtype == CXDTYPE_INT16:  aval.i16 = value
 *         if dtype == CXDTYPE_INT64:  aval.i64 = value
 *         if dtype == CXDTYPE_UINT8:  aval.u8 = value             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UINT16: aval.u16 = value
 *         if dtype == CXDTYPE_UINT32: aval.u32 = value
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UINT8) != 0);
  if (__pyx_t_1) {
    __pyx_t_7 = __Pyx_PyInt_As_uint8(__pyx_v_value); if (unlikely((__pyx_t_7 == (uint8)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 84; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_aval->u8 = __pyx_t_7;
  }

  /* "pycx4/pycdauser.pxi":85
 *         if dtype == CXDTYPE_INT64:  aval.i64 = value
 *         if dtype == CXDTYPE_UINT8:  aval.u8 = value
 *         if dtype == CXDTYPE_UINT16: aval.u16 = value             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UINT32: aval.u32 = value
 *         if dtype == CXDTYPE_UINT64: aval.u64 = value
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UINT16) != 0);
  if (__pyx_t_1) {
    __pyx_t_8 = __Pyx_PyInt_As_uint16(__pyx_v_value); if (unlikely((__pyx_t_8 == (uint16)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 85; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_aval->u16 = __pyx_t_8;
  }

  /* "pycx4/pycdauser.pxi":86
 *         if dtype == CXDTYPE_UINT8:  aval.u8 = value
 *         if dtype == CXDTYPE_UINT16: aval.u16 = value
 *         if dtype == CXDTYPE_UINT32: aval.u32 = value             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UINT64: aval.u64 = value
 *         if dtype == CXDTYPE_SINGLE: aval.f32 = value
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UINT32) != 0);
  if (__pyx_t_1) {
    __pyx_t_9 = __Pyx_PyInt_As_uint32(__pyx_v_value); if (unlikely((__pyx_t_9 == (uint32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 86; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_aval->u32 = __pyx_t_9;
  }

  /* "pycx4/pycdauser.pxi":87
 *         if dtype == CXDTYPE_UINT16: aval.u16 = value
 *         if dtype == CXDTYPE_UINT32: aval.u32 = value
 *         if dtype == CXDTYPE_UINT64: aval.u64 = value             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_SINGLE: aval.f32 = value
 *         if dtype == CXDTYPE_TEXT:   aval.c8 = value
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UINT64) != 0);
  if (__pyx_t_1) {
    __pyx_t_10 = __Pyx_PyInt_As_uint64(__pyx_v_value); if (unlikely((__pyx_t_10 == (uint64)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 87; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_aval->u64 = __pyx_t_10;
  }

  /* "pycx4/pycdauser.pxi":88
 *         if dtype == CXDTYPE_UINT32: aval.u32 = value
 *         if dtype == CXDTYPE_UINT64: aval.u64 = value
 *         if dtype == CXDTYPE_SINGLE: aval.f32 = value             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_TEXT:   aval.c8 = value
 *         if dtype == CXDTYPE_UCTEXT: aval.c32 = value
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_SINGLE) != 0);
  if (__pyx_t_1) {
    __pyx_t_11 = __pyx_PyFloat_AsFloat(__pyx_v_value); if (unlikely((__pyx_t_11 == (float32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 88; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_aval->f32 = __pyx_t_11;
  }

  /* "pycx4/pycdauser.pxi":89
 *         if dtype == CXDTYPE_UINT64: aval.u64 = value
 *         if dtype == CXDTYPE_SINGLE: aval.f32 = value
 *         if dtype == CXDTYPE_TEXT:   aval.c8 = value             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UCTEXT: aval.c32 = value
 * 
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_TEXT) != 0);
  if (__pyx_t_1) {
    __pyx_t_12 = __Pyx_PyInt_As_char8(__pyx_v_value); if (unlikely((__pyx_t_12 == (char8)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 89; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_aval->c8 = __pyx_t_12;
  }

  /* "pycx4/pycdauser.pxi":90
 *         if dtype == CXDTYPE_SINGLE: aval.f32 = value
 *         if dtype == CXDTYPE_TEXT:   aval.c8 = value
 *         if dtype == CXDTYPE_UCTEXT: aval.c32 = value             # <<<<<<<<<<<<<<
 * 
 *     # dtype conversion numpy <--> CX
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UCTEXT) != 0);
  if (__pyx_t_1) {
    __pyx_t_13 = __Pyx_PyInt_As_uint32(__pyx_v_value); if (unlikely((__pyx_t_13 == (char32)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 90; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_aval->c32 = __pyx_t_13;
  }

  /* "pycx4/pycdauser.pxi":78
 *         return None
 * 
 *     void aval_setvalue(value, CxAnyVal_t *aval, cxdtype_t dtype):             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_DOUBLE: aval.f64 = value
 *         if dtype == CXDTYPE_INT32:  aval.i32 = value
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pycx4.pycda.aval_setvalue", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdauser.pxi":95
 *     # there are no text or unicode text.
 *     # Cause text and unicode text can't be mixed with numbers in a good way
 *     object cxdtype2np(cxdtype_t dtype):             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_DOUBLE: return np.double
 *         if dtype == CXDTYPE_INT32:  return np.int32
 */

static PyObject *__pyx_f_5pycx4_5pycda_cxdtype2np(cxdtype_t __pyx_v_dtype) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cxdtype2np", 0);

  /* "pycx4/pycdauser.pxi":96
 *     # Cause text and unicode text can't be mixed with numbers in a good way
 *     object cxdtype2np(cxdtype_t dtype):
 *         if dtype == CXDTYPE_DOUBLE: return np.double             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_INT32:  return np.int32
 *         if dtype == CXDTYPE_INT8:   return np.int8
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_DOUBLE) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 96; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_double); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 96; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":97
 *     object cxdtype2np(cxdtype_t dtype):
 *         if dtype == CXDTYPE_DOUBLE: return np.double
 *         if dtype == CXDTYPE_INT32:  return np.int32             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_INT8:   return np.int8
 *         if dtype == CXDTYPE_INT16:  return np.int16
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_INT32) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 97; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_int32); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 97; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":98
 *         if dtype == CXDTYPE_DOUBLE: return np.double
 *         if dtype == CXDTYPE_INT32:  return np.int32
 *         if dtype == CXDTYPE_INT8:   return np.int8             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_INT16:  return np.int16
 *         if dtype == CXDTYPE_INT64:  return np.int64
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_INT8) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 98; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_int8); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 98; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":99
 *         if dtype == CXDTYPE_INT32:  return np.int32
 *         if dtype == CXDTYPE_INT8:   return np.int8
 *         if dtype == CXDTYPE_INT16:  return np.int16             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_INT64:  return np.int64
 *         if dtype == CXDTYPE_UINT8:  return np.uint8
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_INT16) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_int16); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 99; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":100
 *         if dtype == CXDTYPE_INT8:   return np.int8
 *         if dtype == CXDTYPE_INT16:  return np.int16
 *         if dtype == CXDTYPE_INT64:  return np.int64             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UINT8:  return np.uint8
 *         if dtype == CXDTYPE_UINT16: return np.uint16
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_INT64) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_int64); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 100; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":101
 *         if dtype == CXDTYPE_INT16:  return np.int16
 *         if dtype == CXDTYPE_INT64:  return np.int64
 *         if dtype == CXDTYPE_UINT8:  return np.uint8             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UINT16: return np.uint16
 *         if dtype == CXDTYPE_UINT32: return np.uint32
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UINT8) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 101; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_uint8); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 101; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":102
 *         if dtype == CXDTYPE_INT64:  return np.int64
 *         if dtype == CXDTYPE_UINT8:  return np.uint8
 *         if dtype == CXDTYPE_UINT16: return np.uint16             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UINT32: return np.uint32
 *         if dtype == CXDTYPE_UINT64: return np.uint64
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UINT16) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 102; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_uint16); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 102; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":103
 *         if dtype == CXDTYPE_UINT8:  return np.uint8
 *         if dtype == CXDTYPE_UINT16: return np.uint16
 *         if dtype == CXDTYPE_UINT32: return np.uint32             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_UINT64: return np.uint64
 *         if dtype == CXDTYPE_SINGLE: return np.single
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UINT32) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 103; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_uint32); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 103; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":104
 *         if dtype == CXDTYPE_UINT16: return np.uint16
 *         if dtype == CXDTYPE_UINT32: return np.uint32
 *         if dtype == CXDTYPE_UINT64: return np.uint64             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_SINGLE: return np.single
 *         return None
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_UINT64) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 104; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_uint64); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 104; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":105
 *         if dtype == CXDTYPE_UINT32: return np.uint32
 *         if dtype == CXDTYPE_UINT64: return np.uint64
 *         if dtype == CXDTYPE_SINGLE: return np.single             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
  __pyx_t_1 = ((__pyx_v_dtype == CXDTYPE_SINGLE) != 0);
  if (__pyx_t_1) {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_single); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 105; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":106
 *         if dtype == CXDTYPE_UINT64: return np.uint64
 *         if dtype == CXDTYPE_SINGLE: return np.single
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     cxdtype_t np2cxdtype(object dtype):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_None);
  __pyx_r = Py_None;
  goto __pyx_L0;

  /* "pycx4/pycdauser.pxi":95
 *     # there are no text or unicode text.
 *     # Cause text and unicode text can't be mixed with numbers in a good way
 *     object cxdtype2np(cxdtype_t dtype):             # <<<<<<<<<<<<<<
 *         if dtype == CXDTYPE_DOUBLE: return np.double
 *         if dtype == CXDTYPE_INT32:  return np.int32
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pycx4.pycda.cxdtype2np", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":108
 *         return None
 * 
 *     cxdtype_t np2cxdtype(object dtype):             # <<<<<<<<<<<<<<
 *         if dtype == np.double:      return CXDTYPE_DOUBLE
 *         if dtype == np.int32:       return CXDTYPE_INT32
 */

static cxdtype_t __pyx_f_5pycx4_5pycda_np2cxdtype(PyObject *__pyx_v_dtype) {
  cxdtype_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("np2cxdtype", 0);

  /* "pycx4/pycdauser.pxi":109
 * 
 *     cxdtype_t np2cxdtype(object dtype):
 *         if dtype == np.double:      return CXDTYPE_DOUBLE             # <<<<<<<<<<<<<<
 *         if dtype == np.int32:       return CXDTYPE_INT32
 *         if dtype == np.int8:        return CXDTYPE_INT8
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_double); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_DOUBLE;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":110
 *     cxdtype_t np2cxdtype(object dtype):
 *         if dtype == np.double:      return CXDTYPE_DOUBLE
 *         if dtype == np.int32:       return CXDTYPE_INT32             # <<<<<<<<<<<<<<
 *         if dtype == np.int8:        return CXDTYPE_INT8
 *         if dtype == np.int16:       return CXDTYPE_INT16
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_int32); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_INT32;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":111
 *         if dtype == np.double:      return CXDTYPE_DOUBLE
 *         if dtype == np.int32:       return CXDTYPE_INT32
 *         if dtype == np.int8:        return CXDTYPE_INT8             # <<<<<<<<<<<<<<
 *         if dtype == np.int16:       return CXDTYPE_INT16
 *         if dtype == np.int64:       return CXDTYPE_INT64
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_int8); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_INT8;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":112
 *         if dtype == np.int32:       return CXDTYPE_INT32
 *         if dtype == np.int8:        return CXDTYPE_INT8
 *         if dtype == np.int16:       return CXDTYPE_INT16             # <<<<<<<<<<<<<<
 *         if dtype == np.int64:       return CXDTYPE_INT64
 *         if dtype == np.uint8:       return CXDTYPE_UINT8
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_int16); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_INT16;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":113
 *         if dtype == np.int8:        return CXDTYPE_INT8
 *         if dtype == np.int16:       return CXDTYPE_INT16
 *         if dtype == np.int64:       return CXDTYPE_INT64             # <<<<<<<<<<<<<<
 *         if dtype == np.uint8:       return CXDTYPE_UINT8
 *         if dtype == np.uint16:      return CXDTYPE_UINT16
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 113; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_int64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 113; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 113; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 113; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_INT64;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":114
 *         if dtype == np.int16:       return CXDTYPE_INT16
 *         if dtype == np.int64:       return CXDTYPE_INT64
 *         if dtype == np.uint8:       return CXDTYPE_UINT8             # <<<<<<<<<<<<<<
 *         if dtype == np.uint16:      return CXDTYPE_UINT16
 *         if dtype == np.uint32:      return CXDTYPE_UINT32
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_uint8); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 114; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_UINT8;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":115
 *         if dtype == np.int64:       return CXDTYPE_INT64
 *         if dtype == np.uint8:       return CXDTYPE_UINT8
 *         if dtype == np.uint16:      return CXDTYPE_UINT16             # <<<<<<<<<<<<<<
 *         if dtype == np.uint32:      return CXDTYPE_UINT32
 *         if dtype == np.uint64:      return CXDTYPE_UINT64
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_uint16); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_UINT16;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":116
 *         if dtype == np.uint8:       return CXDTYPE_UINT8
 *         if dtype == np.uint16:      return CXDTYPE_UINT16
 *         if dtype == np.uint32:      return CXDTYPE_UINT32             # <<<<<<<<<<<<<<
 *         if dtype == np.uint64:      return CXDTYPE_UINT64
 *         if dtype == np.single:      return CXDTYPE_SINGLE
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_uint32); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 116; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_UINT32;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":117
 *         if dtype == np.uint16:      return CXDTYPE_UINT16
 *         if dtype == np.uint32:      return CXDTYPE_UINT32
 *         if dtype == np.uint64:      return CXDTYPE_UINT64             # <<<<<<<<<<<<<<
 *         if dtype == np.single:      return CXDTYPE_SINGLE
 *         return 0
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_uint64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_UINT64;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":118
 *         if dtype == np.uint32:      return CXDTYPE_UINT32
 *         if dtype == np.uint64:      return CXDTYPE_UINT64
 *         if dtype == np.single:      return CXDTYPE_SINGLE             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 118; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_single); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 118; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_dtype, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 118; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 118; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {
    __pyx_r = CXDTYPE_SINGLE;
    goto __pyx_L0;
  }

  /* "pycx4/pycdauser.pxi":119
 *         if dtype == np.uint64:      return CXDTYPE_UINT64
 *         if dtype == np.single:      return CXDTYPE_SINGLE
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pycx4/pycdauser.pxi":108
 *         return None
 * 
 *     cxdtype_t np2cxdtype(object dtype):             # <<<<<<<<<<<<<<
 *         if dtype == np.double:      return CXDTYPE_DOUBLE
 *         if dtype == np.int32:       return CXDTYPE_INT32
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("pycx4.pycda.np2cxdtype", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":127
 *         readonly object val, prev_val
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE):             # <<<<<<<<<<<<<<
 *         cda_base_chan.__init__(self, name, context, dtype)
 * 
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_5schan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_5schan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_context = 0;
  cxdtype_t __pyx_v_dtype;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_context,&__pyx_n_s_dtype,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_context);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dtype);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_name = ((PyObject*)values[0]);
    __pyx_v_context = values[1];
    if (values[2]) {
      __pyx_v_dtype = __Pyx_PyInt_As_uint8(values[2]); if (unlikely((__pyx_v_dtype == (cxdtype_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_dtype = __pyx_k__5;
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.schan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyString_Type), 1, "name", 1))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5pycx4_5pycda_5schan___init__(((struct __pyx_obj_5pycx4_5pycda_schan *)__pyx_v_self), __pyx_v_name, __pyx_v_context, __pyx_v_dtype);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_5schan___init__(struct __pyx_obj_5pycx4_5pycda_schan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_context, cxdtype_t __pyx_v_dtype) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/pycdauser.pxi":128
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE):
 *         cda_base_chan.__init__(self, name, context, dtype)             # <<<<<<<<<<<<<<
 * 
 *     cdef void cb(self):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_cda_base_chan), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_uint8(__pyx_v_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(4+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_name);
  __Pyx_INCREF(__pyx_v_context);
  __Pyx_GIVEREF(__pyx_v_context);
  PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_context);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 3+__pyx_t_5, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdauser.pxi":127
 *         readonly object val, prev_val
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE):             # <<<<<<<<<<<<<<
 *         cda_base_chan.__init__(self, name, context, dtype)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pycx4.pycda.schan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":130
 *         cda_base_chan.__init__(self, name, context, dtype)
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         cdef CxAnyVal_t aval
 *         self.prev_val = self.val
 */

static void __pyx_f_5pycx4_5pycda_5schan_cb(struct __pyx_obj_5pycx4_5pycda_schan *__pyx_v_self) {
  CxAnyVal_t __pyx_v_aval;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cb", 0);

  /* "pycx4/pycdauser.pxi":132
 *     cdef void cb(self):
 *         cdef CxAnyVal_t aval
 *         self.prev_val = self.val             # <<<<<<<<<<<<<<
 *         self.get_data(0, self.itemsize, <void*>&aval)
 *         self.val = aval_value(&aval, self.dtype)
 */
  __pyx_t_1 = __pyx_v_self->val;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->prev_val);
  __Pyx_DECREF(__pyx_v_self->prev_val);
  __pyx_v_self->prev_val = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/pycdauser.pxi":133
 *         cdef CxAnyVal_t aval
 *         self.prev_val = self.val
 *         self.get_data(0, self.itemsize, <void*>&aval)             # <<<<<<<<<<<<<<
 *         self.val = aval_value(&aval, self.dtype)
 *         if self.val != self.prev_val or self.first_cycle:
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_schan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.get_data(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self), 0, __pyx_v_self->__pyx_base.itemsize, ((void *)(&__pyx_v_aval)));

  /* "pycx4/pycdauser.pxi":134
 *         self.prev_val = self.val
 *         self.get_data(0, self.itemsize, <void*>&aval)
 *         self.val = aval_value(&aval, self.dtype)             # <<<<<<<<<<<<<<
 *         if self.val != self.prev_val or self.first_cycle:
 *             self.valueChanged.emit(self)
 */
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_aval_value((&__pyx_v_aval), __pyx_v_self->__pyx_base.dtype); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 134; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->val);
  __Pyx_DECREF(__pyx_v_self->val);
  __pyx_v_self->val = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/pycdauser.pxi":135
 *         self.get_data(0, self.itemsize, <void*>&aval)
 *         self.val = aval_value(&aval, self.dtype)
 *         if self.val != self.prev_val or self.first_cycle:             # <<<<<<<<<<<<<<
 *             self.valueChanged.emit(self)
 *             self.first_cycle = False
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_self->val, __pyx_v_self->prev_val, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_self->__pyx_base.first_cycle != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "pycx4/pycdauser.pxi":136
 *         self.val = aval_value(&aval, self.dtype)
 *         if self.val != self.prev_val or self.first_cycle:
 *             self.valueChanged.emit(self)             # <<<<<<<<<<<<<<
 *             self.first_cycle = False
 *         self.valueMeasured.emit(self)
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_signal *)__pyx_v_self->__pyx_base.valueChanged->__pyx_vtab)->emit(__pyx_v_self->__pyx_base.valueChanged, ((PyObject *)__pyx_v_self), 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pycx4/pycdauser.pxi":137
 *         if self.val != self.prev_val or self.first_cycle:
 *             self.valueChanged.emit(self)
 *             self.first_cycle = False             # <<<<<<<<<<<<<<
 *         self.valueMeasured.emit(self)
 * 
 */
    __pyx_v_self->__pyx_base.first_cycle = 0;

    /* "pycx4/pycdauser.pxi":135
 *         self.get_data(0, self.itemsize, <void*>&aval)
 *         self.val = aval_value(&aval, self.dtype)
 *         if self.val != self.prev_val or self.first_cycle:             # <<<<<<<<<<<<<<
 *             self.valueChanged.emit(self)
 *             self.first_cycle = False
 */
  }

  /* "pycx4/pycdauser.pxi":138
 *             self.valueChanged.emit(self)
 *             self.first_cycle = False
 *         self.valueMeasured.emit(self)             # <<<<<<<<<<<<<<
 * 
 *     cpdef setValue(self, value):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_signal *)__pyx_v_self->__pyx_base.valueMeasured->__pyx_vtab)->emit(__pyx_v_self->__pyx_base.valueMeasured, ((PyObject *)__pyx_v_self), 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdauser.pxi":130
 *         cda_base_chan.__init__(self, name, context, dtype)
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         cdef CxAnyVal_t aval
 *         self.prev_val = self.val
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("pycx4.pycda.schan.cb", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdauser.pxi":140
 *         self.valueMeasured.emit(self)
 * 
 *     cpdef setValue(self, value):             # <<<<<<<<<<<<<<
 *         cdef CxAnyVal_t aval
 *         aval_setvalue(value, &aval, self.dtype)
 */

static PyObject *__pyx_pw_5pycx4_5pycda_5schan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_5schan_setValue(struct __pyx_obj_5pycx4_5pycda_schan *__pyx_v_self, PyObject *__pyx_v_value, int __pyx_skip_dispatch) {
  CxAnyVal_t __pyx_v_aval;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setValue", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setValue); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 140; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5pycx4_5pycda_5schan_3setValue)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_value); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 140; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 140; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_value);
        __Pyx_GIVEREF(__pyx_v_value);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_value);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 140; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pycx4/pycdauser.pxi":142
 *     cpdef setValue(self, value):
 *         cdef CxAnyVal_t aval
 *         aval_setvalue(value, &aval, self.dtype)             # <<<<<<<<<<<<<<
 *         self.snd_data(self.dtype, 1, <void*>&aval)
 * 
 */
  __pyx_f_5pycx4_5pycda_aval_setvalue(__pyx_v_value, (&__pyx_v_aval), __pyx_v_self->__pyx_base.dtype);

  /* "pycx4/pycdauser.pxi":143
 *         cdef CxAnyVal_t aval
 *         aval_setvalue(value, &aval, self.dtype)
 *         self.snd_data(self.dtype, 1, <void*>&aval)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_schan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.snd_data(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self), __pyx_v_self->__pyx_base.dtype, 1, ((void *)(&__pyx_v_aval)));

  /* "pycx4/pycdauser.pxi":140
 *         self.valueMeasured.emit(self)
 * 
 *     cpdef setValue(self, value):             # <<<<<<<<<<<<<<
 *         cdef CxAnyVal_t aval
 *         aval_setvalue(value, &aval, self.dtype)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.schan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5schan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5schan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setValue (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5schan_2setValue(((struct __pyx_obj_5pycx4_5pycda_schan *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5schan_2setValue(struct __pyx_obj_5pycx4_5pycda_schan *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setValue", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_5schan_setValue(__pyx_v_self, __pyx_v_value, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 140; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.schan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":125
 * cdef class schan(cda_base_chan):
 *     cdef:
 *         readonly object val, prev_val             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5schan_3val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5schan_3val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5schan_3val___get__(((struct __pyx_obj_5pycx4_5pycda_schan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5schan_3val___get__(struct __pyx_obj_5pycx4_5pycda_schan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->val);
  __pyx_r = __pyx_v_self->val;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5schan_8prev_val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5schan_8prev_val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5schan_8prev_val___get__(((struct __pyx_obj_5pycx4_5pycda_schan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5schan_8prev_val___get__(struct __pyx_obj_5pycx4_5pycda_schan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->prev_val);
  __pyx_r = __pyx_v_self->prev_val;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":152
 *         readonly object npdtype
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE, int max_nelems=1):             # <<<<<<<<<<<<<<
 *         cda_base_chan.__init__(self, name, context, dtype, max_nelems)
 *         self.npdtype = cxdtype2np(dtype)
 */

/* Python wrapper */
static int __pyx_pw_5pycx4_5pycda_5vchan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5pycx4_5pycda_5vchan_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_context = 0;
  cxdtype_t __pyx_v_dtype;
  int __pyx_v_max_nelems;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_context,&__pyx_n_s_dtype,&__pyx_n_s_max_nelems,0};
    PyObject* values[4] = {0,0,0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_context);
          if (value) { values[1] = value; kw_args--; }
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dtype);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_max_nelems);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 152; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_name = ((PyObject*)values[0]);
    __pyx_v_context = values[1];
    if (values[2]) {
      __pyx_v_dtype = __Pyx_PyInt_As_uint8(values[2]); if (unlikely((__pyx_v_dtype == (cxdtype_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 152; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_dtype = __pyx_k__6;
    }
    if (values[3]) {
      __pyx_v_max_nelems = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_max_nelems == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 152; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_max_nelems = ((int)1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 152; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("pycx4.pycda.vchan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyString_Type), 1, "name", 1))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 152; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5pycx4_5pycda_5vchan___init__(((struct __pyx_obj_5pycx4_5pycda_vchan *)__pyx_v_self), __pyx_v_name, __pyx_v_context, __pyx_v_dtype, __pyx_v_max_nelems);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5pycx4_5pycda_5vchan___init__(struct __pyx_obj_5pycx4_5pycda_vchan *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_context, cxdtype_t __pyx_v_dtype, int __pyx_v_max_nelems) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pycx4/pycdauser.pxi":153
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE, int max_nelems=1):
 *         cda_base_chan.__init__(self, name, context, dtype, max_nelems)             # <<<<<<<<<<<<<<
 *         self.npdtype = cxdtype2np(dtype)
 *         self.val = np.zeros(max_nelems, self.npdtype, order='C')
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5pycx4_5pycda_cda_base_chan), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_uint8(__pyx_v_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_max_nelems); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  __pyx_t_7 = PyTuple_New(5+__pyx_t_6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  if (__pyx_t_5) {
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
  }
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, ((PyObject *)__pyx_v_self));
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_v_name);
  __Pyx_INCREF(__pyx_v_context);
  __Pyx_GIVEREF(__pyx_v_context);
  PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_6, __pyx_v_context);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_7, 3+__pyx_t_6, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_7, 4+__pyx_t_6, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdauser.pxi":154
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE, int max_nelems=1):
 *         cda_base_chan.__init__(self, name, context, dtype, max_nelems)
 *         self.npdtype = cxdtype2np(dtype)             # <<<<<<<<<<<<<<
 *         self.val = np.zeros(max_nelems, self.npdtype, order='C')
 * 
 */
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_cxdtype2np(__pyx_v_dtype); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->npdtype);
  __Pyx_DECREF(__pyx_v_self->npdtype);
  __pyx_v_self->npdtype = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pycx4/pycdauser.pxi":155
 *         cda_base_chan.__init__(self, name, context, dtype, max_nelems)
 *         self.npdtype = cxdtype2np(dtype)
 *         self.val = np.zeros(max_nelems, self.npdtype, order='C')             # <<<<<<<<<<<<<<
 * 
 *     cdef void cb(self):
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_max_nelems); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->npdtype);
  __Pyx_GIVEREF(__pyx_v_self->npdtype);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_v_self->npdtype);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_order, __pyx_n_s_C) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5numpy_ndarray))))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->val);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->val));
  __pyx_v_self->val = ((PyArrayObject *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pycx4/pycdauser.pxi":152
 *         readonly object npdtype
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE, int max_nelems=1):             # <<<<<<<<<<<<<<
 *         cda_base_chan.__init__(self, name, context, dtype, max_nelems)
 *         self.npdtype = cxdtype2np(dtype)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pycx4.pycda.vchan.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":157
 *         self.val = np.zeros(max_nelems, self.npdtype, order='C')
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         self.get_data(0, self.itemsize * self.max_nelems, <void*>self.val.data)
 *         self.valueMeasured.emit(self)
 */

static void __pyx_f_5pycx4_5pycda_5vchan_cb(struct __pyx_obj_5pycx4_5pycda_vchan *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("cb", 0);

  /* "pycx4/pycdauser.pxi":158
 * 
 *     cdef void cb(self):
 *         self.get_data(0, self.itemsize * self.max_nelems, <void*>self.val.data)             # <<<<<<<<<<<<<<
 *         self.valueMeasured.emit(self)
 *         self.valueChanged.emit(self)
 */
  ((struct __pyx_vtabstruct_5pycx4_5pycda_vchan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.get_data(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self), 0, (__pyx_v_self->__pyx_base.itemsize * __pyx_v_self->__pyx_base.max_nelems), ((void *)__pyx_v_self->val->data));

  /* "pycx4/pycdauser.pxi":159
 *     cdef void cb(self):
 *         self.get_data(0, self.itemsize * self.max_nelems, <void*>self.val.data)
 *         self.valueMeasured.emit(self)             # <<<<<<<<<<<<<<
 *         self.valueChanged.emit(self)
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_signal *)__pyx_v_self->__pyx_base.valueMeasured->__pyx_vtab)->emit(__pyx_v_self->__pyx_base.valueMeasured, ((PyObject *)__pyx_v_self), 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 159; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdauser.pxi":160
 *         self.get_data(0, self.itemsize * self.max_nelems, <void*>self.val.data)
 *         self.valueMeasured.emit(self)
 *         self.valueChanged.emit(self)             # <<<<<<<<<<<<<<
 * 
 *     cpdef setValue(self, np.ndarray value):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5pycx4_5pycda_cda_signal *)__pyx_v_self->__pyx_base.valueChanged->__pyx_vtab)->emit(__pyx_v_self->__pyx_base.valueChanged, ((PyObject *)__pyx_v_self), 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdauser.pxi":157
 *         self.val = np.zeros(max_nelems, self.npdtype, order='C')
 * 
 *     cdef void cb(self):             # <<<<<<<<<<<<<<
 *         self.get_data(0, self.itemsize * self.max_nelems, <void*>self.val.data)
 *         self.valueMeasured.emit(self)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("pycx4.pycda.vchan.cb", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "pycx4/pycdauser.pxi":162
 *         self.valueChanged.emit(self)
 * 
 *     cpdef setValue(self, np.ndarray value):             # <<<<<<<<<<<<<<
 *         if value.size > self.max_nelems:
 *             raise Exception('value size greater than channel.max_nelems')
 */

static PyObject *__pyx_pw_5pycx4_5pycda_5vchan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static PyObject *__pyx_f_5pycx4_5pycda_5vchan_setValue(struct __pyx_obj_5pycx4_5pycda_vchan *__pyx_v_self, PyArrayObject *__pyx_v_value, int __pyx_skip_dispatch) {
  cxdtype_t __pyx_v_dtype;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setValue", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_setValue); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5pycx4_5pycda_5vchan_3setValue)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_value)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(((PyObject *)__pyx_v_value));
        __Pyx_GIVEREF(((PyObject *)__pyx_v_value));
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)__pyx_v_value));
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pycx4/pycdauser.pxi":163
 * 
 *     cpdef setValue(self, np.ndarray value):
 *         if value.size > self.max_nelems:             # <<<<<<<<<<<<<<
 *             raise Exception('value size greater than channel.max_nelems')
 *         dtype = np2cxdtype(value.dtype)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_value), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.max_nelems); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_6) {

    /* "pycx4/pycdauser.pxi":164
 *     cpdef setValue(self, np.ndarray value):
 *         if value.size > self.max_nelems:
 *             raise Exception('value size greater than channel.max_nelems')             # <<<<<<<<<<<<<<
 *         dtype = np2cxdtype(value.dtype)
 *         self.snd_data(dtype, value.size, <void*>value.data)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_Exception, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "pycx4/pycdauser.pxi":163
 * 
 *     cpdef setValue(self, np.ndarray value):
 *         if value.size > self.max_nelems:             # <<<<<<<<<<<<<<
 *             raise Exception('value size greater than channel.max_nelems')
 *         dtype = np2cxdtype(value.dtype)
 */
  }

  /* "pycx4/pycdauser.pxi":165
 *         if value.size > self.max_nelems:
 *             raise Exception('value size greater than channel.max_nelems')
 *         dtype = np2cxdtype(value.dtype)             # <<<<<<<<<<<<<<
 *         self.snd_data(dtype, value.size, <void*>value.data)
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_value), __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 165; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_dtype = __pyx_f_5pycx4_5pycda_np2cxdtype(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pycx4/pycdauser.pxi":166
 *             raise Exception('value size greater than channel.max_nelems')
 *         dtype = np2cxdtype(value.dtype)
 *         self.snd_data(dtype, value.size, <void*>value.data)             # <<<<<<<<<<<<<<
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_value), __pyx_n_s_size); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 166; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 166; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  ((struct __pyx_vtabstruct_5pycx4_5pycda_vchan *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.snd_data(((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)__pyx_v_self), __pyx_v_dtype, __pyx_t_7, ((void *)__pyx_v_value->data));

  /* "pycx4/pycdauser.pxi":162
 *         self.valueChanged.emit(self)
 * 
 *     cpdef setValue(self, np.ndarray value):             # <<<<<<<<<<<<<<
 *         if value.size > self.max_nelems:
 *             raise Exception('value size greater than channel.max_nelems')
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pycx4.pycda.vchan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5vchan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5vchan_3setValue(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED int __pyx_lineno = 0;
  CYTHON_UNUSED const char *__pyx_filename = NULL;
  CYTHON_UNUSED int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setValue (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), __pyx_ptype_5numpy_ndarray, 1, "value", 0))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_r = __pyx_pf_5pycx4_5pycda_5vchan_2setValue(((struct __pyx_obj_5pycx4_5pycda_vchan *)__pyx_v_self), ((PyArrayObject *)__pyx_v_value));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5vchan_2setValue(struct __pyx_obj_5pycx4_5pycda_vchan *__pyx_v_self, PyArrayObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setValue", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5pycx4_5pycda_5vchan_setValue(__pyx_v_self, __pyx_v_value, 1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pycx4.pycda.vchan.setValue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":149
 * cdef class vchan(cda_base_chan):
 *     cdef:
 *         readonly np.ndarray val             # <<<<<<<<<<<<<<
 *         readonly object npdtype
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5vchan_3val_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5vchan_3val_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5vchan_3val___get__(((struct __pyx_obj_5pycx4_5pycda_vchan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5vchan_3val___get__(struct __pyx_obj_5pycx4_5pycda_vchan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->val));
  __pyx_r = ((PyObject *)__pyx_v_self->val);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pycx4/pycdauser.pxi":150
 *     cdef:
 *         readonly np.ndarray val
 *         readonly object npdtype             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE, int max_nelems=1):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5pycx4_5pycda_5vchan_7npdtype_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5pycx4_5pycda_5vchan_7npdtype_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5pycx4_5pycda_5vchan_7npdtype___get__(((struct __pyx_obj_5pycx4_5pycda_vchan *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5pycx4_5pycda_5vchan_7npdtype___get__(struct __pyx_obj_5pycx4_5pycda_vchan *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->npdtype);
  __pyx_r = __pyx_v_self->npdtype;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5numpy_7ndarray___getbuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_copy_shape;
  int __pyx_v_i;
  int __pyx_v_ndim;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  int __pyx_v_t;
  char *__pyx_v_f;
  PyArray_Descr *__pyx_v_descr = 0;
  int __pyx_v_offset;
  int __pyx_v_hasfields;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  char *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  if (__pyx_v_info != NULL) {
    __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(__pyx_v_info->obj);
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":203
 *             # of flags
 * 
 *             if info == NULL: return             # <<<<<<<<<<<<<<
 * 
 *             cdef int copy_shape, i, ndim
 */
  __pyx_t_1 = ((__pyx_v_info == NULL) != 0);
  if (__pyx_t_1) {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":206
 * 
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":207
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 * 
 *             ndim = PyArray_NDIM(self)
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":209
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 *             ndim = PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_v_ndim = PyArray_NDIM(__pyx_v_self);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":211
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":212
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 copy_shape = 1             # <<<<<<<<<<<<<<
 *             else:
 *                 copy_shape = 0
 */
    __pyx_v_copy_shape = 1;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":211
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
    goto __pyx_L4;
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":214
 *                 copy_shape = 1
 *             else:
 *                 copy_shape = 0             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 */
  /*else*/ {
    __pyx_v_copy_shape = 0;
  }
  __pyx_L4:;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_C_CONTIGUOUS) == PyBUF_C_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":217
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_C_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  if (__pyx_t_1) {

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_F_CONTIGUOUS) == PyBUF_F_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":221
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_F_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  if (__pyx_t_1) {

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":224
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 *             info.buf = PyArray_DATA(self)             # <<<<<<<<<<<<<<
 *             info.ndim = ndim
 *             if copy_shape:
 */
  __pyx_v_info->buf = PyArray_DATA(__pyx_v_self);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":225
 * 
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim             # <<<<<<<<<<<<<<
 *             if copy_shape:
 *                 # Allocate new buffer for strides and shape info.
 */
  __pyx_v_info->ndim = __pyx_v_ndim;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":226
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
  __pyx_t_1 = (__pyx_v_copy_shape != 0);
  if (__pyx_t_1) {

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":229
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)             # <<<<<<<<<<<<<<
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 */
    __pyx_v_info->strides = ((Py_ssize_t *)malloc((((sizeof(Py_ssize_t)) * ((size_t)__pyx_v_ndim)) * 2)));

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":230
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim             # <<<<<<<<<<<<<<
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 */
    __pyx_v_info->shape = (__pyx_v_info->strides + __pyx_v_ndim);

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":231
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):             # <<<<<<<<<<<<<<
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 */
    __pyx_t_4 = __pyx_v_ndim;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":232
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]             # <<<<<<<<<<<<<<
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 */
      (__pyx_v_info->strides[__pyx_v_i]) = (PyArray_STRIDES(__pyx_v_self)[__pyx_v_i]);

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":233
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]             # <<<<<<<<<<<<<<
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 */
      (__pyx_v_info->shape[__pyx_v_i]) = (PyArray_DIMS(__pyx_v_self)[__pyx_v_i]);
    }

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":226
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
    goto __pyx_L11;
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":235
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 */
  /*else*/ {
    __pyx_v_info->strides = ((Py_ssize_t *)PyArray_STRIDES(__pyx_v_self));

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":236
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 */
    __pyx_v_info->shape = ((Py_ssize_t *)PyArray_DIMS(__pyx_v_self));
  }
  __pyx_L11:;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":237
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 */
  __pyx_v_info->suboffsets = NULL;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":238
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)             # <<<<<<<<<<<<<<
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 * 
 */
  __pyx_v_info->itemsize = PyArray_ITEMSIZE(__pyx_v_self);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":239
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)             # <<<<<<<<<<<<<<
 * 
 *             cdef int t
 */
  __pyx_v_info->readonly = (!(PyArray_ISWRITEABLE(__pyx_v_self) != 0));

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":242
 * 
 *             cdef int t
 *             cdef char* f = NULL             # <<<<<<<<<<<<<<
 *             cdef dtype descr = self.descr
 *             cdef int offset
 */
  __pyx_v_f = NULL;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":243
 *             cdef int t
 *             cdef char* f = NULL
 *             cdef dtype descr = self.descr             # <<<<<<<<<<<<<<
 *             cdef int offset
 * 
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->descr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_descr = ((PyArray_Descr *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":246
 *             cdef int offset
 * 
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields and not copy_shape:
 */
  __pyx_v_hasfields = PyDataType_HASFIELDS(__pyx_v_descr);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":248
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
  __pyx_t_2 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_v_copy_shape != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_1) {

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":250
 *             if not hasfields and not copy_shape:
 *                 # do not call releasebuffer
 *                 info.obj = None             # <<<<<<<<<<<<<<
 *             else:
 *                 # need to call releasebuffer
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = Py_None;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":248
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
    goto __pyx_L14;
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":253
 *             else:
 *                 # need to call releasebuffer
 *                 info.obj = self             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields:
 */
  /*else*/ {
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = ((PyObject *)__pyx_v_self);
  }
  __pyx_L14:;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":255
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  __pyx_t_1 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_1) {

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":256
 * 
 *             if not hasfields:
 *                 t = descr.type_num             # <<<<<<<<<<<<<<
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 */
    __pyx_t_4 = __pyx_v_descr->type_num;
    __pyx_v_t = __pyx_t_4;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '>') != 0);
    if (!__pyx_t_2) {
      goto __pyx_L20_next_or;
    } else {
    }
    __pyx_t_2 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_L20_next_or:;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":258
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '<') != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_t_2 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L19_bool_binop_done:;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    if (__pyx_t_1) {

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":259
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":260
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 */
    switch (__pyx_v_t) {
      case NPY_BYTE:
      __pyx_v_f = __pyx_k_b;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":261
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 */
      case NPY_UBYTE:
      __pyx_v_f = __pyx_k_B;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":262
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 */
      case NPY_SHORT:
      __pyx_v_f = __pyx_k_h;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":263
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 */
      case NPY_USHORT:
      __pyx_v_f = __pyx_k_H;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":264
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 */
      case NPY_INT:
      __pyx_v_f = __pyx_k_i;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":265
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 */
      case NPY_UINT:
      __pyx_v_f = __pyx_k_I;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":266
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 */
      case NPY_LONG:
      __pyx_v_f = __pyx_k_l;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":267
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 */
      case NPY_ULONG:
      __pyx_v_f = __pyx_k_L;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":268
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 */
      case NPY_LONGLONG:
      __pyx_v_f = __pyx_k_q;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":269
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 */
      case NPY_ULONGLONG:
      __pyx_v_f = __pyx_k_Q;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":270
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 */
      case NPY_FLOAT:
      __pyx_v_f = __pyx_k_f;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":271
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 */
      case NPY_DOUBLE:
      __pyx_v_f = __pyx_k_d;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":272
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 */
      case NPY_LONGDOUBLE:
      __pyx_v_f = __pyx_k_g;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":273
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 */
      case NPY_CFLOAT:
      __pyx_v_f = __pyx_k_Zf;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"
 */
      case NPY_CDOUBLE:
      __pyx_v_f = __pyx_k_Zd;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":275
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 */
      case NPY_CLONGDOUBLE:
      __pyx_v_f = __pyx_k_Zg;
      break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":276
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      case NPY_OBJECT:
      __pyx_v_f = __pyx_k_O;
      break;
      default:

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":278
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *                 info.format = f
 *                 return
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_t_3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      break;
    }

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":279
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f             # <<<<<<<<<<<<<<
 *                 return
 *             else:
 */
    __pyx_v_info->format = __pyx_v_f;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":280
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f
 *                 return             # <<<<<<<<<<<<<<
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":255
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":282
 *                 return
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)             # <<<<<<<<<<<<<<
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 */
  /*else*/ {
    __pyx_v_info->format = ((char *)malloc(0xFF));

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":283
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment             # <<<<<<<<<<<<<<
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,
 */
    (__pyx_v_info->format[0]) = '^';

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":284
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0             # <<<<<<<<<<<<<<
 *                 f = _util_dtypestring(descr, info.format + 1,
 *                                       info.format + _buffer_format_string_len,
 */
    __pyx_v_offset = 0;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":285
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,             # <<<<<<<<<<<<<<
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 */
    __pyx_t_7 = __pyx_f_5numpy__util_dtypestring(__pyx_v_descr, (__pyx_v_info->format + 1), (__pyx_v_info->format + 0xFF), (&__pyx_v_offset)); if (unlikely(__pyx_t_7 == NULL)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_f = __pyx_t_7;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":288
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 *                 f[0] = c'\0' # Terminate format string             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 */
    (__pyx_v_f[0]) = '\x00';
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("numpy.ndarray.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info != NULL && __pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = NULL;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info != NULL && __pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(Py_None);
    __Pyx_DECREF(Py_None); __pyx_v_info->obj = NULL;
  }
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_descr);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":290
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_5numpy_7ndarray_2__releasebuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":291
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_t_1 = (PyArray_HASFIELDS(__pyx_v_self) != 0);
  if (__pyx_t_1) {

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":292
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)
 */
    free(__pyx_v_info->format);

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":291
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":294
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)             # <<<<<<<<<<<<<<
 *                 # info.shape was stored after info.strides in the same block
 * 
 */
    free(__pyx_v_info->strides);

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":293
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":290
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":770
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":771
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":770
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":773
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":774
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 774; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":773
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":776
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":777
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 777; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":776
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":779
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":780
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 780; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":779
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":782
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":783
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 783; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":782
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":785
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *__pyx_v_descr, char *__pyx_v_f, char *__pyx_v_end, int *__pyx_v_offset) {
  PyArray_Descr *__pyx_v_child = 0;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_childname = NULL;
  PyObject *__pyx_v_new_offset = NULL;
  PyObject *__pyx_v_t = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  long __pyx_t_8;
  char *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_util_dtypestring", 0);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":790
 * 
 *     cdef dtype child
 *     cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 *     cdef tuple fields
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":791
 *     cdef dtype child
 *     cdef int endian_detector = 1
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 *     cdef tuple fields
 * 
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  if (unlikely(__pyx_v_descr->names == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[3]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __pyx_v_descr->names; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_childname, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":795
 * 
 *     for childname in descr.names:
 *         fields = descr.fields[childname]             # <<<<<<<<<<<<<<
 *         child, new_offset = fields
 * 
 */
    if (unlikely(__pyx_v_descr->fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_descr->fields, __pyx_v_childname); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyTuple_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_3)->tp_name), 0))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_fields, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":796
 *     for childname in descr.names:
 *         fields = descr.fields[childname]
 *         child, new_offset = fields             # <<<<<<<<<<<<<<
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 */
    if (likely(__pyx_v_fields != Py_None)) {
      PyObject* sequence = __pyx_v_fields;
      #if CYTHON_COMPILING_IN_CPYTHON
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        {__pyx_filename = __pyx_f[3]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      #endif
    } else {
      __Pyx_RaiseNoneNotIterableError(); {__pyx_filename = __pyx_f[3]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_dtype))))) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_child, ((PyArray_Descr *)__pyx_t_3));
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_new_offset, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":798
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_new_offset, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = ((((__pyx_v_end - __pyx_v_f) - ((int)__pyx_t_5)) < 15) != 0);
    if (__pyx_t_6) {

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":798
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    }

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '>') != 0);
    if (!__pyx_t_7) {
      goto __pyx_L8_next_or;
    } else {
    }
    __pyx_t_7 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_L8_next_or:;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":802
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *             raise ValueError(u"Non-native byte order not supported")
 *             # One could encode it in the format string and have Cython
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '<') != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_7 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    if (__pyx_t_6) {

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[3]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":813
 * 
 *         # Output padding bytes
 *         while offset[0] < new_offset:             # <<<<<<<<<<<<<<
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 */
    while (1) {
      __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_v_new_offset, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!__pyx_t_6) break;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":814
 *         # Output padding bytes
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte             # <<<<<<<<<<<<<<
 *             f += 1
 *             offset[0] += 1
 */
      (__pyx_v_f[0]) = 0x78;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":815
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte
 *             f += 1             # <<<<<<<<<<<<<<
 *             offset[0] += 1
 * 
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":816
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 *             offset[0] += 1             # <<<<<<<<<<<<<<
 * 
 *         offset[0] += child.itemsize
 */
      __pyx_t_8 = 0;
      (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + 1);
    }

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":818
 *             offset[0] += 1
 * 
 *         offset[0] += child.itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not PyDataType_HASFIELDS(child):
 */
    __pyx_t_8 = 0;
    (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + __pyx_v_child->elsize);

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":820
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
    __pyx_t_6 = ((!(PyDataType_HASFIELDS(__pyx_v_child) != 0)) != 0);
    if (__pyx_t_6) {

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":821
 * 
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num             # <<<<<<<<<<<<<<
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")
 */
      __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_child->type_num); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 821; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":822
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      __pyx_t_6 = (((__pyx_v_end - __pyx_v_f) < 5) != 0);
      if (__pyx_t_6) {

        /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        {__pyx_filename = __pyx_f[3]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

        /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":822
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":826
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_BYTE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 98;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":827
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"             # <<<<<<<<<<<<<<
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UBYTE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 66;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":828
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"             # <<<<<<<<<<<<<<
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_SHORT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x68;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":829
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"             # <<<<<<<<<<<<<<
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_USHORT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 72;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":830
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_INT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x69;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":831
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UINT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 73;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":832
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x6C;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":833
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 76;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":834
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGLONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x71;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":835
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONGLONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 81;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":836
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"             # <<<<<<<<<<<<<<
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_FLOAT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x66;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":837
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_DOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x64;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":838
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"             # <<<<<<<<<<<<<<
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x67;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":839
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf             # <<<<<<<<<<<<<<
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CFLOAT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x66;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":840
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd             # <<<<<<<<<<<<<<
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x64;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":841
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg             # <<<<<<<<<<<<<<
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CLONGDOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x67;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":842
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_OBJECT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 79;
        goto __pyx_L15;
      }

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":844
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *             f += 1
 *         else:
 */
      /*else*/ {
        __pyx_t_3 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        {__pyx_filename = __pyx_f[3]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L15:;

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":845
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *             f += 1             # <<<<<<<<<<<<<<
 *         else:
 *             # Cython ignores struct boundary information ("T{...}"),
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":820
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
      goto __pyx_L13;
    }

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":849
 *             # Cython ignores struct boundary information ("T{...}"),
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)             # <<<<<<<<<<<<<<
 *     return f
 * 
 */
    /*else*/ {
      __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_child, __pyx_v_f, __pyx_v_end, __pyx_v_offset); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_f = __pyx_t_9;
    }
    __pyx_L13:;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":850
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)
 *     return f             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_f;
  goto __pyx_L0;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":785
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("numpy._util_dtypestring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_childname);
  __Pyx_XDECREF(__pyx_v_new_offset);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  PyObject *__pyx_v_baseptr;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":968
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
  __pyx_t_1 = (__pyx_v_base == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":969
 *      cdef PyObject* baseptr
 *      if base is None:
 *          baseptr = NULL             # <<<<<<<<<<<<<<
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 */
    __pyx_v_baseptr = NULL;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":968
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
    goto __pyx_L3;
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":971
 *          baseptr = NULL
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!             # <<<<<<<<<<<<<<
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 */
  /*else*/ {
    Py_INCREF(__pyx_v_base);

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":972
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base             # <<<<<<<<<<<<<<
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr
 */
    __pyx_v_baseptr = ((PyObject *)__pyx_v_base);
  }
  __pyx_L3:;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":973
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)             # <<<<<<<<<<<<<<
 *      arr.base = baseptr
 * 
 */
  Py_XDECREF(__pyx_v_arr->base);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":974
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  __pyx_v_arr->base = __pyx_v_baseptr;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":977
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_arr->base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":978
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     else:
 *         return <object>arr.base
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;

    /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":977
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":980
 *         return None
 *     else:
 *         return <object>arr.base             # <<<<<<<<<<<<<<
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_arr->base));
    __pyx_r = ((PyObject *)__pyx_v_arr->base);
    goto __pyx_L0;
  }

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_5pycx4_5pycda_cda_signal __pyx_vtable_5pycx4_5pycda_cda_signal;

static PyObject *__pyx_tp_new_5pycx4_5pycda_cda_signal(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_cda_signal *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_cda_signal *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pycx4_5pycda_cda_signal;
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_cda_signal(PyObject *o) {
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5pycx4_5pycda_10cda_signal_1__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_cda_signal[] = {
  {"connect", (PyCFunction)__pyx_pw_5pycx4_5pycda_10cda_signal_3connect, METH_O, 0},
  {"disconnect", (PyCFunction)__pyx_pw_5pycx4_5pycda_10cda_signal_5disconnect, METH_O, 0},
  {"emit", (PyCFunction)__pyx_pw_5pycx4_5pycda_10cda_signal_7emit, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_cda_signal = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.cda_signal", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_cda_signal), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_cda_signal, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_cda_signal, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_cda_signal, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_cda_object __pyx_vtable_5pycx4_5pycda_cda_object;

static PyObject *__pyx_tp_new_5pycx4_5pycda_cda_object(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_cda_object *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_cda_object *)o);
  p->__pyx_vtab = __pyx_vtabptr_5pycx4_5pycda_cda_object;
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_cda_object(PyObject *o) {
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5pycx4_5pycda_10cda_object_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_cda_object[] = {
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_cda_object = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.cda_object", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_cda_object), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_cda_object, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_cda_object, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_10cda_object_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_cda_object, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_cda_context __pyx_vtable_5pycx4_5pycda_cda_context;

static PyObject *__pyx_tp_new_5pycx4_5pycda_cda_context(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_cda_context *p;
  PyObject *o = __pyx_tp_new_5pycx4_5pycda_cda_object(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_cda_context *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5pycx4_5pycda_cda_object*)__pyx_vtabptr_5pycx4_5pycda_cda_context;
  p->defpfx = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->serverCycle = ((struct __pyx_obj_5pycx4_5pycda_cda_signal *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_cda_context(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_cda_context *p = (struct __pyx_obj_5pycx4_5pycda_cda_context *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5pycx4_5pycda_11cda_context_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->defpfx);
  Py_CLEAR(p->serverCycle);
  #if CYTHON_COMPILING_IN_CPYTHON
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5pycx4_5pycda_cda_object(o);
}

static int __pyx_tp_traverse_5pycx4_5pycda_cda_context(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pycx4_5pycda_cda_context *p = (struct __pyx_obj_5pycx4_5pycda_cda_context *)o;
  e = ((likely(__pyx_ptype_5pycx4_5pycda_cda_object)) ? ((__pyx_ptype_5pycx4_5pycda_cda_object->tp_traverse) ? __pyx_ptype_5pycx4_5pycda_cda_object->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_5pycx4_5pycda_cda_context)); if (e) return e;
  if (p->serverCycle) {
    e = (*v)(((PyObject*)p->serverCycle), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pycx4_5pycda_cda_context(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pycx4_5pycda_cda_context *p = (struct __pyx_obj_5pycx4_5pycda_cda_context *)o;
  if (likely(__pyx_ptype_5pycx4_5pycda_cda_object)) { if (__pyx_ptype_5pycx4_5pycda_cda_object->tp_clear) __pyx_ptype_5pycx4_5pycda_cda_object->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_5pycx4_5pycda_cda_context);
  tmp = ((PyObject*)p->serverCycle);
  p->serverCycle = ((struct __pyx_obj_5pycx4_5pycda_cda_signal *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5pycx4_5pycda_11cda_context_cid(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_11cda_context_3cid_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_11cda_context_defpfx(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_11cda_context_6defpfx_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_11cda_context_serverCycle(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_11cda_context_11serverCycle_1__get__(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_cda_context[] = {
  {"enable_serverCycle", (PyCFunction)__pyx_pw_5pycx4_5pycda_11cda_context_7enable_serverCycle, METH_NOARGS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_cda_context[] = {
  {(char *)"cid", __pyx_getprop_5pycx4_5pycda_11cda_context_cid, 0, 0, 0},
  {(char *)"defpfx", __pyx_getprop_5pycx4_5pycda_11cda_context_defpfx, 0, 0, 0},
  {(char *)"serverCycle", __pyx_getprop_5pycx4_5pycda_11cda_context_serverCycle, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_cda_context = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.cda_context", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_cda_context), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_cda_context, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_5pycx4_5pycda_11cda_context_5__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_cda_context, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_cda_context, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_cda_context, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_cda_context, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_11cda_context_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_cda_context, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_cda_base_chan __pyx_vtable_5pycx4_5pycda_cda_base_chan;

static PyObject *__pyx_tp_new_5pycx4_5pycda_cda_base_chan(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_cda_base_chan *p;
  PyObject *o = __pyx_tp_new_5pycx4_5pycda_cda_object(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5pycx4_5pycda_cda_object*)__pyx_vtabptr_5pycx4_5pycda_cda_base_chan;
  p->name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->valueMeasured = ((struct __pyx_obj_5pycx4_5pycda_cda_signal *)Py_None); Py_INCREF(Py_None);
  p->valueChanged = ((struct __pyx_obj_5pycx4_5pycda_cda_signal *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_cda_base_chan(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_cda_base_chan *p = (struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5pycx4_5pycda_13cda_base_chan_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->name);
  Py_CLEAR(p->valueMeasured);
  Py_CLEAR(p->valueChanged);
  #if CYTHON_COMPILING_IN_CPYTHON
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5pycx4_5pycda_cda_object(o);
}

static int __pyx_tp_traverse_5pycx4_5pycda_cda_base_chan(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pycx4_5pycda_cda_base_chan *p = (struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)o;
  e = ((likely(__pyx_ptype_5pycx4_5pycda_cda_object)) ? ((__pyx_ptype_5pycx4_5pycda_cda_object->tp_traverse) ? __pyx_ptype_5pycx4_5pycda_cda_object->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_5pycx4_5pycda_cda_base_chan)); if (e) return e;
  if (p->valueMeasured) {
    e = (*v)(((PyObject*)p->valueMeasured), a); if (e) return e;
  }
  if (p->valueChanged) {
    e = (*v)(((PyObject*)p->valueChanged), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pycx4_5pycda_cda_base_chan(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pycx4_5pycda_cda_base_chan *p = (struct __pyx_obj_5pycx4_5pycda_cda_base_chan *)o;
  if (likely(__pyx_ptype_5pycx4_5pycda_cda_object)) { if (__pyx_ptype_5pycx4_5pycda_cda_object->tp_clear) __pyx_ptype_5pycx4_5pycda_cda_object->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_5pycx4_5pycda_cda_base_chan);
  tmp = ((PyObject*)p->valueMeasured);
  p->valueMeasured = ((struct __pyx_obj_5pycx4_5pycda_cda_signal *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->valueChanged);
  p->valueChanged = ((struct __pyx_obj_5pycx4_5pycda_cda_signal *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5pycx4_5pycda_13cda_base_chan_ref(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_13cda_base_chan_3ref_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_13cda_base_chan_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_13cda_base_chan_4name_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_13cda_base_chan_max_nelems(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_13cda_base_chan_10max_nelems_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_13cda_base_chan_dtype(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_13cda_base_chan_5dtype_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_13cda_base_chan_itemsize(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_13cda_base_chan_8itemsize_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_13cda_base_chan_time(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_13cda_base_chan_4time_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_13cda_base_chan_prev_time(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_13cda_base_chan_9prev_time_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_13cda_base_chan_found(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_13cda_base_chan_5found_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_13cda_base_chan_valueMeasured(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_13cda_base_chan_13valueMeasured_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_13cda_base_chan_valueChanged(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_13cda_base_chan_12valueChanged_1__get__(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_cda_base_chan[] = {
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_cda_base_chan[] = {
  {(char *)"ref", __pyx_getprop_5pycx4_5pycda_13cda_base_chan_ref, 0, 0, 0},
  {(char *)"name", __pyx_getprop_5pycx4_5pycda_13cda_base_chan_name, 0, 0, 0},
  {(char *)"max_nelems", __pyx_getprop_5pycx4_5pycda_13cda_base_chan_max_nelems, 0, 0, 0},
  {(char *)"dtype", __pyx_getprop_5pycx4_5pycda_13cda_base_chan_dtype, 0, 0, 0},
  {(char *)"itemsize", __pyx_getprop_5pycx4_5pycda_13cda_base_chan_itemsize, 0, 0, 0},
  {(char *)"time", __pyx_getprop_5pycx4_5pycda_13cda_base_chan_time, 0, 0, 0},
  {(char *)"prev_time", __pyx_getprop_5pycx4_5pycda_13cda_base_chan_prev_time, 0, 0, 0},
  {(char *)"found", __pyx_getprop_5pycx4_5pycda_13cda_base_chan_found, 0, 0, 0},
  {(char *)"valueMeasured", __pyx_getprop_5pycx4_5pycda_13cda_base_chan_valueMeasured, 0, 0, 0},
  {(char *)"valueChanged", __pyx_getprop_5pycx4_5pycda_13cda_base_chan_valueChanged, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_cda_base_chan = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.cda_base_chan", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_cda_base_chan), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_cda_base_chan, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_5pycx4_5pycda_13cda_base_chan_5__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_cda_base_chan, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_cda_base_chan, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_cda_base_chan, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_cda_base_chan, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_13cda_base_chan_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_cda_base_chan, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_sdchan __pyx_vtable_5pycx4_5pycda_sdchan;

static PyObject *__pyx_tp_new_5pycx4_5pycda_sdchan(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_sdchan *p;
  PyObject *o = __pyx_tp_new_5pycx4_5pycda_cda_base_chan(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_sdchan *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5pycx4_5pycda_cda_object*)__pyx_vtabptr_5pycx4_5pycda_sdchan;
  return o;
}

static PyObject *__pyx_getprop_5pycx4_5pycda_6sdchan_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_6sdchan_3val_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_6sdchan_prev_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_6sdchan_8prev_val_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_6sdchan_tolerance(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_6sdchan_9tolerance_1__get__(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_sdchan[] = {
  {"setValue", (PyCFunction)__pyx_pw_5pycx4_5pycda_6sdchan_3setValue, METH_O, 0},
  {"setTolerance", (PyCFunction)__pyx_pw_5pycx4_5pycda_6sdchan_5setTolerance, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_sdchan[] = {
  {(char *)"val", __pyx_getprop_5pycx4_5pycda_6sdchan_val, 0, 0, 0},
  {(char *)"prev_val", __pyx_getprop_5pycx4_5pycda_6sdchan_prev_val, 0, 0, 0},
  {(char *)"tolerance", __pyx_getprop_5pycx4_5pycda_6sdchan_tolerance, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_sdchan = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.sdchan", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_sdchan), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_cda_base_chan, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5pycx4_5pycda_13cda_base_chan_5__str__, /*tp_str*/
  #else
  0, /*tp_str*/
  #endif
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_cda_base_chan, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_cda_base_chan, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_sdchan, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_sdchan, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_6sdchan_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_sdchan, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_strchan __pyx_vtable_5pycx4_5pycda_strchan;

static PyObject *__pyx_tp_new_5pycx4_5pycda_strchan(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_strchan *p;
  PyObject *o = __pyx_tp_new_5pycx4_5pycda_cda_base_chan(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_strchan *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5pycx4_5pycda_cda_object*)__pyx_vtabptr_5pycx4_5pycda_strchan;
  p->val = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_strchan(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_strchan *p = (struct __pyx_obj_5pycx4_5pycda_strchan *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5pycx4_5pycda_7strchan_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->val);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5pycx4_5pycda_cda_base_chan(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_7strchan_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_7strchan_3val_1__get__(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_strchan[] = {
  {"setValue", (PyCFunction)__pyx_pw_5pycx4_5pycda_7strchan_5setValue, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_strchan[] = {
  {(char *)"val", __pyx_getprop_5pycx4_5pycda_7strchan_val, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_strchan = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.strchan", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_strchan), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_strchan, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5pycx4_5pycda_13cda_base_chan_5__str__, /*tp_str*/
  #else
  0, /*tp_str*/
  #endif
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_cda_base_chan, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_cda_base_chan, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_strchan, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_strchan, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_7strchan_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_strchan, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_schan __pyx_vtable_5pycx4_5pycda_schan;

static PyObject *__pyx_tp_new_5pycx4_5pycda_schan(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_schan *p;
  PyObject *o = __pyx_tp_new_5pycx4_5pycda_cda_base_chan(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_schan *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5pycx4_5pycda_cda_object*)__pyx_vtabptr_5pycx4_5pycda_schan;
  p->val = Py_None; Py_INCREF(Py_None);
  p->prev_val = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_schan(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_schan *p = (struct __pyx_obj_5pycx4_5pycda_schan *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->val);
  Py_CLEAR(p->prev_val);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5pycx4_5pycda_cda_base_chan(o);
}

static int __pyx_tp_traverse_5pycx4_5pycda_schan(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pycx4_5pycda_schan *p = (struct __pyx_obj_5pycx4_5pycda_schan *)o;
  e = __pyx_tp_traverse_5pycx4_5pycda_cda_base_chan(o, v, a); if (e) return e;
  if (p->val) {
    e = (*v)(p->val, a); if (e) return e;
  }
  if (p->prev_val) {
    e = (*v)(p->prev_val, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pycx4_5pycda_schan(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pycx4_5pycda_schan *p = (struct __pyx_obj_5pycx4_5pycda_schan *)o;
  __pyx_tp_clear_5pycx4_5pycda_cda_base_chan(o);
  tmp = ((PyObject*)p->val);
  p->val = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->prev_val);
  p->prev_val = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5schan_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5schan_3val_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5schan_prev_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5schan_8prev_val_1__get__(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_schan[] = {
  {"setValue", (PyCFunction)__pyx_pw_5pycx4_5pycda_5schan_3setValue, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_schan[] = {
  {(char *)"val", __pyx_getprop_5pycx4_5pycda_5schan_val, 0, 0, 0},
  {(char *)"prev_val", __pyx_getprop_5pycx4_5pycda_5schan_prev_val, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_schan = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.schan", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_schan), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_schan, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5pycx4_5pycda_13cda_base_chan_5__str__, /*tp_str*/
  #else
  0, /*tp_str*/
  #endif
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_schan, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_schan, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_schan, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_schan, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_5schan_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_schan, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5pycx4_5pycda_vchan __pyx_vtable_5pycx4_5pycda_vchan;

static PyObject *__pyx_tp_new_5pycx4_5pycda_vchan(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5pycx4_5pycda_vchan *p;
  PyObject *o = __pyx_tp_new_5pycx4_5pycda_cda_base_chan(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5pycx4_5pycda_vchan *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5pycx4_5pycda_cda_object*)__pyx_vtabptr_5pycx4_5pycda_vchan;
  p->val = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->npdtype = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5pycx4_5pycda_vchan(PyObject *o) {
  struct __pyx_obj_5pycx4_5pycda_vchan *p = (struct __pyx_obj_5pycx4_5pycda_vchan *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->val);
  Py_CLEAR(p->npdtype);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5pycx4_5pycda_cda_base_chan(o);
}

static int __pyx_tp_traverse_5pycx4_5pycda_vchan(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5pycx4_5pycda_vchan *p = (struct __pyx_obj_5pycx4_5pycda_vchan *)o;
  e = __pyx_tp_traverse_5pycx4_5pycda_cda_base_chan(o, v, a); if (e) return e;
  if (p->val) {
    e = (*v)(((PyObject*)p->val), a); if (e) return e;
  }
  if (p->npdtype) {
    e = (*v)(p->npdtype, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5pycx4_5pycda_vchan(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5pycx4_5pycda_vchan *p = (struct __pyx_obj_5pycx4_5pycda_vchan *)o;
  __pyx_tp_clear_5pycx4_5pycda_cda_base_chan(o);
  tmp = ((PyObject*)p->val);
  p->val = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->npdtype);
  p->npdtype = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5vchan_val(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5vchan_3val_1__get__(o);
}

static PyObject *__pyx_getprop_5pycx4_5pycda_5vchan_npdtype(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5pycx4_5pycda_5vchan_7npdtype_1__get__(o);
}

static PyMethodDef __pyx_methods_5pycx4_5pycda_vchan[] = {
  {"setValue", (PyCFunction)__pyx_pw_5pycx4_5pycda_5vchan_3setValue, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5pycx4_5pycda_vchan[] = {
  {(char *)"val", __pyx_getprop_5pycx4_5pycda_5vchan_val, 0, 0, 0},
  {(char *)"npdtype", __pyx_getprop_5pycx4_5pycda_5vchan_npdtype, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5pycx4_5pycda_vchan = {
  PyVarObject_HEAD_INIT(0, 0)
  "pycx4.pycda.vchan", /*tp_name*/
  sizeof(struct __pyx_obj_5pycx4_5pycda_vchan), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5pycx4_5pycda_vchan, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_5pycx4_5pycda_13cda_base_chan_5__str__, /*tp_str*/
  #else
  0, /*tp_str*/
  #endif
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5pycx4_5pycda_vchan, /*tp_traverse*/
  __pyx_tp_clear_5pycx4_5pycda_vchan, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5pycx4_5pycda_vchan, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5pycx4_5pycda_vchan, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5pycx4_5pycda_5vchan_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5pycx4_5pycda_vchan, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "pycda",
    0, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_A_function_was_expected, __pyx_k_A_function_was_expected, sizeof(__pyx_k_A_function_was_expected), 0, 0, 1, 0},
  {&__pyx_n_s_C, __pyx_k_C, sizeof(__pyx_k_C), 0, 0, 1, 1},
  {&__pyx_n_s_Exception, __pyx_k_Exception, sizeof(__pyx_k_Exception), 0, 0, 1, 1},
  {&__pyx_kp_u_Format_string_allocated_too_shor, __pyx_k_Format_string_allocated_too_shor, sizeof(__pyx_k_Format_string_allocated_too_shor), 0, 1, 0, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor_2, __pyx_k_Format_string_allocated_too_shor_2, sizeof(__pyx_k_Format_string_allocated_too_shor_2), 0, 1, 0, 0},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_kp_u_Non_native_byte_order_not_suppor, __pyx_k_Non_native_byte_order_not_suppor, sizeof(__pyx_k_Non_native_byte_order_not_suppor), 0, 1, 0, 0},
  {&__pyx_n_s_PY_CXDTYPE_DOUBLE, __pyx_k_PY_CXDTYPE_DOUBLE, sizeof(__pyx_k_PY_CXDTYPE_DOUBLE), 0, 0, 1, 1},
  {&__pyx_n_s_PY_CXDTYPE_INT16, __pyx_k_PY_CXDTYPE_INT16, sizeof(__pyx_k_PY_CXDTYPE_INT16), 0, 0, 1, 1},
  {&__pyx_n_s_PY_CXDTYPE_INT32, __pyx_k_PY_CXDTYPE_INT32, sizeof(__pyx_k_PY_CXDTYPE_INT32), 0, 0, 1, 1},
  {&__pyx_n_s_PY_CXDTYPE_INT64, __pyx_k_PY_CXDTYPE_INT64, sizeof(__pyx_k_PY_CXDTYPE_INT64), 0, 0, 1, 1},
  {&__pyx_n_s_PY_CXDTYPE_INT8, __pyx_k_PY_CXDTYPE_INT8, sizeof(__pyx_k_PY_CXDTYPE_INT8), 0, 0, 1, 1},
  {&__pyx_n_s_PY_CXDTYPE_SINGLE, __pyx_k_PY_CXDTYPE_SINGLE, sizeof(__pyx_k_PY_CXDTYPE_SINGLE), 0, 0, 1, 1},
  {&__pyx_n_s_PY_CXDTYPE_TEXT, __pyx_k_PY_CXDTYPE_TEXT, sizeof(__pyx_k_PY_CXDTYPE_TEXT), 0, 0, 1, 1},
  {&__pyx_n_s_PY_CXDTYPE_UCTEXT, __pyx_k_PY_CXDTYPE_UCTEXT, sizeof(__pyx_k_PY_CXDTYPE_UCTEXT), 0, 0, 1, 1},
  {&__pyx_n_s_PY_CXDTYPE_UINT16, __pyx_k_PY_CXDTYPE_UINT16, sizeof(__pyx_k_PY_CXDTYPE_UINT16), 0, 0, 1, 1},
  {&__pyx_n_s_PY_CXDTYPE_UINT32, __pyx_k_PY_CXDTYPE_UINT32, sizeof(__pyx_k_PY_CXDTYPE_UINT32), 0, 0, 1, 1},
  {&__pyx_n_s_PY_CXDTYPE_UINT64, __pyx_k_PY_CXDTYPE_UINT64, sizeof(__pyx_k_PY_CXDTYPE_UINT64), 0, 0, 1, 1},
  {&__pyx_n_s_PY_CXDTYPE_UINT8, __pyx_k_PY_CXDTYPE_UINT8, sizeof(__pyx_k_PY_CXDTYPE_UINT8), 0, 0, 1, 1},
  {&__pyx_n_s_PY_CXDTYPE_UNKNOWN, __pyx_k_PY_CXDTYPE_UNKNOWN, sizeof(__pyx_k_PY_CXDTYPE_UNKNOWN), 0, 0, 1, 1},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_ascii, __pyx_k_ascii, sizeof(__pyx_k_ascii), 0, 0, 1, 1},
  {&__pyx_kp_s_cda_channel_ref_d_name_s, __pyx_k_cda_channel_ref_d_name_s, sizeof(__pyx_k_cda_channel_ref_d_name_s), 0, 0, 1, 0},
  {&__pyx_kp_s_cda_context_cid_d_defpfx_s_chan, __pyx_k_cda_context_cid_d_defpfx_s_chan, sizeof(__pyx_k_cda_context_cid_d_defpfx_s_chan), 0, 0, 1, 0},
  {&__pyx_kp_s_cda_error_s_errcode_s, __pyx_k_cda_error_s_errcode_s, sizeof(__pyx_k_cda_error_s_errcode_s), 0, 0, 1, 0},
  {&__pyx_n_s_connect, __pyx_k_connect, sizeof(__pyx_k_connect), 0, 0, 1, 1},
  {&__pyx_n_s_context, __pyx_k_context, sizeof(__pyx_k_context), 0, 0, 1, 1},
  {&__pyx_kp_s_cx, __pyx_k_cx, sizeof(__pyx_k_cx), 0, 0, 1, 0},
  {&__pyx_n_s_defpfx, __pyx_k_defpfx, sizeof(__pyx_k_defpfx), 0, 0, 1, 1},
  {&__pyx_n_s_disconnect, __pyx_k_disconnect, sizeof(__pyx_k_disconnect), 0, 0, 1, 1},
  {&__pyx_n_s_double, __pyx_k_double, sizeof(__pyx_k_double), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_emit, __pyx_k_emit, sizeof(__pyx_k_emit), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_kp_s_home_femanov_python_pycx4_pycx4, __pyx_k_home_femanov_python_pycx4_pycx4, sizeof(__pyx_k_home_femanov_python_pycx4_pycx4), 0, 0, 1, 0},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_int16, __pyx_k_int16, sizeof(__pyx_k_int16), 0, 0, 1, 1},
  {&__pyx_n_s_int32, __pyx_k_int32, sizeof(__pyx_k_int32), 0, 0, 1, 1},
  {&__pyx_n_s_int64, __pyx_k_int64, sizeof(__pyx_k_int64), 0, 0, 1, 1},
  {&__pyx_n_s_int8, __pyx_k_int8, sizeof(__pyx_k_int8), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_max_nelems, __pyx_k_max_nelems, sizeof(__pyx_k_max_nelems), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_kp_u_ndarray_is_not_C_contiguous, __pyx_k_ndarray_is_not_C_contiguous, sizeof(__pyx_k_ndarray_is_not_C_contiguous), 0, 1, 0, 0},
  {&__pyx_kp_u_ndarray_is_not_Fortran_contiguou, __pyx_k_ndarray_is_not_Fortran_contiguou, sizeof(__pyx_k_ndarray_is_not_Fortran_contiguou), 0, 1, 0, 0},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_n_s_order, __pyx_k_order, sizeof(__pyx_k_order), 0, 0, 1, 1},
  {&__pyx_n_s_py_sl_break, __pyx_k_py_sl_break, sizeof(__pyx_k_py_sl_break), 0, 0, 1, 1},
  {&__pyx_n_s_py_sl_main_loop, __pyx_k_py_sl_main_loop, sizeof(__pyx_k_py_sl_main_loop), 0, 0, 1, 1},
  {&__pyx_n_s_pycx4_pycda, __pyx_k_pycx4_pycda, sizeof(__pyx_k_pycx4_pycda), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_setTolerance, __pyx_k_setTolerance, sizeof(__pyx_k_setTolerance), 0, 0, 1, 1},
  {&__pyx_n_s_setValue, __pyx_k_setValue, sizeof(__pyx_k_setValue), 0, 0, 1, 1},
  {&__pyx_n_s_single, __pyx_k_single, sizeof(__pyx_k_single), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_uint16, __pyx_k_uint16, sizeof(__pyx_k_uint16), 0, 0, 1, 1},
  {&__pyx_n_s_uint32, __pyx_k_uint32, sizeof(__pyx_k_uint32), 0, 0, 1, 1},
  {&__pyx_n_s_uint64, __pyx_k_uint64, sizeof(__pyx_k_uint64), 0, 0, 1, 1},
  {&__pyx_n_s_uint8, __pyx_k_uint8, sizeof(__pyx_k_uint8), 0, 0, 1, 1},
  {&__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_k_unknown_dtype_code_in_numpy_pxd, sizeof(__pyx_k_unknown_dtype_code_in_numpy_pxd), 0, 1, 0, 0},
  {&__pyx_kp_s_value_size_greater_than_channel, __pyx_k_value_size_greater_than_channel, sizeof(__pyx_k_value_size_greater_than_channel), 0, 0, 1, 0},
  {&__pyx_n_s_zeros, __pyx_k_zeros, sizeof(__pyx_k_zeros), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_Exception = __Pyx_GetBuiltinName(__pyx_n_s_Exception); if (!__pyx_builtin_Exception) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "pycx4/cda_signal.pxi":20
 *             void *tmp
 *             int ind
 *         if not callable(callback): raise Exception('A function was expected')             # <<<<<<<<<<<<<<
 *         for ind in range(self.cnum):
 *             if callback == <object>self.callbacks[ind]:
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_A_function_was_expected); if (unlikely(!__pyx_tuple_)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "pycx4/pycdabase.pxi":160
 *             int ret
 *             char *c_defpfx
 *         ascii_pfx = defpfx.encode("ascii") # encode to ascii             # <<<<<<<<<<<<<<
 *         c_defpfx = ascii_pfx  # convert to char*
 * 
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_n_s_ascii); if (unlikely(!__pyx_tuple__2)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "pycx4/pycdabase.pxi":259
 *             self.valueMeasured = self.signaler.valueMeasured
 * 
 *         b_name = name.encode("ascii")             # <<<<<<<<<<<<<<
 *         cdef:
 *             char *c_name = b_name
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_n_s_ascii); if (unlikely(!__pyx_tuple__4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "pycx4/pycdauser.pxi":164
 *     cpdef setValue(self, np.ndarray value):
 *         if value.size > self.max_nelems:
 *             raise Exception('value size greater than channel.max_nelems')             # <<<<<<<<<<<<<<
 *         dtype = np2cxdtype(value.dtype)
 *         self.snd_data(dtype, value.size, <void*>value.data)
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_value_size_greater_than_channel); if (unlikely(!__pyx_tuple__7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_C_contiguous); if (unlikely(!__pyx_tuple__8)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_Fortran_contiguou); if (unlikely(!__pyx_tuple__9)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":259
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__10)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor); if (unlikely(!__pyx_tuple__11)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__12)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor_2); if (unlikely(!__pyx_tuple__13)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "pycx4/pycda.pyx":3
 * from cxscheduler cimport *
 * 
 * def py_sl_main_loop():             # <<<<<<<<<<<<<<
 *     sl_main_loop()
 * 
 */
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_femanov_python_pycx4_pycx4, __pyx_n_s_py_sl_main_loop, 3, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 3; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "pycx4/pycda.pyx":6
 *     sl_main_loop()
 * 
 * def py_sl_break():             # <<<<<<<<<<<<<<
 *     sl_break()
 * 
 */
  __pyx_codeobj__15 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_femanov_python_pycx4_pycx4, __pyx_n_s_py_sl_break, 6, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__15)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 6; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initpycda(void); /*proto*/
PyMODINIT_FUNC initpycda(void)
#else
PyMODINIT_FUNC PyInit_pycda(void); /*proto*/
PyMODINIT_FUNC PyInit_pycda(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_pycda(void)", 0);
  if (__Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("pycda", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  if (__pyx_module_is_main_pycx4__pycda) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "pycx4.pycda")) {
      if (unlikely(PyDict_SetItemString(modules, "pycx4.pycda", __pyx_m) < 0)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  __pyx_v_5pycx4_5pycda_default_context = ((struct __pyx_obj_5pycx4_5pycda_cda_context *)Py_None); Py_INCREF(Py_None);
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_vtabptr_5pycx4_5pycda_cda_signal = &__pyx_vtable_5pycx4_5pycda_cda_signal;
  __pyx_vtable_5pycx4_5pycda_cda_signal.connect = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_cda_signal *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_10cda_signal_connect;
  __pyx_vtable_5pycx4_5pycda_cda_signal.disconnect = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_cda_signal *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_10cda_signal_disconnect;
  __pyx_vtable_5pycx4_5pycda_cda_signal.emit = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_cda_signal *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_10cda_signal_emit;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_cda_signal) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5pycx4_5pycda_cda_signal.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_cda_signal.tp_dict, __pyx_vtabptr_5pycx4_5pycda_cda_signal) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "cda_signal", (PyObject *)&__pyx_type_5pycx4_5pycda_cda_signal) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5pycx4_5pycda_cda_signal = &__pyx_type_5pycx4_5pycda_cda_signal;
  __pyx_vtabptr_5pycx4_5pycda_cda_object = &__pyx_vtable_5pycx4_5pycda_cda_object;
  __pyx_vtable_5pycx4_5pycda_cda_object.add_event = (int (*)(struct __pyx_obj_5pycx4_5pycda_cda_object *, int, void *, void *, void *))__pyx_f_5pycx4_5pycda_10cda_object_add_event;
  __pyx_vtable_5pycx4_5pycda_cda_object.del_event = (int (*)(struct __pyx_obj_5pycx4_5pycda_cda_object *, int, void *, void *, void *))__pyx_f_5pycx4_5pycda_10cda_object_del_event;
  __pyx_vtable_5pycx4_5pycda_cda_object.search_event = (int (*)(struct __pyx_obj_5pycx4_5pycda_cda_object *, __pyx_t_5pycx4_5pycda_event *))__pyx_f_5pycx4_5pycda_10cda_object_search_event;
  __pyx_vtable_5pycx4_5pycda_cda_object.register_event = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_object *, __pyx_t_5pycx4_5pycda_event *))__pyx_f_5pycx4_5pycda_10cda_object_register_event;
  __pyx_vtable_5pycx4_5pycda_cda_object.unregister_event = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_object *, __pyx_t_5pycx4_5pycda_event *))__pyx_f_5pycx4_5pycda_10cda_object_unregister_event;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_cda_object) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 77; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5pycx4_5pycda_cda_object.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_cda_object.tp_dict, __pyx_vtabptr_5pycx4_5pycda_cda_object) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 77; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "cda_object", (PyObject *)&__pyx_type_5pycx4_5pycda_cda_object) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 77; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5pycx4_5pycda_cda_object = &__pyx_type_5pycx4_5pycda_cda_object;
  __pyx_vtabptr_5pycx4_5pycda_cda_context = &__pyx_vtable_5pycx4_5pycda_cda_context;
  __pyx_vtable_5pycx4_5pycda_cda_context.__pyx_base = *__pyx_vtabptr_5pycx4_5pycda_cda_object;
  __pyx_vtable_5pycx4_5pycda_cda_context.__pyx_base.register_event = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_object *, __pyx_t_5pycx4_5pycda_event *))__pyx_f_5pycx4_5pycda_11cda_context_register_event;
  __pyx_vtable_5pycx4_5pycda_cda_context.__pyx_base.unregister_event = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_object *, __pyx_t_5pycx4_5pycda_event *))__pyx_f_5pycx4_5pycda_11cda_context_unregister_event;
  __pyx_vtable_5pycx4_5pycda_cda_context.save_chan = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_context *, void *))__pyx_f_5pycx4_5pycda_11cda_context_save_chan;
  __pyx_vtable_5pycx4_5pycda_cda_context.drop_chan = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_context *, void *))__pyx_f_5pycx4_5pycda_11cda_context_drop_chan;
  __pyx_type_5pycx4_5pycda_cda_context.tp_base = __pyx_ptype_5pycx4_5pycda_cda_object;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_cda_context) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 139; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5pycx4_5pycda_cda_context.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_cda_context.tp_dict, __pyx_vtabptr_5pycx4_5pycda_cda_context) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 139; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "cda_context", (PyObject *)&__pyx_type_5pycx4_5pycda_cda_context) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 139; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5pycx4_5pycda_cda_context = &__pyx_type_5pycx4_5pycda_cda_context;
  __pyx_vtabptr_5pycx4_5pycda_cda_base_chan = &__pyx_vtable_5pycx4_5pycda_cda_base_chan;
  __pyx_vtable_5pycx4_5pycda_cda_base_chan.__pyx_base = *__pyx_vtabptr_5pycx4_5pycda_cda_object;
  __pyx_vtable_5pycx4_5pycda_cda_base_chan.__pyx_base.register_event = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_object *, __pyx_t_5pycx4_5pycda_event *))__pyx_f_5pycx4_5pycda_13cda_base_chan_register_event;
  __pyx_vtable_5pycx4_5pycda_cda_base_chan.__pyx_base.unregister_event = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_object *, __pyx_t_5pycx4_5pycda_event *))__pyx_f_5pycx4_5pycda_13cda_base_chan_unregister_event;
  __pyx_vtable_5pycx4_5pycda_cda_base_chan.cb = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *))__pyx_f_5pycx4_5pycda_13cda_base_chan_cb;
  __pyx_vtable_5pycx4_5pycda_cda_base_chan.snd_data = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *, cxdtype_t, int, void *))__pyx_f_5pycx4_5pycda_13cda_base_chan_snd_data;
  __pyx_vtable_5pycx4_5pycda_cda_base_chan.get_data = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *, size_t, size_t, void *))__pyx_f_5pycx4_5pycda_13cda_base_chan_get_data;
  __pyx_vtable_5pycx4_5pycda_cda_base_chan.current_nelems = (int (*)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *))__pyx_f_5pycx4_5pycda_13cda_base_chan_current_nelems;
  __pyx_vtable_5pycx4_5pycda_cda_base_chan.get_src = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *, char const **))__pyx_f_5pycx4_5pycda_13cda_base_chan_get_src;
  __pyx_type_5pycx4_5pycda_cda_base_chan.tp_base = __pyx_ptype_5pycx4_5pycda_cda_object;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_cda_base_chan) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 221; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5pycx4_5pycda_cda_base_chan.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_cda_base_chan.tp_dict, __pyx_vtabptr_5pycx4_5pycda_cda_base_chan) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 221; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "cda_base_chan", (PyObject *)&__pyx_type_5pycx4_5pycda_cda_base_chan) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 221; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5pycx4_5pycda_cda_base_chan = &__pyx_type_5pycx4_5pycda_cda_base_chan;
  __pyx_vtabptr_5pycx4_5pycda_sdchan = &__pyx_vtable_5pycx4_5pycda_sdchan;
  __pyx_vtable_5pycx4_5pycda_sdchan.__pyx_base = *__pyx_vtabptr_5pycx4_5pycda_cda_base_chan;
  __pyx_vtable_5pycx4_5pycda_sdchan.__pyx_base.cb = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *))__pyx_f_5pycx4_5pycda_6sdchan_cb;
  __pyx_vtable_5pycx4_5pycda_sdchan.setValue = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_sdchan *, double, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_6sdchan_setValue;
  __pyx_vtable_5pycx4_5pycda_sdchan.setTolerance = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_sdchan *, double, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_6sdchan_setTolerance;
  __pyx_type_5pycx4_5pycda_sdchan.tp_base = __pyx_ptype_5pycx4_5pycda_cda_base_chan;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_sdchan) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 6; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5pycx4_5pycda_sdchan.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_sdchan.tp_dict, __pyx_vtabptr_5pycx4_5pycda_sdchan) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 6; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "sdchan", (PyObject *)&__pyx_type_5pycx4_5pycda_sdchan) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 6; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5pycx4_5pycda_sdchan = &__pyx_type_5pycx4_5pycda_sdchan;
  __pyx_vtabptr_5pycx4_5pycda_strchan = &__pyx_vtable_5pycx4_5pycda_strchan;
  __pyx_vtable_5pycx4_5pycda_strchan.__pyx_base = *__pyx_vtabptr_5pycx4_5pycda_cda_base_chan;
  __pyx_vtable_5pycx4_5pycda_strchan.__pyx_base.cb = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *))__pyx_f_5pycx4_5pycda_7strchan_cb;
  __pyx_vtable_5pycx4_5pycda_strchan.setValue = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_strchan *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_7strchan_setValue;
  __pyx_type_5pycx4_5pycda_strchan.tp_base = __pyx_ptype_5pycx4_5pycda_cda_base_chan;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_strchan) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5pycx4_5pycda_strchan.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_strchan.tp_dict, __pyx_vtabptr_5pycx4_5pycda_strchan) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "strchan", (PyObject *)&__pyx_type_5pycx4_5pycda_strchan) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 30; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5pycx4_5pycda_strchan = &__pyx_type_5pycx4_5pycda_strchan;
  __pyx_vtabptr_5pycx4_5pycda_schan = &__pyx_vtable_5pycx4_5pycda_schan;
  __pyx_vtable_5pycx4_5pycda_schan.__pyx_base = *__pyx_vtabptr_5pycx4_5pycda_cda_base_chan;
  __pyx_vtable_5pycx4_5pycda_schan.__pyx_base.cb = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *))__pyx_f_5pycx4_5pycda_5schan_cb;
  __pyx_vtable_5pycx4_5pycda_schan.setValue = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_schan *, PyObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_5schan_setValue;
  __pyx_type_5pycx4_5pycda_schan.tp_base = __pyx_ptype_5pycx4_5pycda_cda_base_chan;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_schan) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5pycx4_5pycda_schan.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_schan.tp_dict, __pyx_vtabptr_5pycx4_5pycda_schan) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "schan", (PyObject *)&__pyx_type_5pycx4_5pycda_schan) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5pycx4_5pycda_schan = &__pyx_type_5pycx4_5pycda_schan;
  __pyx_vtabptr_5pycx4_5pycda_vchan = &__pyx_vtable_5pycx4_5pycda_vchan;
  __pyx_vtable_5pycx4_5pycda_vchan.__pyx_base = *__pyx_vtabptr_5pycx4_5pycda_cda_base_chan;
  __pyx_vtable_5pycx4_5pycda_vchan.__pyx_base.cb = (void (*)(struct __pyx_obj_5pycx4_5pycda_cda_base_chan *))__pyx_f_5pycx4_5pycda_5vchan_cb;
  __pyx_vtable_5pycx4_5pycda_vchan.setValue = (PyObject *(*)(struct __pyx_obj_5pycx4_5pycda_vchan *, PyArrayObject *, int __pyx_skip_dispatch))__pyx_f_5pycx4_5pycda_5vchan_setValue;
  __pyx_type_5pycx4_5pycda_vchan.tp_base = __pyx_ptype_5pycx4_5pycda_cda_base_chan;
  if (PyType_Ready(&__pyx_type_5pycx4_5pycda_vchan) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_5pycx4_5pycda_vchan.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5pycx4_5pycda_vchan.tp_dict, __pyx_vtabptr_5pycx4_5pycda_vchan) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "vchan", (PyObject *)&__pyx_type_5pycx4_5pycda_vchan) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5pycx4_5pycda_vchan = &__pyx_type_5pycx4_5pycda_vchan;
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) {__pyx_filename = __pyx_f[5]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType("numpy", "dtype", sizeof(PyArray_Descr), 0); if (unlikely(!__pyx_ptype_5numpy_dtype)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType("numpy", "flatiter", sizeof(PyArrayIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_flatiter)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType("numpy", "broadcast", sizeof(PyArrayMultiIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_broadcast)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_5numpy_ndarray)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType("numpy", "ufunc", sizeof(PyUFuncObject), 0); if (unlikely(!__pyx_ptype_5numpy_ufunc)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 861; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif

  /* "pycx4/pycda.pyx":3
 * from cxscheduler cimport *
 * 
 * def py_sl_main_loop():             # <<<<<<<<<<<<<<
 *     sl_main_loop()
 * 
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pycx4_5pycda_1py_sl_main_loop, NULL, __pyx_n_s_pycx4_pycda); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 3; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_py_sl_main_loop, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 3; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycda.pyx":6
 *     sl_main_loop()
 * 
 * def py_sl_break():             # <<<<<<<<<<<<<<
 *     sl_break()
 * 
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5pycx4_5pycda_3py_sl_break, NULL, __pyx_n_s_pycx4_pycda); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 6; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_py_sl_break, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 6; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":10
 *     include 'qt_signalers.pxi'
 * 
 * PY_CXDTYPE_UNKNOWN = CXDTYPE_UNKNOWN             # <<<<<<<<<<<<<<
 * PY_CXDTYPE_INT8 = CXDTYPE_INT8
 * PY_CXDTYPE_INT16 = CXDTYPE_INT16
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_UNKNOWN); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 10; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PY_CXDTYPE_UNKNOWN, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 10; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":11
 * 
 * PY_CXDTYPE_UNKNOWN = CXDTYPE_UNKNOWN
 * PY_CXDTYPE_INT8 = CXDTYPE_INT8             # <<<<<<<<<<<<<<
 * PY_CXDTYPE_INT16 = CXDTYPE_INT16
 * PY_CXDTYPE_INT32 = CXDTYPE_INT32
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_INT8); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 11; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PY_CXDTYPE_INT8, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 11; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":12
 * PY_CXDTYPE_UNKNOWN = CXDTYPE_UNKNOWN
 * PY_CXDTYPE_INT8 = CXDTYPE_INT8
 * PY_CXDTYPE_INT16 = CXDTYPE_INT16             # <<<<<<<<<<<<<<
 * PY_CXDTYPE_INT32 = CXDTYPE_INT32
 * PY_CXDTYPE_INT64 = CXDTYPE_INT64
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_INT16); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 12; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PY_CXDTYPE_INT16, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 12; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":13
 * PY_CXDTYPE_INT8 = CXDTYPE_INT8
 * PY_CXDTYPE_INT16 = CXDTYPE_INT16
 * PY_CXDTYPE_INT32 = CXDTYPE_INT32             # <<<<<<<<<<<<<<
 * PY_CXDTYPE_INT64 = CXDTYPE_INT64
 * PY_CXDTYPE_UINT8 = CXDTYPE_UINT8
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_INT32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 13; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PY_CXDTYPE_INT32, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 13; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":14
 * PY_CXDTYPE_INT16 = CXDTYPE_INT16
 * PY_CXDTYPE_INT32 = CXDTYPE_INT32
 * PY_CXDTYPE_INT64 = CXDTYPE_INT64             # <<<<<<<<<<<<<<
 * PY_CXDTYPE_UINT8 = CXDTYPE_UINT8
 * PY_CXDTYPE_UINT16 = CXDTYPE_UINT16
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_INT64); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 14; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PY_CXDTYPE_INT64, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 14; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":15
 * PY_CXDTYPE_INT32 = CXDTYPE_INT32
 * PY_CXDTYPE_INT64 = CXDTYPE_INT64
 * PY_CXDTYPE_UINT8 = CXDTYPE_UINT8             # <<<<<<<<<<<<<<
 * PY_CXDTYPE_UINT16 = CXDTYPE_UINT16
 * PY_CXDTYPE_UINT32 = CXDTYPE_UINT32
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_UINT8); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 15; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PY_CXDTYPE_UINT8, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 15; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":16
 * PY_CXDTYPE_INT64 = CXDTYPE_INT64
 * PY_CXDTYPE_UINT8 = CXDTYPE_UINT8
 * PY_CXDTYPE_UINT16 = CXDTYPE_UINT16             # <<<<<<<<<<<<<<
 * PY_CXDTYPE_UINT32 = CXDTYPE_UINT32
 * PY_CXDTYPE_UINT64 = CXDTYPE_UINT64
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_UINT16); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 16; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PY_CXDTYPE_UINT16, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 16; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":17
 * PY_CXDTYPE_UINT8 = CXDTYPE_UINT8
 * PY_CXDTYPE_UINT16 = CXDTYPE_UINT16
 * PY_CXDTYPE_UINT32 = CXDTYPE_UINT32             # <<<<<<<<<<<<<<
 * PY_CXDTYPE_UINT64 = CXDTYPE_UINT64
 * PY_CXDTYPE_SINGLE = CXDTYPE_SINGLE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_UINT32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 17; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PY_CXDTYPE_UINT32, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 17; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":18
 * PY_CXDTYPE_UINT16 = CXDTYPE_UINT16
 * PY_CXDTYPE_UINT32 = CXDTYPE_UINT32
 * PY_CXDTYPE_UINT64 = CXDTYPE_UINT64             # <<<<<<<<<<<<<<
 * PY_CXDTYPE_SINGLE = CXDTYPE_SINGLE
 * PY_CXDTYPE_DOUBLE = CXDTYPE_DOUBLE
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_UINT64); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 18; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PY_CXDTYPE_UINT64, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 18; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":19
 * PY_CXDTYPE_UINT32 = CXDTYPE_UINT32
 * PY_CXDTYPE_UINT64 = CXDTYPE_UINT64
 * PY_CXDTYPE_SINGLE = CXDTYPE_SINGLE             # <<<<<<<<<<<<<<
 * PY_CXDTYPE_DOUBLE = CXDTYPE_DOUBLE
 * PY_CXDTYPE_TEXT = CXDTYPE_TEXT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_SINGLE); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 19; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PY_CXDTYPE_SINGLE, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 19; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":20
 * PY_CXDTYPE_UINT64 = CXDTYPE_UINT64
 * PY_CXDTYPE_SINGLE = CXDTYPE_SINGLE
 * PY_CXDTYPE_DOUBLE = CXDTYPE_DOUBLE             # <<<<<<<<<<<<<<
 * PY_CXDTYPE_TEXT = CXDTYPE_TEXT
 * PY_CXDTYPE_UCTEXT = CXDTYPE_UCTEXT
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_DOUBLE); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PY_CXDTYPE_DOUBLE, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":21
 * PY_CXDTYPE_SINGLE = CXDTYPE_SINGLE
 * PY_CXDTYPE_DOUBLE = CXDTYPE_DOUBLE
 * PY_CXDTYPE_TEXT = CXDTYPE_TEXT             # <<<<<<<<<<<<<<
 * PY_CXDTYPE_UCTEXT = CXDTYPE_UCTEXT
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_TEXT); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PY_CXDTYPE_TEXT, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":22
 * PY_CXDTYPE_DOUBLE = CXDTYPE_DOUBLE
 * PY_CXDTYPE_TEXT = CXDTYPE_TEXT
 * PY_CXDTYPE_UCTEXT = CXDTYPE_UCTEXT             # <<<<<<<<<<<<<<
 * 
 * # struct to extend private pointer
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(CXDTYPE_UCTEXT); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 22; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_PY_CXDTYPE_UCTEXT, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 22; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":218
 * 
 * 
 * cdef cda_context default_context=cda_context()             # <<<<<<<<<<<<<<
 * 
 * # wrapper-class for low-level functions and channel registration
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5pycx4_5pycda_cda_context), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(((PyObject *)__pyx_v_5pycx4_5pycda_default_context));
  __Pyx_DECREF_SET(__pyx_v_5pycx4_5pycda_default_context, ((struct __pyx_obj_5pycx4_5pycda_cda_context *)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pycx4/pycdabase.pxi":246
 *             public object valueMeasured
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE, int max_nelems=1):             # <<<<<<<<<<<<<<
 *         cda_object.__init__(self)
 *         if isinstance(context, cda_context): self.context = <void*>context
 */
  __pyx_k__3 = CXDTYPE_DOUBLE;

  /* "pycx4/pycdauser.pxi":2
 * # user-level channel classes
 * import numpy as np             # <<<<<<<<<<<<<<
 * cimport numpy as np
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 2; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 2; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pycx4/pycdauser.pxi":127
 *         readonly object val, prev_val
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE):             # <<<<<<<<<<<<<<
 *         cda_base_chan.__init__(self, name, context, dtype)
 * 
 */
  __pyx_k__5 = CXDTYPE_DOUBLE;

  /* "pycx4/pycdauser.pxi":152
 *         readonly object npdtype
 * 
 *     def __init__(self, str name, object context=None, cxdtype_t dtype=CXDTYPE_DOUBLE, int max_nelems=1):             # <<<<<<<<<<<<<<
 *         cda_base_chan.__init__(self, name, context, dtype, max_nelems)
 *         self.npdtype = cxdtype2np(dtype)
 */
  __pyx_k__6 = CXDTYPE_DOUBLE;

  /* "pycx4/pycda.pyx":1
 * from cxscheduler cimport *             # <<<<<<<<<<<<<<
 * 
 * def py_sl_main_loop():
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[4]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../../usr/lib64/python3.3/site-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init pycx4.pycda", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pycx4.pycda");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#endif
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

static CYTHON_INLINE int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_CheckExact(key)) && unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static void __Pyx_RaiseArgumentTypeInvalid(const char* name, PyObject *obj, PyTypeObject *type) {
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
}
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (likely(Py_TYPE(obj) == type)) return 1;
        #if PY_MAJOR_VERSION == 2
        else if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(PyObject_TypeCheck(obj, type))) return 1;
    }
    __Pyx_RaiseArgumentTypeInvalid(name, obj, type);
    return 0;
}

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
#endif

static CYTHON_INLINE uint8 __Pyx_PyInt_As_uint8(PyObject *x) {
    const uint8 neg_one = (uint8) -1, const_zero = (uint8) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint8) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint8, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint8) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint8) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint8, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) >= 2 * PyLong_SHIFT) {
                            return (uint8) (((((uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) >= 3 * PyLong_SHIFT) {
                            return (uint8) (((((((uint8)digits[2]) << PyLong_SHIFT) | (uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) >= 4 * PyLong_SHIFT) {
                            return (uint8) (((((((((uint8)digits[3]) << PyLong_SHIFT) | (uint8)digits[2]) << PyLong_SHIFT) | (uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint8) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint8) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(uint8) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint8) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint8, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(uint8,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint8) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) - 1 > 2 * PyLong_SHIFT) {
                            return (uint8) (((uint8)-1)*(((((uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) - 1 > 2 * PyLong_SHIFT) {
                            return (uint8) ((((((uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint8) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) - 1 > 3 * PyLong_SHIFT) {
                            return (uint8) (((uint8)-1)*(((((((uint8)digits[2]) << PyLong_SHIFT) | (uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) - 1 > 3 * PyLong_SHIFT) {
                            return (uint8) ((((((((uint8)digits[2]) << PyLong_SHIFT) | (uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint8) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) - 1 > 4 * PyLong_SHIFT) {
                            return (uint8) (((uint8)-1)*(((((((((uint8)digits[3]) << PyLong_SHIFT) | (uint8)digits[2]) << PyLong_SHIFT) | (uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8) - 1 > 4 * PyLong_SHIFT) {
                            return (uint8) ((((((((((uint8)digits[3]) << PyLong_SHIFT) | (uint8)digits[2]) << PyLong_SHIFT) | (uint8)digits[1]) << PyLong_SHIFT) | (uint8)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint8) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8, long, PyLong_AsLong(x))
            } else if (sizeof(uint8) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint8 val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint8) -1;
        }
    } else {
        uint8 val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (uint8) -1;
        val = __Pyx_PyInt_As_uint8(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint8");
    return (uint8) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint8");
    return (uint8) -1;
}

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_cda_context_t(cda_context_t value) {
    const cda_context_t neg_one = (cda_context_t) -1, const_zero = (cda_context_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(cda_context_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(cda_context_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(cda_context_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(cda_context_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(cda_context_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(cda_context_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_CxDataRef_t(CxDataRef_t value) {
    const CxDataRef_t neg_one = (CxDataRef_t) -1, const_zero = (CxDataRef_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(CxDataRef_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(CxDataRef_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(CxDataRef_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(CxDataRef_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(CxDataRef_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(CxDataRef_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint8(uint8 value) {
    const uint8 neg_one = (uint8) -1, const_zero = (uint8) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint8) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint8) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(uint8) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(uint8) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint8) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint8),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int64(int64 value) {
    const int64 neg_one = (int64) -1, const_zero = (int64) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int64) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int64) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int64) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int64) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int64) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int64),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int32(int32 value) {
    const int32 neg_one = (int32) -1, const_zero = (int32) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int32) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int32) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int32) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int32) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int32) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int32),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int8(int8 value) {
    const int8 neg_one = (int8) -1, const_zero = (int8) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int8) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int8) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int8) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int8) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int8) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int8),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int16(int16 value) {
    const int16 neg_one = (int16) -1, const_zero = (int16) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int16) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int16) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int16) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int16) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int16) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int16),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint16(uint16 value) {
    const uint16 neg_one = (uint16) -1, const_zero = (uint16) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint16) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint16) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(uint16) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(uint16) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint16) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint16),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint32(uint32 value) {
    const uint32 neg_one = (uint32) -1, const_zero = (uint32) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint32) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint32) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(uint32) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(uint32) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint32) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint32),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint64(uint64 value) {
    const uint64 neg_one = (uint64) -1, const_zero = (uint64) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint64) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint64) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(uint64) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(uint64) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint64) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint64),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_char8(char8 value) {
    const char8 neg_one = (char8) -1, const_zero = (char8) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(char8) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(char8) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(char8) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(char8) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(char8) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(char8),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE int32 __Pyx_PyInt_As_int32(PyObject *x) {
    const int32 neg_one = (int32) -1, const_zero = (int32) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int32) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int32, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int32) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int32) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int32, digit, digits[0])
                case 2:
                    if (8 * sizeof(int32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) >= 2 * PyLong_SHIFT) {
                            return (int32) (((((int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) >= 3 * PyLong_SHIFT) {
                            return (int32) (((((((int32)digits[2]) << PyLong_SHIFT) | (int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) >= 4 * PyLong_SHIFT) {
                            return (int32) (((((((((int32)digits[3]) << PyLong_SHIFT) | (int32)digits[2]) << PyLong_SHIFT) | (int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int32) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int32) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int32) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int32) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int32, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(int32,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int32) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) - 1 > 2 * PyLong_SHIFT) {
                            return (int32) (((int32)-1)*(((((int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) - 1 > 2 * PyLong_SHIFT) {
                            return (int32) ((((((int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int32) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) - 1 > 3 * PyLong_SHIFT) {
                            return (int32) (((int32)-1)*(((((((int32)digits[2]) << PyLong_SHIFT) | (int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) - 1 > 3 * PyLong_SHIFT) {
                            return (int32) ((((((((int32)digits[2]) << PyLong_SHIFT) | (int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int32) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) - 1 > 4 * PyLong_SHIFT) {
                            return (int32) (((int32)-1)*(((((((((int32)digits[3]) << PyLong_SHIFT) | (int32)digits[2]) << PyLong_SHIFT) | (int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32) - 1 > 4 * PyLong_SHIFT) {
                            return (int32) ((((((((((int32)digits[3]) << PyLong_SHIFT) | (int32)digits[2]) << PyLong_SHIFT) | (int32)digits[1]) << PyLong_SHIFT) | (int32)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int32) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32, long, PyLong_AsLong(x))
            } else if (sizeof(int32) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int32 val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int32) -1;
        }
    } else {
        int32 val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int32) -1;
        val = __Pyx_PyInt_As_int32(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int32");
    return (int32) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int32");
    return (int32) -1;
}

static CYTHON_INLINE int8 __Pyx_PyInt_As_int8(PyObject *x) {
    const int8 neg_one = (int8) -1, const_zero = (int8) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int8) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int8, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int8) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int8) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int8, digit, digits[0])
                case 2:
                    if (8 * sizeof(int8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) >= 2 * PyLong_SHIFT) {
                            return (int8) (((((int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) >= 3 * PyLong_SHIFT) {
                            return (int8) (((((((int8)digits[2]) << PyLong_SHIFT) | (int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) >= 4 * PyLong_SHIFT) {
                            return (int8) (((((((((int8)digits[3]) << PyLong_SHIFT) | (int8)digits[2]) << PyLong_SHIFT) | (int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int8) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int8) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int8, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int8) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int8, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int8) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int8, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(int8,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int8) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) - 1 > 2 * PyLong_SHIFT) {
                            return (int8) (((int8)-1)*(((((int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) - 1 > 2 * PyLong_SHIFT) {
                            return (int8) ((((((int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int8) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) - 1 > 3 * PyLong_SHIFT) {
                            return (int8) (((int8)-1)*(((((((int8)digits[2]) << PyLong_SHIFT) | (int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) - 1 > 3 * PyLong_SHIFT) {
                            return (int8) ((((((((int8)digits[2]) << PyLong_SHIFT) | (int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int8) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) - 1 > 4 * PyLong_SHIFT) {
                            return (int8) (((int8)-1)*(((((((((int8)digits[3]) << PyLong_SHIFT) | (int8)digits[2]) << PyLong_SHIFT) | (int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int8) - 1 > 4 * PyLong_SHIFT) {
                            return (int8) ((((((((((int8)digits[3]) << PyLong_SHIFT) | (int8)digits[2]) << PyLong_SHIFT) | (int8)digits[1]) << PyLong_SHIFT) | (int8)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int8) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int8, long, PyLong_AsLong(x))
            } else if (sizeof(int8) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int8, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int8 val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int8) -1;
        }
    } else {
        int8 val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int8) -1;
        val = __Pyx_PyInt_As_int8(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int8");
    return (int8) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int8");
    return (int8) -1;
}

static CYTHON_INLINE int16 __Pyx_PyInt_As_int16(PyObject *x) {
    const int16 neg_one = (int16) -1, const_zero = (int16) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int16) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int16, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int16) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int16) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int16, digit, digits[0])
                case 2:
                    if (8 * sizeof(int16) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) >= 2 * PyLong_SHIFT) {
                            return (int16) (((((int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int16) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) >= 3 * PyLong_SHIFT) {
                            return (int16) (((((((int16)digits[2]) << PyLong_SHIFT) | (int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int16) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) >= 4 * PyLong_SHIFT) {
                            return (int16) (((((((((int16)digits[3]) << PyLong_SHIFT) | (int16)digits[2]) << PyLong_SHIFT) | (int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int16) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int16) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int16, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int16) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int16, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int16) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int16, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(int16,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int16) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) - 1 > 2 * PyLong_SHIFT) {
                            return (int16) (((int16)-1)*(((((int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int16) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) - 1 > 2 * PyLong_SHIFT) {
                            return (int16) ((((((int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int16) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) - 1 > 3 * PyLong_SHIFT) {
                            return (int16) (((int16)-1)*(((((((int16)digits[2]) << PyLong_SHIFT) | (int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int16) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) - 1 > 3 * PyLong_SHIFT) {
                            return (int16) ((((((((int16)digits[2]) << PyLong_SHIFT) | (int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int16) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) - 1 > 4 * PyLong_SHIFT) {
                            return (int16) (((int16)-1)*(((((((((int16)digits[3]) << PyLong_SHIFT) | (int16)digits[2]) << PyLong_SHIFT) | (int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int16) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int16, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int16) - 1 > 4 * PyLong_SHIFT) {
                            return (int16) ((((((((((int16)digits[3]) << PyLong_SHIFT) | (int16)digits[2]) << PyLong_SHIFT) | (int16)digits[1]) << PyLong_SHIFT) | (int16)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int16) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int16, long, PyLong_AsLong(x))
            } else if (sizeof(int16) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int16, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int16 val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int16) -1;
        }
    } else {
        int16 val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int16) -1;
        val = __Pyx_PyInt_As_int16(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int16");
    return (int16) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int16");
    return (int16) -1;
}

static CYTHON_INLINE int64 __Pyx_PyInt_As_int64(PyObject *x) {
    const int64 neg_one = (int64) -1, const_zero = (int64) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int64) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int64, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int64) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int64) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int64, digit, digits[0])
                case 2:
                    if (8 * sizeof(int64) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) >= 2 * PyLong_SHIFT) {
                            return (int64) (((((int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int64) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) >= 3 * PyLong_SHIFT) {
                            return (int64) (((((((int64)digits[2]) << PyLong_SHIFT) | (int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int64) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) >= 4 * PyLong_SHIFT) {
                            return (int64) (((((((((int64)digits[3]) << PyLong_SHIFT) | (int64)digits[2]) << PyLong_SHIFT) | (int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int64) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int64) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int64) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int64) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int64, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(int64,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int64) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) - 1 > 2 * PyLong_SHIFT) {
                            return (int64) (((int64)-1)*(((((int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int64) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) - 1 > 2 * PyLong_SHIFT) {
                            return (int64) ((((((int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int64) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) - 1 > 3 * PyLong_SHIFT) {
                            return (int64) (((int64)-1)*(((((((int64)digits[2]) << PyLong_SHIFT) | (int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int64) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) - 1 > 3 * PyLong_SHIFT) {
                            return (int64) ((((((((int64)digits[2]) << PyLong_SHIFT) | (int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int64) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) - 1 > 4 * PyLong_SHIFT) {
                            return (int64) (((int64)-1)*(((((((((int64)digits[3]) << PyLong_SHIFT) | (int64)digits[2]) << PyLong_SHIFT) | (int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int64) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64) - 1 > 4 * PyLong_SHIFT) {
                            return (int64) ((((((((((int64)digits[3]) << PyLong_SHIFT) | (int64)digits[2]) << PyLong_SHIFT) | (int64)digits[1]) << PyLong_SHIFT) | (int64)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int64) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64, long, PyLong_AsLong(x))
            } else if (sizeof(int64) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int64 val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int64) -1;
        }
    } else {
        int64 val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int64) -1;
        val = __Pyx_PyInt_As_int64(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int64");
    return (int64) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int64");
    return (int64) -1;
}

static CYTHON_INLINE uint16 __Pyx_PyInt_As_uint16(PyObject *x) {
    const uint16 neg_one = (uint16) -1, const_zero = (uint16) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint16) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint16, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint16) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint16) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint16, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint16) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) >= 2 * PyLong_SHIFT) {
                            return (uint16) (((((uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint16) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) >= 3 * PyLong_SHIFT) {
                            return (uint16) (((((((uint16)digits[2]) << PyLong_SHIFT) | (uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint16) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) >= 4 * PyLong_SHIFT) {
                            return (uint16) (((((((((uint16)digits[3]) << PyLong_SHIFT) | (uint16)digits[2]) << PyLong_SHIFT) | (uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint16) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint16) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(uint16) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint16) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint16, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(uint16,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint16) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) - 1 > 2 * PyLong_SHIFT) {
                            return (uint16) (((uint16)-1)*(((((uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint16) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) - 1 > 2 * PyLong_SHIFT) {
                            return (uint16) ((((((uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint16) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) - 1 > 3 * PyLong_SHIFT) {
                            return (uint16) (((uint16)-1)*(((((((uint16)digits[2]) << PyLong_SHIFT) | (uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint16) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) - 1 > 3 * PyLong_SHIFT) {
                            return (uint16) ((((((((uint16)digits[2]) << PyLong_SHIFT) | (uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint16) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) - 1 > 4 * PyLong_SHIFT) {
                            return (uint16) (((uint16)-1)*(((((((((uint16)digits[3]) << PyLong_SHIFT) | (uint16)digits[2]) << PyLong_SHIFT) | (uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint16) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint16, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint16) - 1 > 4 * PyLong_SHIFT) {
                            return (uint16) ((((((((((uint16)digits[3]) << PyLong_SHIFT) | (uint16)digits[2]) << PyLong_SHIFT) | (uint16)digits[1]) << PyLong_SHIFT) | (uint16)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint16) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16, long, PyLong_AsLong(x))
            } else if (sizeof(uint16) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint16, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint16 val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint16) -1;
        }
    } else {
        uint16 val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (uint16) -1;
        val = __Pyx_PyInt_As_uint16(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint16");
    return (uint16) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint16");
    return (uint16) -1;
}

static CYTHON_INLINE uint32 __Pyx_PyInt_As_uint32(PyObject *x) {
    const uint32 neg_one = (uint32) -1, const_zero = (uint32) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint32) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint32, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint32) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint32, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) >= 2 * PyLong_SHIFT) {
                            return (uint32) (((((uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) >= 3 * PyLong_SHIFT) {
                            return (uint32) (((((((uint32)digits[2]) << PyLong_SHIFT) | (uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) >= 4 * PyLong_SHIFT) {
                            return (uint32) (((((((((uint32)digits[3]) << PyLong_SHIFT) | (uint32)digits[2]) << PyLong_SHIFT) | (uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint32) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint32) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(uint32) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint32, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(uint32,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint32) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32) (((uint32)-1)*(((((uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint32) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32) ((((((uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint32) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32) (((uint32)-1)*(((((((uint32)digits[2]) << PyLong_SHIFT) | (uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32) ((((((((uint32)digits[2]) << PyLong_SHIFT) | (uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint32) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32) (((uint32)-1)*(((((((((uint32)digits[3]) << PyLong_SHIFT) | (uint32)digits[2]) << PyLong_SHIFT) | (uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32) ((((((((((uint32)digits[3]) << PyLong_SHIFT) | (uint32)digits[2]) << PyLong_SHIFT) | (uint32)digits[1]) << PyLong_SHIFT) | (uint32)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint32) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32, long, PyLong_AsLong(x))
            } else if (sizeof(uint32) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint32 val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint32) -1;
        }
    } else {
        uint32 val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (uint32) -1;
        val = __Pyx_PyInt_As_uint32(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint32");
    return (uint32) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint32");
    return (uint32) -1;
}

static CYTHON_INLINE uint64 __Pyx_PyInt_As_uint64(PyObject *x) {
    const uint64 neg_one = (uint64) -1, const_zero = (uint64) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint64) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint64, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint64) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint64) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint64, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint64) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) >= 2 * PyLong_SHIFT) {
                            return (uint64) (((((uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint64) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) >= 3 * PyLong_SHIFT) {
                            return (uint64) (((((((uint64)digits[2]) << PyLong_SHIFT) | (uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint64) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) >= 4 * PyLong_SHIFT) {
                            return (uint64) (((((((((uint64)digits[3]) << PyLong_SHIFT) | (uint64)digits[2]) << PyLong_SHIFT) | (uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint64) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint64) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(uint64) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint64) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint64, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(uint64,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint64) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) - 1 > 2 * PyLong_SHIFT) {
                            return (uint64) (((uint64)-1)*(((((uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint64) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) - 1 > 2 * PyLong_SHIFT) {
                            return (uint64) ((((((uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint64) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) - 1 > 3 * PyLong_SHIFT) {
                            return (uint64) (((uint64)-1)*(((((((uint64)digits[2]) << PyLong_SHIFT) | (uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint64) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) - 1 > 3 * PyLong_SHIFT) {
                            return (uint64) ((((((((uint64)digits[2]) << PyLong_SHIFT) | (uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint64) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) - 1 > 4 * PyLong_SHIFT) {
                            return (uint64) (((uint64)-1)*(((((((((uint64)digits[3]) << PyLong_SHIFT) | (uint64)digits[2]) << PyLong_SHIFT) | (uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint64) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64) - 1 > 4 * PyLong_SHIFT) {
                            return (uint64) ((((((((((uint64)digits[3]) << PyLong_SHIFT) | (uint64)digits[2]) << PyLong_SHIFT) | (uint64)digits[1]) << PyLong_SHIFT) | (uint64)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint64) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64, long, PyLong_AsLong(x))
            } else if (sizeof(uint64) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint64 val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint64) -1;
        }
    } else {
        uint64 val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (uint64) -1;
        val = __Pyx_PyInt_As_uint64(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint64");
    return (uint64) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint64");
    return (uint64) -1;
}

static CYTHON_INLINE char8 __Pyx_PyInt_As_char8(PyObject *x) {
    const char8 neg_one = (char8) -1, const_zero = (char8) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(char8) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(char8, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (char8) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char8) 0;
                case  1: __PYX_VERIFY_RETURN_INT(char8, digit, digits[0])
                case 2:
                    if (8 * sizeof(char8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) >= 2 * PyLong_SHIFT) {
                            return (char8) (((((char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) >= 3 * PyLong_SHIFT) {
                            return (char8) (((((((char8)digits[2]) << PyLong_SHIFT) | (char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) >= 4 * PyLong_SHIFT) {
                            return (char8) (((((((((char8)digits[3]) << PyLong_SHIFT) | (char8)digits[2]) << PyLong_SHIFT) | (char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (char8) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(char8) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char8, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(char8) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char8, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char8) 0;
                case -1: __PYX_VERIFY_RETURN_INT(char8, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(char8,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(char8) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) - 1 > 2 * PyLong_SHIFT) {
                            return (char8) (((char8)-1)*(((((char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(char8) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) - 1 > 2 * PyLong_SHIFT) {
                            return (char8) ((((((char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(char8) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) - 1 > 3 * PyLong_SHIFT) {
                            return (char8) (((char8)-1)*(((((((char8)digits[2]) << PyLong_SHIFT) | (char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char8) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) - 1 > 3 * PyLong_SHIFT) {
                            return (char8) ((((((((char8)digits[2]) << PyLong_SHIFT) | (char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(char8) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) - 1 > 4 * PyLong_SHIFT) {
                            return (char8) (((char8)-1)*(((((((((char8)digits[3]) << PyLong_SHIFT) | (char8)digits[2]) << PyLong_SHIFT) | (char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char8) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char8, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char8) - 1 > 4 * PyLong_SHIFT) {
                            return (char8) ((((((((((char8)digits[3]) << PyLong_SHIFT) | (char8)digits[2]) << PyLong_SHIFT) | (char8)digits[1]) << PyLong_SHIFT) | (char8)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(char8) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char8, long, PyLong_AsLong(x))
            } else if (sizeof(char8) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char8, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            char8 val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (char8) -1;
        }
    } else {
        char8 val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (char8) -1;
        val = __Pyx_PyInt_As_char8(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to char8");
    return (char8) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to char8");
    return (char8) -1;
}

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        float denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(a, a);
                    case 3:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, a);
                    case 4:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_absf(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        double denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(a, a);
                    case 3:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, a);
                    case 4:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_abs(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value) {
    const enum NPY_TYPES neg_one = (enum NPY_TYPES) -1, const_zero = (enum NPY_TYPES) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum NPY_TYPES) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(enum NPY_TYPES) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum NPY_TYPES),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
  m = Py_TYPE(x)->tp_as_number;
#if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
